/* BearPaw1200CU panel

   Copyright (C) 2001 Mustek. 
   Maintained by Tom Wang and Alex Lin
   <tom.wang@mustek.com.tw, alex_lin@mustek.com.tw>

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston,
   MA 02111-1307, USA.
*/


//#include <stdlib.h>
#include "FRuler.h"

CFastRuler::CFastRuler()
{
	m_pRuler=NULL;
	m_MaxDestLength=0;
	m_Step=0;
	m_Bound=0;
}

CFastRuler::~CFastRuler()
{
}


void CFastRuler::FreeRuler()
{
	if (m_pRuler) {
		free(m_pRuler);
		m_pRuler = NULL;
	}
}

LONG CFastRuler::GetRange(LONG DestDpi,LONG SrcDpi,LONG DestLength)
{
	if (!MinDivable(DestDpi,SrcDpi))
		return 0;
	return (((DestLength-1)*SrcDpi)/(DestDpi))+2;
}

BOOL CFastRuler::MinDivable(LONG& Dest, LONG& Src)
{
	if (Dest<Src) {
		return FALSE;
	}
	LONG ModMom=Src;
	LONG ModSon=Dest;
	LONG ModNum=ModSon % ModMom;
	while (ModNum!=0) {
		ModSon=ModMom;
		ModMom=ModNum;
		ModNum=ModSon % ModMom;
	}
	Dest/=ModMom;
	Src/=ModMom;
	return TRUE;
}

WORD CFastRuler::SetRange(DWORD DestLength,DWORD SrcLength)	
{
	DWORD DestDpi, SrcDpi;
	
	m_MaxDestLength= DestLength;
	DestDpi = m_MaxDestLength;
	SrcDpi = SrcLength;
	if (!MinDivable(DestDpi,SrcDpi))
		return 0;
		
	FreeRuler();
/*	
	m_hRuler = GlobalAllocate(m_MaxDestLength*sizeof(UNIT));
	if (m_hRuler == nil ) return FALSE;
	HLock(m_hRuler);
*/	
	m_pRuler = (PUNIT)malloc(m_MaxDestLength*sizeof(UNIT));
	
	m_Step=SrcDpi;
	m_Bound=DestDpi;
	LONG m_Count=0;
	LONG m_Ratio=0;
	BOOL m_Cross=FALSE;
	while (m_Count<m_MaxDestLength){
		m_pRuler[m_Count].Ratio=m_Ratio;
		m_pRuler[m_Count].Cross=m_Cross;
		m_Ratio+=m_Step;
		if (m_Ratio>=m_Bound){
			m_Cross=TRUE;
			m_Ratio-=m_Bound;
		}
		else{
			m_Cross=FALSE;
		}
		m_Count++;
	}
	m_nCurRuler=0;
	m_nEndRuler=DestLength;
	return DestDpi;
}
BOOL CFastRuler::MinDivable(DWORD& Dest, DWORD& Src)
{
	if (Dest<Src) {
		return FALSE;
	}
	WORD ModMom=Src;
	WORD ModSon=Dest;
	WORD ModNum=(WORD)(ModSon % ModMom);
	while (ModNum!=0) {
		ModSon=ModMom;
		ModMom=ModNum;
		ModNum=(WORD)(ModSon % ModMom);
	}
	Dest/=ModMom;
	Src/=ModMom;
	return TRUE;
}
