/* BearPaw1200CU panel

   Copyright (C) 2001 Mustek. 
   Maintained by Tom Wang and Alex Lin
   <tom.wang@mustek.com.tw, alex_lin@mustek.com.tw>

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston,
   MA 02111-1307, USA.
*/


#ifndef _CFastRuler_
#define _CFastRuler_
#include "BaseTypedef.h"

typedef struct{
	LONG Ratio;
	BOOL Cross; 
} UNIT, *PUNIT;


class CFastRuler
{
public:
	CFastRuler();
 	~CFastRuler();
 	void FreeRuler();
	WORD SetRange(DWORD DestLength,DWORD SrcLength);
	LONG GetRange(LONG DestDpi,LONG SrcDpi,LONG DestLength);
    inline BOOL GetNextRGB(LONG& Ratio,LPSTR& lpOne,LPSTR& lpTwo);
    inline BOOL GetNextRGB(LONG& Ratio,LPSTR& lpOne);
    inline BOOL GetNextGray(LONG& Ratio,LPSTR& lpOne);
    inline BOOL GetNextGray(LONG& Ratio,LPSTR& lpOne,LPSTR& lpTwo);
	inline BOOL GetNext(LONG& Ratio,BOOL& isChange);
	inline void Reset();
	inline BOOL GetNextRGB30_36(LONG& Ratio,LPWORD& lpOne,LPWORD& lpTwo);
    inline BOOL GetNextRGB30_36(LONG& Ratio,LPWORD& lpOne);
    inline BOOL GetNextGray30_36(LONG& Ratio,LPWORD& lpOne);
    inline BOOL GetNextGray30_36(LONG& Ratio,LPWORD& lpOne,LPWORD& lpTwo);
    inline BOOL EndRuler();
private:
	PUNIT m_pRuler;
	LONG m_MaxDestLength;
	LONG m_Step;
	LONG m_Bound; 
	long m_nCurRuler;
	long m_nEndRuler;
	BOOL MinDivable(LONG& Dest, LONG& Src);
	BOOL MinDivable(DWORD& Dest, DWORD& Src);
};

inline void CFastRuler::Reset()
{
	m_nCurRuler=0;
}

inline BOOL CFastRuler::EndRuler()
{
	if (m_nCurRuler >= m_nEndRuler)
		return TRUE;
	else 
		return FALSE;
}
inline BOOL CFastRuler::GetNextRGB(LONG& Ratio,LPSTR& lpOne,LPSTR& lpTwo)
{
	if(m_nCurRuler > m_nEndRuler)
		return FALSE;
	Ratio=m_pRuler[m_nCurRuler].Ratio;
	if (m_pRuler[m_nCurRuler].Cross){
		lpOne+=3;
		lpTwo+=3;
	}
	m_nCurRuler++;
	return TRUE;
}

inline BOOL CFastRuler::GetNextRGB(LONG& Ratio,LPSTR& lpOne)
{
	if(m_nCurRuler > m_nEndRuler)
		return FALSE;
	Ratio=m_pRuler[m_nCurRuler].Ratio;
	if (m_pRuler[m_nCurRuler].Cross){
		lpOne+=3;
	}
	m_nCurRuler++;
	return TRUE;
}

inline BOOL CFastRuler::GetNextGray(LONG& Ratio,LPSTR& lpOne,LPSTR& lpTwo)
{
	if(m_nCurRuler>m_nEndRuler)
		return FALSE;
	Ratio=m_pRuler[m_nCurRuler].Ratio;
	if (m_pRuler[m_nCurRuler].Cross){
		lpOne++;
		lpTwo++;
	}
	m_nCurRuler++;
	return TRUE;
}

inline BOOL CFastRuler::GetNextGray(LONG& Ratio,LPSTR& lpOne)
{
	if(m_nCurRuler > m_nEndRuler)
		return FALSE;
	Ratio=m_pRuler[m_nCurRuler].Ratio;
	if (m_pRuler[m_nCurRuler].Cross){
		lpOne++;
	}
	m_nCurRuler++;
	return TRUE;
}

inline BOOL CFastRuler::GetNext(LONG& Ratio,BOOL& isChange)
{
	if(m_nCurRuler>m_nEndRuler)
		return FALSE;
	Ratio=m_pRuler[m_nCurRuler].Ratio;
	isChange=m_pRuler[m_nCurRuler].Cross;
	m_nCurRuler++;
	return TRUE;
}

inline BOOL CFastRuler::GetNextRGB30_36(LONG& Ratio,LPWORD& lpOne,LPWORD& lpTwo)
{
	if(m_nCurRuler>m_nEndRuler)
		return FALSE;
	Ratio=m_pRuler[m_nCurRuler].Ratio;
	if (m_pRuler[m_nCurRuler].Cross){
		lpOne+=3;
		lpTwo+=3;
	}
	m_nCurRuler++;
	return TRUE;
}

inline BOOL CFastRuler::GetNextRGB30_36(LONG& Ratio,LPWORD& lpOne)
{
	if(m_nCurRuler>m_nEndRuler)
		return FALSE;
	Ratio=m_pRuler[m_nCurRuler].Ratio;
	if (m_pRuler[m_nCurRuler].Cross){
		lpOne+=3;
	}
	m_pRuler++;
	return TRUE;
}

inline BOOL CFastRuler::GetNextGray30_36(LONG& Ratio,LPWORD& lpOne,LPWORD& lpTwo)
{
	if(m_nCurRuler>m_nEndRuler)
		return FALSE;
	Ratio=m_pRuler[m_nCurRuler].Ratio;
	if (m_pRuler[m_nCurRuler].Cross){
		lpOne++;
		lpTwo++;
	}
	m_nCurRuler++;
	return TRUE;
}

inline BOOL CFastRuler::GetNextGray30_36(LONG& Ratio,LPWORD& lpOne)
{
	if(m_nCurRuler>m_nEndRuler)
		return FALSE;
	Ratio=m_pRuler[m_nCurRuler].Ratio;
	if (m_pRuler[m_nCurRuler].Cross){
		lpOne++;
	}
	m_nCurRuler++;
	return TRUE;
}

#endif
