/* BearPaw1200CU panel

   Copyright (C) 2001 Mustek. 
   Maintained by Tom Wang and Alex Lin
   <tom.wang@mustek.com.tw, alex_lin@mustek.com.tw>

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston,
   MA 02111-1307, USA.
*/


#include <stdlib.h>
#include <memory.h>
#include <string.h>
#include "Filter.h"

CFilter::CFilter()
{
	m_lpBuffer1 = NULL;
	m_lpBuffer2 = NULL;
}

CFilter::~CFilter()
{
	if (m_lpBuffer1) {
		free(m_lpBuffer1);
		m_lpBuffer1 = NULL;
	}
	if (m_lpBuffer2) {
		free(m_lpBuffer2);
		m_lpBuffer2 = NULL;
	}
}

void CFilter::DeMoire(LPSTR lpSource,DS_SCANMODE ScanMode,DWORD ScanLines,DWORD BytesPerLine)
{
	AllocateBuffer(BytesPerLine);
	//Coef Matrix
	//	 0  1  0
	//	 1  3  1
	//	 0  1  0
	Coef[0][0]=Coef[0][2]=Coef[2][0]=Coef[2][2]= 0;
	Coef[0][1]=Coef[1][0]=Coef[1][2]=Coef[2][1]= 1;
	Coef[1][1]=3 ;
	DataProcessor(lpSource,ScanMode,ScanLines,BytesPerLine);
	
	//Coef Matrix
	//	 1  1  1
	//	 1  1  1
	//	 1  1  1
	Coef[0][0]=Coef[0][2]=Coef[2][0]=Coef[2][2]= 1;
	Coef[0][1]=Coef[1][0]=Coef[1][2]=Coef[2][1]= 1;
	Coef[1][1]=1;
	DataProcessor(lpSource,ScanMode,ScanLines,BytesPerLine);	
	ReleaseBuffer();
}

void CFilter::UnSharpMask(LPSTR lpSource,DS_SCANMODE ScanMode,DWORD ScanLines,DWORD BytesPerLine)
{
	AllocateBuffer(BytesPerLine);
	//Coef Matrix
	//	-5 -1 -5
	//	-1 29 -1
	//	-5 -1 -5
	Coef[0][0]=Coef[0][2]=Coef[2][0]=Coef[2][2]=-5;
	Coef[0][1]=Coef[1][0]=Coef[1][2]=Coef[2][1]=-1;
	Coef[1][1]=29;

	DataProcessor(lpSource,ScanMode,ScanLines,BytesPerLine);	
	ReleaseBuffer();
}

void CFilter::DeMoire_LoRes1(LPSTR lpSource,DS_SCANMODE ScanMode,DWORD ScanLines,DWORD BytesPerLine)
{
	AllocateBuffer(BytesPerLine);
	//Coef Matrix
	//	 0  1  0
	//	 1  3  1
	//	 0  1  0
	Coef[0][0]=Coef[0][2]=Coef[2][0]=Coef[2][2]= 0;
	Coef[0][1]=Coef[1][0]=Coef[1][2]=Coef[2][1]= 1;
	Coef[1][1]=3 ;
	DataProcessor(lpSource,ScanMode,ScanLines,BytesPerLine);	
	ReleaseBuffer();
}

void CFilter::DeMoire_LoRes2(LPSTR lpSource,DS_SCANMODE ScanMode,DWORD ScanLines,DWORD BytesPerLine)
{
	AllocateBuffer(BytesPerLine);
	//Coef Matrix
	//	 1  1  1
	//	 1  1  1
	//	 1  1  1
	Coef[0][0]=Coef[0][2]=Coef[2][0]=Coef[2][2]= 1;
	Coef[0][1]=Coef[1][0]=Coef[1][2]=Coef[2][1]= 1;
	Coef[1][1]=1;
	DataProcessor(lpSource,ScanMode,ScanLines,BytesPerLine);	
	ReleaseBuffer();
}

void CFilter::Emboss(LPSTR lpSource,DS_SCANMODE ScanMode,DWORD ScanLines,DWORD BytesPerLine)
{
	AllocateBuffer(BytesPerLine);
	
	LPSTR	tmpPtr=lpSource;
	DWORD	row,index;
	DWORD   ScanPixles;
	int    tmp;

    switch ( ScanMode )
      {
        case DSSM_LINEART:
			break;
		case DSSM_GRAY:
			ScanPixles = BytesPerLine;
			for( row=0;row<ScanLines;row++ ){
			 for( index=0;index<ScanPixles;index++ ){
				if(ScanLines == 1)// force to set pixels to 128, //09/17 by Brian
				{
				  if(index == ScanPixles-1)
					 *(tmpPtr+index) = (BYTE)128;
				  else
				  {
					 tmp = (BYTE)*(tmpPtr+index);
					 tmp -= (BYTE)*(tmpPtr+index+1);
					 tmp += 128;
					 if( tmp>255 ) tmp = 255;
					 if( tmp<0 ) tmp = 0;
					 *(tmpPtr+index) = (BYTE)tmp;
				  }
				}
				else
                 if( row==ScanLines-1 ) // the last line
						 *(tmpPtr+index) = *(tmpPtr+index-BytesPerLine);
                 else{
					 tmp = (BYTE)*(tmpPtr+index);
					 tmp -= (BYTE)*(tmpPtr+index+BytesPerLine);
					 tmp += 128;
					 if( tmp>255 ) tmp = 255;
					 if( tmp<0 ) tmp = 0;
					 *(tmpPtr+index) = (BYTE)tmp;
                 }
             }
             tmpPtr += BytesPerLine;
           }
			break;
		case DSSM_COLOR24:
			ScanPixles = BytesPerLine/3;
            for( row=0;row<ScanLines;row++ ){
			 for( index=0;index<ScanPixles;index++ ){
				if(ScanLines == 1)//09/17 by Brian
				{  // when lines == 1, we have no choice but to set pixels to 128
				  if(index==ScanPixles-1)
				  {
					 *(tmpPtr+index*3) = (BYTE)128;
					 *(tmpPtr+index*3+1) = (BYTE)128;
					 *(tmpPtr+index*3+2) = (BYTE)128;
				  }
				  else
				  {
				   tmp = (BYTE)*(tmpPtr+index*3);
				   tmp -= (BYTE)*(tmpPtr+(index+1)*3);
                   tmp += 128;
                   if( tmp>255 ) tmp = (BYTE)255;
                   if( tmp<0 ) tmp = 0;
                   *(tmpPtr+index*3) = (BYTE)tmp;
                   tmp = (BYTE)*(tmpPtr+index*3+1);
				   tmp -= (BYTE)*(tmpPtr+(index+1)*3+1);
                   tmp += 128;
                   if( tmp>255 ) tmp = (BYTE)255;
                   if( tmp<0 ) tmp = 0;
                   *(tmpPtr+index*3+1) = (BYTE)tmp;
                   tmp = (BYTE)*(tmpPtr+index*3+2);
			       tmp -= (BYTE)*(tmpPtr+(index+1)*3+2);
                   tmp += 128;
                   if( tmp>255 ) tmp = 255;
                   if( tmp<0 ) tmp = 0;
						 *(tmpPtr+index*3+2) = (BYTE)tmp;
				  }
				}
				else
                 if( row==ScanLines-1 ){
					*(tmpPtr+index*3) = *(tmpPtr+index*3-BytesPerLine);
					*(tmpPtr+index*3+1) = *(tmpPtr+index*3+1-BytesPerLine);
					*(tmpPtr+index*3+2) = *(tmpPtr+index*3+2-BytesPerLine);
					}
                 else{
                   tmp = (BYTE)*(tmpPtr+index*3);
                   tmp -= (BYTE)*(tmpPtr+index*3+BytesPerLine);
                   tmp += 128;
                   if( tmp>255 ) tmp = 255;
                   if( tmp<0 ) tmp = 0;
                   *(tmpPtr+index*3) = (BYTE)tmp;
                   tmp = (BYTE)*(tmpPtr+index*3+1);
                   tmp -= (BYTE)*(tmpPtr+index*3+1+BytesPerLine);
                   tmp += 128;
                   if( tmp>255 ) tmp = 255;
                   if( tmp<0 ) tmp = 0;
                   *(tmpPtr+index*3+1) = (BYTE)tmp;
                   tmp = (BYTE)*(tmpPtr+index*3+2);
                   tmp -= (BYTE)*(tmpPtr+index*3+2+BytesPerLine);
                   tmp += 128;
                   if( tmp>255 ) tmp = 255;
                   if( tmp<0 ) tmp = 0;
                   *(tmpPtr+index*3+2) = (BYTE)tmp;
                 }
             }
             tmpPtr += BytesPerLine;
           }
		break;
	}	
	
	ReleaseBuffer();
}

void CFilter::Sharpen(LPSTR lpSource,DS_SCANMODE ScanMode,DWORD ScanLines,DWORD BytesPerLine)
{
	AllocateBuffer(BytesPerLine);
	LPSTR	tmpPtr=lpSource;
	//WORD	row,index;		// Jewel 06/22/98
	//WORD    ScanPixles;	// Jewel 06/22/98
	DWORD	row,index;		// Jewel 06/22/98
	DWORD    ScanPixles;	// Jewel 06/22/98
	int    tmp,diff;

    switch ( ScanMode )  // for ADF
      {
        case DSSM_LINEART :
		case DSSM_GRAY:
		   ScanPixles = BytesPerLine;
           for( row=0;row<ScanLines;row++ ){
			   for( index=0;index<ScanPixles;index++ ){
               if( index==0 || index==ScanPixles-1 )
                 continue;
               else{
                 tmp = (BYTE)*(tmpPtr+index);
                 tmp *=3;
                 tmp -= (BYTE)*(tmpPtr+index+1);
                 tmp -= (BYTE)*(tmpPtr+index-1);
                 if( tmp>255 ) tmp = 255;
                 if( tmp<0   ) tmp = 0;
                 diff = tmp-(BYTE)*(tmpPtr+index);
                 if( abs(diff)>25 )
                   continue;
                 else
                   *(tmpPtr+index) = (BYTE)tmp;
               }
             }
             tmpPtr += BytesPerLine;
           }
			break;
		case DSSM_COLOR24:
		   ScanPixles = BytesPerLine/3;
           for( row=0;row<ScanLines;row++ ){
             for( index=0;index<ScanPixles;index++ ){
               if( index==0 || index==ScanPixles-1 )
                 continue;
               else{
                 tmp = (BYTE)*(tmpPtr+index*3);
                 tmp *=3;
                 tmp -= (BYTE)*(tmpPtr+index*3+3);
                 tmp -= (BYTE)*(tmpPtr+index*3-3);
                 if( tmp>255 ) tmp = 255;
                 if( tmp<0   ) tmp = 0;
                 diff = tmp-(BYTE)*(tmpPtr+index*3);
                 if( abs(diff)>25 )
                   continue;
                 else
                   *(tmpPtr+index*3) = (BYTE)tmp;
                 tmp = (BYTE)*(tmpPtr+index*3+1);
                 tmp *=3;
                 tmp -= (BYTE)*(tmpPtr+index*3+3+1);
                 tmp -= (BYTE)*(tmpPtr+index*3-3+1);
                 if( tmp>255 ) tmp = 255;
                 if( tmp<0   ) tmp = 0;
                 diff = tmp-(BYTE)*(tmpPtr+index*3+1);
                 if( abs(diff)>25 )
                   continue;
                 else
                   *(tmpPtr+index*3+1) = (BYTE)tmp;
                 tmp = (BYTE)*(tmpPtr+index*3+2);
                 tmp *=3;
                 tmp -= (BYTE)*(tmpPtr+index*3+3+2);
                 tmp -= (BYTE)*(tmpPtr+index*3-3+2);
                 if( tmp>255 ) tmp = 255;
                 if( tmp<0   ) tmp = 0;
                 diff = tmp-(BYTE)*(tmpPtr+index*3+2);
                 if( abs(diff)>25 )
                   continue;
                 else
                   *(tmpPtr+index*3+2) = (BYTE)tmp;
               }
             }
             tmpPtr += BytesPerLine;
           }
			break;
	}
	
	ReleaseBuffer();

}

void CFilter::Blur(LPSTR lpSource,DS_SCANMODE ScanMode,DWORD ScanLines,DWORD BytesPerLine)
{
	AllocateBuffer(BytesPerLine);
	//Coef Matrix
	//	5 3 5
	//	3 1 3
	//	5 3 5
	Coef[0][0]=Coef[0][2]=Coef[2][0]=Coef[2][2]=5;
	Coef[0][1]=Coef[1][0]=Coef[1][2]=Coef[2][1]=3;
	Coef[1][1]=1;
	DataProcessor(lpSource,ScanMode,ScanLines,BytesPerLine);	
	ReleaseBuffer();
}

void CFilter::Flip(LPSTR lpSource,DS_SCANMODE ScanMode,DWORD ScanLines,DWORD BytesPerLine)
{
	AllocateBuffer(BytesPerLine);
	
	LPSTR	tmpPtr=lpSource;
	DWORD	row,index;
	DWORD    ScanPixles;	
	int    tmp;

	switch ( ScanMode )
	{
        case DSSM_LINEART :
        	LPSTR lpFlipBuf,lpOrgBuf,tmPtr_MemXfer, lpFree;
        	lpFlipBuf = (LPSTR) malloc (BytesPerLine);
        	if (!lpFlipBuf) break;
		lpFree = lpFlipBuf;	
		lpOrgBuf = lpSource;
		tmPtr_MemXfer = lpFlipBuf;
		for ( row = 0; row < ScanLines; row++ ) 
		{
			memcpy( lpFlipBuf , lpOrgBuf , BytesPerLine );
			lpOrgBuf = lpOrgBuf+BytesPerLine;
			tmpPtr = lpOrgBuf - 1;
			for ( index = 0; index < BytesPerLine; index++ ) 
			{
				*tmpPtr = (BYTE) 0 ;
				*tmpPtr |= ((BYTE) *lpFlipBuf & 0x80 ) >> 7 ;
				*tmpPtr |= ((BYTE) *lpFlipBuf & 0x40 ) >> 5 ;
				*tmpPtr |= ((BYTE) *lpFlipBuf & 0x20 ) >> 3 ;
				*tmpPtr |= ((BYTE) *lpFlipBuf & 0x10 ) >> 1 ;
				*tmpPtr |= ((BYTE) *lpFlipBuf & 0x08 ) << 1 ;
				*tmpPtr |= ((BYTE) *lpFlipBuf & 0x04 ) << 3 ;
				*tmpPtr |= ((BYTE) *lpFlipBuf & 0x02 ) << 5 ;
				*tmpPtr |= ((BYTE) *lpFlipBuf & 0x01 ) << 7 ;
				tmpPtr-- ;
				lpFlipBuf++ ;
			}
			lpFlipBuf = tmPtr_MemXfer;
		}
		free(lpFree);
		break;
        case DSSM_GRAY :
		ScanPixles = BytesPerLine;
		for( row=0;row<ScanLines;row++ ){
			for( index=0;index<=ScanPixles/2;index++ ){
				tmp = *(tmpPtr+index) ;
				*(tmpPtr+index) = *(tmpPtr+BytesPerLine-index-1 );
				*(tmpPtr+BytesPerLine-index-1 ) = (BYTE)tmp;
			}
			tmpPtr += BytesPerLine;
		}
		break;
	case DSSM_COLOR24:
		ScanPixles = BytesPerLine/3;
		for( row=0;row<ScanLines;row++ ){
			for( index=0;index<=ScanPixles/2;index++ ){
				tmp = *(tmpPtr+index*3) ; // Red
				*(tmpPtr+index*3) = *(tmpPtr+BytesPerLine-index*3-3 );
				*(tmpPtr+BytesPerLine-index*3-3 ) = (BYTE)tmp;
				tmp = *(tmpPtr+index*3+1) ; // Green
				*(tmpPtr+index*3+1) = *(tmpPtr+BytesPerLine-index*3-2 );
				*(tmpPtr+BytesPerLine-index*3-2 ) = (BYTE)tmp;
				tmp = *(tmpPtr+index*3+2) ; // Blue
				*(tmpPtr+index*3+2) = *(tmpPtr+BytesPerLine-index*3-1 );
				*(tmpPtr+BytesPerLine-index*3-1 ) = (BYTE)tmp;
			}
			tmpPtr += BytesPerLine;
		}
		break;
	}
	ReleaseBuffer();
}

void CFilter::Invert(LPSTR lpSource,DS_SCANMODE scanMode,DWORD ScanLines,DWORD BytesPerLine)
{
	AllocateBuffer(BytesPerLine);
	LPSTR	tmpPtr=lpSource;
	DWORD	row,index;	// Jewel 06/22/98

	for(row=0;row<ScanLines;row++)
	{
		for(index=0;index<BytesPerLine;index++)
			*(tmpPtr+index)=~*(tmpPtr+index);
		tmpPtr+=BytesPerLine;
	}

	ReleaseBuffer();
}

void CFilter::AllocateBuffer(DWORD BytesPerLine)
{
	m_lpBuffer1 = (LPSTR) malloc (BytesPerLine);
	m_lpBuffer2 = (LPSTR) malloc (BytesPerLine);	
}

void CFilter::ReleaseBuffer()
{
	free (m_lpBuffer1);
	free (m_lpBuffer2);
}
BYTE CFilter::MatrixCalculate()
{
	int	x,y;
	int     Dev;
	float	tmpData;

	tmpData=0;
	Dev=0;
	for(x=0;x<3;x++)
	  for(y=0;y<3;y++)
	  {
		Dev += Coef[x][y];
		tmpData+=(Coef[x][y]*ImageMatrix[x][y]);
	  }

	if(Dev!=0)	tmpData/=Dev;
	if(tmpData>255) tmpData=255;
	if(tmpData<0) tmpData=0;
	return (BYTE)tmpData;

}
	
void CFilter::AutoLevel(LPSTR lpSource, DS_SCANMODE scanMode, DWORD ScanLines, DWORD BytesPerLine)
{
   int    ii;
   DWORD  i,j;
   DWORD  tLines,CountPixels,TotalImgSize;
   WORD   R,G,B,max_R,max_G,max_B,min_R,min_G,min_B;
   float  fmax_R,fmax_G,fmax_B;
   DWORD  sum_R = 0, sum_G = 0, sum_B = 0;
   float  mean_R, mean_G, mean_B;
   DWORD  hisgram_R[256], hisgram_G[256], hisgram_B[256];
      
      
   if (scanMode != DSSM_COLOR24) return;
   
   i = j = 0;
   tLines = CountPixels = TotalImgSize = 0;
	DWORD iWidth=BytesPerLine/3;
	DWORD iHeight=ScanLines;
	BYTE* pbmpdata=(BYTE*)lpSource;

   TotalImgSize = iWidth * iHeight;
//////////////////////////////////////////////////////////////////
   for(i=0; i<256; i++){
          
      hisgram_R[i] = 0; hisgram_G[i] = 0; hisgram_B[i] = 0;
   }
   
//////////////////////////////////////////////////////////////////
//Find min , max, mean
   max_R = max_G = max_B =0;
   min_R = min_G = min_B =255;
   tLines = 0;
   for(j=0; j<iHeight; j++){
          tLines = j * iWidth *3;
		  for(i=0;i<iWidth;i++)
		  {
			 R = (WORD)(BYTE)*(pbmpdata + (tLines + i * 3 + 2));
			 G = (WORD)(BYTE)*(pbmpdata + (tLines + i * 3 + 1));
			 B = (WORD)(BYTE)*(pbmpdata + (tLines + i * 3));

			 max_R = max(R,max_R);
			 max_G = max(G,max_G);
			 max_B = max(B,max_B);

             min_R = min(R,min_R);
             min_G = min(G,min_G);
			 min_B = min(B,min_B);

			 hisgram_R[(BYTE)R]++; hisgram_G[(BYTE)G]++; hisgram_B[(BYTE)B]++;

			 sum_R += R; sum_G += G; sum_B += B;

			 *(pbmpdata + (tLines + i * 3 + 2)) = (BYTE)R;
			 *(pbmpdata + (tLines + i * 3 + 1)) = (BYTE)G;
			 *(pbmpdata + (tLines + i * 3))     = (BYTE)B;

             CountPixels++;
		  }//end for
          
   }//end for
   mean_R = (float)(sum_R / TotalImgSize);
   mean_G = (float)(sum_G / TotalImgSize);
   mean_B = (float)(sum_B / TotalImgSize);
//////////////////////////////////////////////////////////////////

   DWORD   tmp = 0;
   WORD   imin_threshold[3];
   WORD   imax_threshold[3];

   imin_threshold[0] = 0;    imin_threshold[1] = 0;    imin_threshold[2] = 0;
   imax_threshold[0] = 0;    imax_threshold[1] = 0;    imax_threshold[2] = 0;

   for(ii = 0; ii < 256; ii++)
   {
	  if(hisgram_R[ii] > 0)
		if(hisgram_R[ii] >= imin_threshold[0])
		{
		  min_R = ii;
		  break;
		}
		//else{
        //  tmp += hisgram_R[i];
		//}
   }

   tmp = 0;
   for (ii = 255; ii >= 0; ii--)
   {
	   if (hisgram_R[ii] > 0)
		if(hisgram_R[ii] >= imax_threshold[0])
		{
		   max_R = ii;
		   break;
		}
		//else{
        //   tmp += hisgram_R[i];
		//}
	   
   }

   tmp = 0;
   for (ii = 0; ii < 256; ii++)
   {
	   if(hisgram_G[ii] > 0)
	    if(hisgram_G[ii] >= imin_threshold[1])
		{
		  min_G = ii;
		  break;
		}
		//else{
        //  tmp += hisgram_G[i];
		//}
   }

   tmp = 0;
   for (ii = 255; ii >= 0; ii--)
   {
	   if(hisgram_G[ii] > 0)
		if(hisgram_G[ii] >= imax_threshold[1])
		{
		   max_G = ii;
		   break;
		}
		//else{
        //   tmp += hisgram_G[i];
		//}
   }

   tmp = 0;
   for (ii = 0; ii < 256; ii++)
   {
	   if(hisgram_B[ii] > 0)
	    if(hisgram_B[ii]  >= imin_threshold[2])
		{
		  min_B = ii;
		  break;
		}
		//else{
        //  tmp += hisgram_B[i];
		//}
   }

   tmp = 0;
   for (ii = 255; ii >= 0; ii--)
   {
	   if(hisgram_B[ii] > 0)
	    if(hisgram_B[ii] >= imax_threshold[2])
		{
		  max_B = ii;
		  break;
		}
		//else{
        //  tmp += hisgram_B[i];
		//}
   }

///////////////////////////////////////////////////////////////////////////
//Autolevel:

   sum_R = max_R-min_R;
   sum_G = max_G-min_G;
   sum_B = max_B-min_B;

//////////////////////////////////////////////////////////////////////////
   
   for(j=0; j<iHeight; j++){
          tLines = j * iWidth *3;
		  for(i=0;i<iWidth;i++)
		  {
			 R = (WORD)(BYTE)*(pbmpdata + (tLines + i * 3 + 2));
			 G = (WORD)(BYTE)*(pbmpdata + (tLines + i * 3 + 1));
			 B = (WORD)(BYTE)*(pbmpdata + (tLines + i * 3));
			 
			 //-------------------------R
			 if(sum_R == 0) R = max_R;
			 else if (R < min_R) R = 0;
			 else if(R<=255){
				 fmax_R = ((float)((R - min_R)*255)/(float)sum_R); 
				 R = (WORD)fmax_R;
				 fmax_R = (fmax_R - R)*10;

				if(fmax_R >=5)
					 R++;
			 }
			 if (R > 255)   R = 255;
			 //-------------------------G	
			 if(sum_G == 0) G = max_G;
			 else if (G < min_G) G = 0;
			 else if(G<=255){
				 fmax_G = ((float)((G - min_G)*255)/(float)sum_G); 
				 G = (WORD)fmax_G;
				 fmax_G = (fmax_G - G)*10;
				 if(fmax_G >=5)
					 G++;
				 
			 }
			 if (G > 255)   G = 255;
			 //-------------------------B
			 if(sum_B == 0) B = max_B;
			 else if (B < min_B) B = 0;
			 else if(B<=255){
				 fmax_B = ((float)(B - min_B)*255/(float)sum_B); 
				 B = (WORD)fmax_B;
				 fmax_B = (fmax_B - B)*10;
				 if(fmax_B >= 5)
					 B++;
			 }
			 if (B > 255)   B = 255;

			 hisgram_R[(BYTE)R]++; hisgram_G[(BYTE)G]++; hisgram_B[(BYTE)B]++;

			*(pbmpdata + (tLines + i * 3 + 2)) = (BYTE)R;
			*(pbmpdata + (tLines + i * 3 + 1)) = (BYTE)G;
			*(pbmpdata + (tLines + i * 3))     = (BYTE)B;

		  }//end for

	   }//end for
	return;
}


void CFilter::QBETDetectAutoLevel(void* pDIB, DWORD ImageWidth, DWORD ImageHeight)
{
   WORD	  *pbmpdata;
   
   if (!pDIB)
      return;

   pbmpdata    = (WORD *) pDIB;

   int    i,j;
   DWORD  tLines,CountPixels,TotalImgSize;
   WORD   R,G,B,max_R,max_G,max_B,min_R,min_G,min_B;
   WORD   wIndexR,wIndexG,wIndexB;
   float  fmax_R,fmax_G,fmax_B;
   DWORD  sum_R = 0, sum_G = 0, sum_B = 0;
   DWORD  hisgram_R[1024], hisgram_G[1024], hisgram_B[1024];
      

   CountPixels = 0;
   TotalImgSize = ImageWidth * ImageHeight;
  
//////////////////////////////////////////////////////////////////
   for(i=0; i<1024; i++){
          
      hisgram_R[i] = 0; hisgram_G[i] = 0; hisgram_B[i] = 0;
   }
   
//////////////////////////////////////////////////////////////////
//Find min , max, mean
   max_R = max_G = max_B =0;
   min_R = min_G = min_B =1023;
   tLines = 0;

   for(j=0; j<(int)ImageHeight; j++){
          tLines = j * ImageWidth *3;
		  for(i=0;i<(int)ImageWidth;i++)
		  {
			 R = *(pbmpdata + (tLines + i * 3 + 2));
			 G = *(pbmpdata + (tLines + i * 3 + 1));
			 B = *(pbmpdata + (tLines + i * 3));

			 max_R = max(R,max_R);
			 max_G = max(G,max_G);
			 max_B = max(B,max_B);

             min_R = min(R,min_R);
             min_G = min(G,min_G);
			 min_B = min(B,min_B);

			 hisgram_R[R]++; hisgram_G[G]++; hisgram_B[B]++;

			 sum_R += R; sum_G += G; sum_B += B;

			 *(pbmpdata + (tLines + i * 3 + 2)) = R;
			 *(pbmpdata + (tLines + i * 3 + 1)) = G;
			 *(pbmpdata + (tLines + i * 3))     = B;

             CountPixels++;
		  }//end for
          
   }//end for
///////////////////////////////////////////////////////////////////////////
   float  fRPercent = 0.0;
   float  fGPercent = 0.0;
   float  fBPercent = 0.0;
   float  fRSum,fGSum,fBSum;

   fRSum = 0.0;
   fGSum = 0.0;
   fBSum = 0.0;

   wIndexR = 511;
   wIndexG = 511;
   wIndexB = 511;

   for(i=0;i<1024;i++){
     fRSum += (float)hisgram_R[i];
     fRPercent = (fRSum / CountPixels)*100;
	 if(fRPercent > 50) 
     { 
       wIndexR = i;
	   break;
	 }
	 
   }

   for(i=0;i<1024;i++){
     fGSum += (float)hisgram_G[i];
     fGPercent = (fGSum / CountPixels) * 100;
	 if(fGPercent > 50) 
     { 
       wIndexG = i;
	   break;
	 }
	 
   }

   for(i=0;i<1024;i++){
     fBSum += (float)hisgram_B[i];
     fBPercent = (fBSum / CountPixels) * 100;
	 if(fBPercent > 50) 
     { 
       wIndexB = i;
	   break;
	 }
	 
   }
//====================================================
   fRSum = 0.0;
   for(i=wIndexR;i>=0;i--){
     fRSum += (float)hisgram_R[i];
     fRPercent = (fRSum / CountPixels) * 100;
	 if(fRPercent >= 48) 
     { 
       min_R = i;
	   break;
	 }
	 
   }

   fRSum = 0.0;
   for(i=wIndexR;i<1024;i++){
     fRSum += (float)hisgram_R[i];
     fRPercent = (fRSum / CountPixels) * 100;
	 if(fRPercent >= 47) 
     { 
       max_R = i;
	   break;
	 }
	 
   }
   //======
   fGSum = 0.0;
   for(i=wIndexG;i>=0;i--){
     fGSum += (float)hisgram_G[i];
     fGPercent = (fGSum / CountPixels) * 100;
	 if(fGPercent >= 48) 
     { 
       min_G = i;
	   break;
	 }
	 
   }

   fGSum = 0.0;
   for(i=wIndexG;i<1024;i++){
     fGSum += (float)hisgram_G[i];
     fGPercent = (fGSum / CountPixels) * 100;
	 if(fGPercent >= 47) 
     { 
       max_G = i;
	   break;
	 }
	 
   }

   //======
   fBSum = 0.0;
   for(i=wIndexB;i>=0;i--){
     fBSum += (float)hisgram_B[i];
     fBPercent = (fBSum / CountPixels) * 100;
	 if(fBPercent >= 46)//50) 
     { 
       min_B = i;
	   break;
	 }
	 
   }

   fBSum = 0.0;
   for(i=wIndexB;i<1024;i++){
     fBSum += (float)hisgram_B[i];
     fBPercent = (fBSum / CountPixels) * 100;
	 if(fBPercent >= 47)//49) 
     { 
       max_B = i;
	   break;
	 }
	 
   }

///////////////////////////////////////////////////////////////////////////
//Autolevel:
  
   sum_R = max_R- min_R;
   sum_G = max_G- min_G;
   sum_B = max_B- min_B;
   
   for(j=0; j<(int)ImageHeight; j++){
          tLines = j * ImageWidth *3;
		  for(i=0;i<(int)ImageWidth;i++)
		  {
			 R = *(pbmpdata + (tLines + i * 3 + 2));
			 G = *(pbmpdata + (tLines + i * 3 + 1));
			 B = *(pbmpdata + (tLines + i * 3));
			 
		
			 //-------------------------R
			 if(sum_R == 0) R = max_R;
			 else if(R<min_R){
				  
				  R=0;
			 }
			 else if((R>=min_R) && (R<=1023)){
				 fmax_R = ((float)((R - min_R)*923)/(float)sum_R)+100; 
				 R = (WORD)fmax_R;
				 fmax_R = (fmax_R - R)*10;
				 if(fmax_R >=5)
				    R++;
			 }
			 if (R > 1023)   R = 1023;

			 //-------------------------G	
			 if(sum_G == 0) G = max_G;
			 else if(G<min_G){
                 
				 G=0;
			 }
			 else if((G>=min_G) && (G<=1023)){
				 fmax_G = ((float)((G - min_G)*923)/(float)sum_G)+100; 
				 G = (WORD)fmax_G;
				 fmax_G = (fmax_G - G)*10;
				 if(fmax_G >=5)
					 G++;
			 }
			 if (G > 1023)   G = 1023;

			 //-------------------------B
			 if(sum_B == 0) B = max_B;
			 else if(B<min_R){
				 
				 B=0;
			 }
			 else if((B>=min_B) && (R<=1023)){
				 fmax_B = ((float)(B - min_B)*923/(float)sum_B)+100; 
				 B = (WORD)fmax_B;
				 fmax_B = (fmax_B - B)*10;
				 if(fmax_B >= 5)
					 B++;
			 }
			 if (B > 1023)   B = 1023;
			 
			*(pbmpdata + (tLines + i * 3 + 2)) = R;
			*(pbmpdata + (tLines + i * 3 + 1)) = G;
			*(pbmpdata + (tLines + i * 3))     = B;

		  }//end for

	   }//end for
   return;

}//end QBETAUTOLEVEL()



void CFilter::QBetChange(LPSTR lpSource, DS_SCANMODE scanMode, DWORD ScanLines, DWORD BytesPerLine)
{
   WORD			i,j;
   DWORD		tLines,TotalImgSize;
   WORD			R1,G1,B1,R,G,B,R2,G2,B2,QBET_RGB,PointF,PointB;
   WORD*		pwRGB;
   
   if (scanMode != DSSM_COLOR24) return;
   DWORD ImageWidth=BytesPerLine/3;
   DWORD ImageHeight=ScanLines;
   BYTE* pbmpdata=(BYTE*)lpSource;


//////////////////////////////////ALLOCATE COPY BUFFER/////////////////////////////
   TotalImgSize = ImageWidth * ImageHeight *3 *2;
   if((pwRGB = (WORD*) malloc (TotalImgSize) ) == NULL)
   {
	  return ;
   }      

//////////////////////////////////QBET///////////////////////////////////////////////
   for(j=0; j<ImageHeight; j++){
          tLines = j * ImageWidth *3;
		  for(i=0;i<ImageWidth;i++)
		  {
             if(i == 0){ 
                 //p[i-1]
			     //p[i]
				 R1= R  = (WORD)(BYTE)*(pbmpdata + (tLines + i * 3 + 2));
			     G1= G  = (WORD)(BYTE)*(pbmpdata + (tLines + i * 3 + 1));
			     B1= B  = (WORD)(BYTE)*(pbmpdata + (tLines + i * 3));
				 //p[i+1]
			     R2 = (WORD)(BYTE)*(pbmpdata + (tLines + (i+1) * 3 + 2));
			     G2 = (WORD)(BYTE)*(pbmpdata + (tLines + (i+1) * 3 + 1));
			     B2 = (WORD)(BYTE)*(pbmpdata + (tLines + (i+1) * 3));
			 }
			 else if(i == (ImageWidth-1)){
			     //p[i-1]
				 R1 = (WORD)(BYTE)*(pbmpdata + (tLines + (i-1) * 3 + 2));
			     G1 = (WORD)(BYTE)*(pbmpdata + (tLines + (i-1) * 3 + 1));
			     B1 = (WORD)(BYTE)*(pbmpdata + (tLines + (i-1) * 3));
			     //p[i]
				 //p[i+1]
				 R2= R  = (WORD)(BYTE)*(pbmpdata + (tLines + i * 3 + 2));
			     G2= G  = (WORD)(BYTE)*(pbmpdata + (tLines + i * 3 + 1));
			     B2= B  = (WORD)(BYTE)*(pbmpdata + (tLines + i * 3));
				 
			     
			 }
			 else{
			    //p[i-1]
			    R1 = (WORD)(BYTE)*(pbmpdata + (tLines + (i-1) * 3 + 2));
			    G1 = (WORD)(BYTE)*(pbmpdata + (tLines + (i-1) * 3 + 1));
			    B1 = (WORD)(BYTE)*(pbmpdata + (tLines + (i-1) * 3));
			    //p[i]
			    R  = (WORD)(BYTE)*(pbmpdata + (tLines + i * 3 + 2));
			    G  = (WORD)(BYTE)*(pbmpdata + (tLines + i * 3 + 1));
			    B  = (WORD)(BYTE)*(pbmpdata + (tLines + i * 3));
			    //p[i+1]
			    R2 = (WORD)(BYTE)*(pbmpdata + (tLines + (i+1) * 3 + 2));
			    G2 = (WORD)(BYTE)*(pbmpdata + (tLines + (i+1) * 3 + 1));
			    B2 = (WORD)(BYTE)*(pbmpdata + (tLines + (i+1) * 3));
			 }



             R1 = R1 & 0x0003;//16i
			 G1 = G1 & 0x0003;
			 B1 = B1 & 0x0003;

			 R2 = R2 & 0x0003;
			 G2 = G2 & 0x0003;
			 B2 = B2 & 0x0003;
			 for(int k=0;k<3;k++){
			 	if(k == 0){ 
			       PointF = R1;
			       PointB = R2;
				}
				else if(k == 1)
				{
				   PointF = G1;
				   PointB = G2;
				}
				else if(k == 2)
				{
				   PointF = B1;
				   PointB = B2;
				}
                      
				switch(PointF){
				   case 0:
				   case 1:	
					   if(PointB == 0)
						   QBET_RGB = 0xFFFC;
					   else if(PointB == 1)
						   QBET_RGB = 0xFFFC;
					   else if(PointB == 2)
						   QBET_RGB = 0xFFFD;
					   else if(PointB == 3)
						   QBET_RGB = 0xFFFE;
				       break;
				   case 2:
					   if(PointB == 0)
						   QBET_RGB = 0xFFFD;
					   else if(PointB == 1)
						   QBET_RGB = 0xFFFD;
					   else if(PointB == 2)
						   QBET_RGB = 0xFFFF;
					   else if(PointB == 3)
						   QBET_RGB = 0xFFFF;
					   break;
				   case 3:
					   if(PointB == 0)
						   QBET_RGB = 0xFFFE;
					   else if(PointB == 1)
						   QBET_RGB = 0xFFFE;
					   else if(PointB == 2)
						   QBET_RGB = 0xFFFF;
					   else if(PointB == 3)
						   QBET_RGB = 0xFFFF;
					   break;
				   default:
					   break;	
				}//end switch()

                if(k==0){
                    R = R << 2;
                    R = R + 0x0003;
                    R = R & QBET_RGB;
				}
				else if(k==1){
					G = G << 2;
                    G = G + 0x0003;
                    G = G & QBET_RGB;
				}
				else if(k==2){
                    B = B << 2;
                    B = B + 0x0003;
                    B = B & QBET_RGB;
				}//end if

			 }//end for(k=...)
             
			 *(pwRGB + (tLines + i * 3 + 2)) = R;
			 *(pwRGB + (tLines + i * 3 + 1)) = G;
			 *(pwRGB + (tLines + i * 3))     = B;
			
		  }//end for
          
   }//end for
///////////////////////////////////////////////////////////////
// Do AutoLevel:    
   QBETDetectAutoLevel(pwRGB,ImageWidth,ImageHeight); 
//////Autolevel End/////////////QBET -0> 8bit//////////////////
    for(j=0; j<ImageHeight; j++){
          tLines = j * ImageWidth *3;
		  for(i=0;i<ImageWidth;i++)
		  {
			  R = *(pwRGB + (tLines + i * 3 + 2)) ;
			  G = *(pwRGB + (tLines + i * 3 + 1)) ;
			  B = *(pwRGB + (tLines + i * 3))     ;
            
			  R = R >> 2;
			  G = G >> 2;
			  B = B >> 2;
			  if(R>255) R = 255;
              		  if(G>255) G = 255;
			  if(B>255) B = 255;

			  *(pbmpdata + (tLines + i * 3 + 2)) = (BYTE)R;
			  *(pbmpdata + (tLines + i * 3 + 1)) = (BYTE)G;
			  *(pbmpdata + (tLines + i * 3))     = (BYTE)B;
			
		  }//end for
          
   }//end for

//////////////////////////////////////////////////////////////////
   if(pwRGB!=NULL){
     free(pwRGB);
   }

   
   return;

}

void CFilter::DataProcessor(LPSTR lpSource, DS_SCANMODE ScanMode, DWORD ScanLines, DWORD BytesPerLine)
{
	LPSTR	tmpPtr=lpSource;
	DWORD	row,index,rgbindex;
	DWORD   ScanPixles;	

	switch ( ScanMode ) {
		case DSSM_GRAY:
			ScanPixles = BytesPerLine;
			memcpy(m_lpBuffer1, tmpPtr, BytesPerLine);	
			for( row=0;row<ScanLines;row++ ){
				memcpy(m_lpBuffer2, tmpPtr, BytesPerLine);	
				for( index=0;index<ScanPixles;index++ ){
					if(row!=(ScanLines-1))
					{
						if(index==0)
						{
							ImageMatrix[0][0]=ImageMatrix[0][1]=(BYTE)*(m_lpBuffer1+index);
							ImageMatrix[1][0]=ImageMatrix[1][1]=(BYTE)*(m_lpBuffer2+index);
							ImageMatrix[0][2]=(BYTE)*(m_lpBuffer1+index+1);
							ImageMatrix[1][2]=(BYTE)*(m_lpBuffer2+index+1);
							ImageMatrix[2][0]=ImageMatrix[2][1]=(BYTE)*(tmpPtr+index+BytesPerLine);
							ImageMatrix[2][2]=(BYTE)*(tmpPtr+index+BytesPerLine+1);
						}
						else if(index==ScanPixles-1)
						{
							ImageMatrix[0][2]=ImageMatrix[0][1]=(BYTE)*(m_lpBuffer1+index);
							ImageMatrix[1][2]=ImageMatrix[1][1]=(BYTE)*(m_lpBuffer2+index);
							ImageMatrix[0][0]=(BYTE)*(m_lpBuffer1+index-1);
							ImageMatrix[1][0]=(BYTE)*(m_lpBuffer2+index-1);
							ImageMatrix[2][2]=ImageMatrix[2][1]=(BYTE)*(tmpPtr+index+BytesPerLine);
							ImageMatrix[2][0]=(BYTE)*(tmpPtr+index+BytesPerLine-1);
						}
						else
						{
							ImageMatrix[0][0]=(BYTE)*(m_lpBuffer1+index-1);
							ImageMatrix[0][1]=(BYTE)*(m_lpBuffer1+index);
							ImageMatrix[0][2]=(BYTE)*(m_lpBuffer1+index+1);
							ImageMatrix[1][0]=(BYTE)*(m_lpBuffer2+index-1);
							ImageMatrix[1][1]=(BYTE)*(m_lpBuffer2+index);
							ImageMatrix[1][2]=(BYTE)*(m_lpBuffer2+index+1);
							ImageMatrix[2][0]=(BYTE)*(tmpPtr+index+BytesPerLine-1);
							ImageMatrix[2][1]=(BYTE)*(tmpPtr+index+BytesPerLine);
							ImageMatrix[2][2]=(BYTE)*(tmpPtr+index+BytesPerLine+1);
						}
					}
					else
					{
						if(index==0)
						{
							ImageMatrix[0][0]=ImageMatrix[0][1]=(BYTE)*(m_lpBuffer1+index);
							ImageMatrix[1][0]=ImageMatrix[1][1]=(BYTE)*(m_lpBuffer2+index);
							ImageMatrix[0][2]=(BYTE)*(m_lpBuffer1+index+1);
							ImageMatrix[1][2]=(BYTE)*(m_lpBuffer2+index+1);
							ImageMatrix[2][0]=ImageMatrix[2][1]=(BYTE)*(m_lpBuffer2+index);
							ImageMatrix[2][2]=(BYTE)*(m_lpBuffer2+index+1);
						}
						else if(index==ScanPixles-1)
						{
							ImageMatrix[0][2]=ImageMatrix[0][1]=(BYTE)*(m_lpBuffer1+index);
							ImageMatrix[1][2]=ImageMatrix[1][1]=(BYTE)*(m_lpBuffer2+index);
							ImageMatrix[0][0]=(BYTE)*(m_lpBuffer1+index-1);
							ImageMatrix[1][0]=(BYTE)*(m_lpBuffer2+index-1);
							ImageMatrix[2][2]=ImageMatrix[2][1]=(BYTE)*(m_lpBuffer2+index);
							ImageMatrix[2][0]=(BYTE)*(m_lpBuffer2+index-1);
						}
						else
						{
							ImageMatrix[0][0]=(BYTE)*(m_lpBuffer1+index-1);
							ImageMatrix[0][1]=(BYTE)*(m_lpBuffer1+index);
							ImageMatrix[0][2]=(BYTE)*(m_lpBuffer1+index+1);
							ImageMatrix[1][0]=(BYTE)*(m_lpBuffer2+index-1);
							ImageMatrix[1][1]=(BYTE)*(m_lpBuffer2+index);
							ImageMatrix[1][2]=(BYTE)*(m_lpBuffer2+index+1);
							ImageMatrix[2][0]=(BYTE)*(m_lpBuffer2+index-1);
							ImageMatrix[2][1]=(BYTE)*(m_lpBuffer2+index);
							ImageMatrix[2][2]=(BYTE)*(m_lpBuffer2+index+1);
						}
					}
					*(tmpPtr+index)=MatrixCalculate();
				}
				memcpy(m_lpBuffer1,m_lpBuffer2, BytesPerLine);
				tmpPtr += BytesPerLine;
			}
			break;
		case DSSM_COLOR24:
			ScanPixles = BytesPerLine/3;
			memcpy(m_lpBuffer1, tmpPtr, BytesPerLine);	
			for( row=0;row<ScanLines;row++ ){
				memcpy(m_lpBuffer2, tmpPtr, BytesPerLine);	
				for( index=0;index<ScanPixles;index++ ) {
					for(rgbindex=0;rgbindex<3;rgbindex++){
						if(row!=(ScanLines-1))
				 		{
							if(index==0)
							{
								ImageMatrix[0][0]=ImageMatrix[0][1]=(BYTE)*(m_lpBuffer1+(index*3+rgbindex));
								ImageMatrix[1][0]=ImageMatrix[1][1]=(BYTE)*(m_lpBuffer2+(index*3+rgbindex));
								ImageMatrix[0][2]=(BYTE)*(m_lpBuffer1+(index*3+rgbindex)+3);
								ImageMatrix[1][2]=(BYTE)*(m_lpBuffer2+(index*3+rgbindex)+3);
								ImageMatrix[2][0]=ImageMatrix[2][1]=(BYTE)*(tmpPtr+(index*3+rgbindex)+BytesPerLine);
								ImageMatrix[2][2]=(BYTE)*(tmpPtr+(index*3+rgbindex)+BytesPerLine+3); // gordon 98.8.6
							}
							else if(index==ScanPixles-1)
							{
								ImageMatrix[0][2]=ImageMatrix[0][1]=(BYTE)*(m_lpBuffer1+(index*3+rgbindex));
								ImageMatrix[1][2]=ImageMatrix[1][1]=(BYTE)*(m_lpBuffer2+(index*3+rgbindex));
								ImageMatrix[0][0]=(BYTE)*(m_lpBuffer1+(index*3+rgbindex)-3);
								ImageMatrix[1][0]=(BYTE)*(m_lpBuffer2+(index*3+rgbindex)-3);
								ImageMatrix[2][2]=ImageMatrix[2][1]=(BYTE)*(tmpPtr+(index*3+rgbindex)+BytesPerLine);
								ImageMatrix[2][0]=(BYTE)*(tmpPtr+(index*3+rgbindex)+BytesPerLine-3);
							}
							else
							{
								ImageMatrix[0][0]=(BYTE)*(m_lpBuffer1+(index*3+rgbindex)-3);
								ImageMatrix[0][1]=(BYTE)*(m_lpBuffer1+(index*3+rgbindex));
								ImageMatrix[0][2]=(BYTE)*(m_lpBuffer1+(index*3+rgbindex)+3);
								ImageMatrix[1][0]=(BYTE)*(m_lpBuffer2+(index*3+rgbindex)-3);
								ImageMatrix[1][1]=(BYTE)*(m_lpBuffer2+(index*3+rgbindex));
								ImageMatrix[1][2]=(BYTE)*(m_lpBuffer2+(index*3+rgbindex)+3);
								ImageMatrix[2][0]=(BYTE)*(tmpPtr+(index*3+rgbindex)+BytesPerLine-3);
								ImageMatrix[2][1]=(BYTE)*(tmpPtr+(index*3+rgbindex)+BytesPerLine);
								ImageMatrix[2][2]=(BYTE)*(tmpPtr+(index*3+rgbindex)+BytesPerLine+3);
							}
						}
						else
						{
							if(index==0)
							{
								ImageMatrix[0][0]=ImageMatrix[0][1]=(BYTE)*(m_lpBuffer1+(index*3+rgbindex));
								ImageMatrix[1][0]=ImageMatrix[1][1]=(BYTE)*(m_lpBuffer2+(index*3+rgbindex));
								ImageMatrix[0][2]=(BYTE)*(m_lpBuffer1+(index*3+rgbindex)+3);
								ImageMatrix[1][2]=(BYTE)*(m_lpBuffer2+(index*3+rgbindex)+3);
								ImageMatrix[2][0]=ImageMatrix[2][1]=(BYTE)*(m_lpBuffer2+(index*3+rgbindex));
								ImageMatrix[2][2]=(BYTE)*(m_lpBuffer2+(index*3+rgbindex)+3);
							}
							else if(index==ScanPixles-1)
							{
								ImageMatrix[0][2]=ImageMatrix[0][1]=(BYTE)*(m_lpBuffer1+(index*3+rgbindex));
								ImageMatrix[1][2]=ImageMatrix[1][1]=(BYTE)*(m_lpBuffer2+(index*3+rgbindex));
								ImageMatrix[0][0]=(BYTE)*(m_lpBuffer1+(index*3+rgbindex)-3);
								ImageMatrix[1][0]=(BYTE)*(m_lpBuffer2+(index*3+rgbindex)-3);
								ImageMatrix[2][2]=ImageMatrix[2][1]=(BYTE)*(m_lpBuffer2+(index*3+rgbindex));
								ImageMatrix[2][0]=(BYTE)*(m_lpBuffer2+(index*3+rgbindex)-3);
							}
							else
							{
								ImageMatrix[0][0]=(BYTE)*(m_lpBuffer1+(index*3+rgbindex)-3);
								ImageMatrix[0][1]=(BYTE)*(m_lpBuffer1+(index*3+rgbindex));
								ImageMatrix[0][2]=(BYTE)*(m_lpBuffer1+(index*3+rgbindex)+3);
								ImageMatrix[1][0]=(BYTE)*(m_lpBuffer2+(index*3+rgbindex)-3);
								ImageMatrix[1][1]=(BYTE)*(m_lpBuffer2+(index*3+rgbindex));
								ImageMatrix[1][2]=(BYTE)*(m_lpBuffer2+(index*3+rgbindex)+3);
								ImageMatrix[2][0]=(BYTE)*(m_lpBuffer2+(index*3+rgbindex)-3);
								ImageMatrix[2][1]=(BYTE)*(m_lpBuffer2+(index*3+rgbindex));
								ImageMatrix[2][2]=(BYTE)*(m_lpBuffer2+(index*3+rgbindex)+3);
							}
						}
						*(tmpPtr+(index*3+rgbindex))=MatrixCalculate();
					}
				}
				memcpy(m_lpBuffer1,m_lpBuffer2, BytesPerLine);
				tmpPtr += BytesPerLine;
			}
			break;
		default:
			break;
	}
}
