/* BearPaw1200CU panel

   Copyright (C) 2001 Mustek. 
   Maintained by Tom Wang and Alex Lin
   <tom.wang@mustek.com.tw, alex_lin@mustek.com.tw>

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston,
   MA 02111-1307, USA.
*/


#ifndef _CLASS_FRACTION_HEADER_
#define _CLASS_FRACTION_HEADER_

#include "BaseTypedef.h"

class CFraction
{
public:
	CFraction();
	CFraction(long);
	CFraction(long,long);
	CFraction(long,long,long);
	~CFraction();
public:
	void Reduction();
	inline void SetValue(const long,const long,bool isReduction = false);
	inline long GetDenominator();
	inline long GetNumerator();
	inline double Value();
	inline void AutoReduction(bool);
	inline void operator-=(const long);
	inline void operator+=(const long);
	inline void operator+=(const CFraction);
	inline void operator--(void);	//Prefix operator like --tmp
	inline void operator--(int);	//Postfix operator like tmp--,int is a dummy argument
	inline bool operator>=(const long);
	inline bool operator>=(const CFraction);
private:
	long m_lDenominator;
	long m_lNumerator;
	bool m_isAutoReduction;
};

inline void CFraction::SetValue(const long lNumerator,const long lDenominator,bool isReduction)
{
	m_lDenominator = lDenominator;
	m_lNumerator = lNumerator;
	if(isReduction) Reduction();
}
inline long CFraction::GetDenominator()
{
	return m_lDenominator;
}
inline long CFraction::GetNumerator()
{
	return m_lNumerator;
}
inline double CFraction::Value()
{
	return (double)m_lNumerator/m_lDenominator;
}
inline void CFraction::AutoReduction(bool isEnable)
{
	m_isAutoReduction = isEnable;
	if(m_isAutoReduction) Reduction();
}
inline void CFraction::operator-=(const long lSubtrahend)
{
	m_lNumerator -= m_lDenominator * lSubtrahend;
	if(m_isAutoReduction) Reduction();
}
inline void CFraction::operator+=(const long lAddend)
{
	m_lNumerator += m_lDenominator * lAddend;
	if(m_isAutoReduction) Reduction();
}
inline void CFraction::operator+=(const CFraction frAddend) //unfinish
{
	if(frAddend.m_lDenominator)
		m_lNumerator += frAddend.m_lNumerator;
	else
	{	//Unfinish
	
	}
	if(m_isAutoReduction) Reduction();
}
inline void CFraction::operator--(void)
{
	m_lNumerator -= m_lDenominator;
	if(m_isAutoReduction) Reduction();
}
inline void CFraction::operator--(int)
{
	m_lNumerator -= m_lDenominator;
	if(m_isAutoReduction) Reduction();
}

inline bool CFraction::operator>=(const long lComparison)
{
	return	(m_lNumerator<lComparison*m_lDenominator)?false:true;
}
inline bool CFraction::operator>=(const CFraction frComparison)
{
	if(m_lDenominator == frComparison.m_lDenominator)
		return (m_lNumerator<frComparison.m_lNumerator)?false:true;
	else	//unfinish
		return false; //(m_lNumerator<frComparison*m_lDenominator)?false:true;
}
#endif
