/* BearPaw1200CU panel

   Copyright (C) 2001 Mustek. 
   Maintained by Tom Wang and Alex Lin
   <tom.wang@mustek.com.tw, alex_lin@mustek.com.tw>

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston,
   MA 02111-1307, USA.
*/


#include "ScanModule.h"
static CScanModuleGlobal theGlobal;

/*********************************************************/
/*							 */
/* functions that are exported to the TWAIN Module	 */
/*							 */
/*********************************************************/
DS_RETURNCODE GetDeviceInfo(bool* pisPaperLoad)
{
	if (theGlobal.m_bDSOpen == FALSE) return DSRC_DSNOTOPENED;

	DEVICESTATUS deviceStatus;
	if (GetDeviceStatus(&deviceStatus)) {
		if (deviceStatus.sbPaperLoadedInADF ==SB_TRUE)
			*pisPaperLoad = true;
		else 
			*pisPaperLoad = false;
		return DSRC_NOERROR;
	} else {
		return DSRC_ERROR;
	}

	return DSRC_NOERROR;
}


DS_RETURNCODE OpenDS (LPDS_IMAGEDESCRIPTOR pDefaultSetting)
{
	// check if the DS is already opened.
	
	if (theGlobal.m_bDSOpen) return DSRC_ERROR;	
	theGlobal.Attach();
	// check if the size of structures are correct...
	if (pDefaultSetting->u32Size != sizeof(DS_IMAGEDESCRIPTOR) ) return DSRC_INVALIDPARAM;
	// load ini file here
	
	if (!SpicallOpen()) {
		theGlobal.Detach();
		return DSRC_ERROR;
	}
	LAMPCONTROL control;
	control.sbTurnOn = SB_TRUE;
	if (!PowerControl(&control)) {
		theGlobal.Detach();
		return DSRC_ERROR;
	}
	if (control.ecErrorCode != EC_OK) {
		theGlobal.Detach();
		return DSRC_ERROR;
	}
	COMMON common;
	if (!CarriageHome(&common)) {
		theGlobal.Detach();
		return DSRC_ERROR;
	}
	if (common.ecErrorCode != EC_OK) {
		theGlobal.Detach();
		return DSRC_ERROR;
	}

	DEVICEINFO deviceInfo;
	if (!GetDeviceInfo(&deviceInfo)) {
		theGlobal.Detach();
		return DSRC_ERROR;
	}
	if (deviceInfo.ecErrorCode != EC_OK) {
		theGlobal.Detach();
		return DSRC_ERROR;
	}
	theGlobal.m_bDownloadGamma = deviceInfo.sbDownLoadGamma;

	theGlobal.m_nADFBrightOffset = 600;
	theGlobal.m_nADFBWBrightOffset = 300;
	theGlobal.m_nADFContrastOffset = 300;
	theGlobal.m_fADFGammaOffset = 0.8f;
	
	SM_PRIVATE* myPrivate;
	
	myPrivate = (SM_PRIVATE*) &(pDefaultSetting->cScanModulePrivate);
	pDefaultSetting->u32Size = sizeof(DS_IMAGEDESCRIPTOR);
	pDefaultSetting->u32Resolution = 100;
	pDefaultSetting->u32XOrigin = 0;
	pDefaultSetting->u32YOrigin = 0;
	pDefaultSetting->u32ImageWidth = 850;
	pDefaultSetting->u32ImageHeight= 1170;
	pDefaultSetting->smScanMode = DSSM_COLOR24;
	pDefaultSetting->isFromADF = false;
	pDefaultSetting->lsLightSource = DSLS_REFLECTIVE;
	pDefaultSetting->u32Scaling = 100;
	
	pDefaultSetting->ehBrightness.i16Red = 0;
	pDefaultSetting->ehBrightness.i16Green = 0;
	pDefaultSetting->ehBrightness.i16Blue = 0;
	pDefaultSetting->ehBrightness.i16Gray = 0;
	pDefaultSetting->ehBrightness.u32Size = sizeof (DS_ENHANCE);

	pDefaultSetting->ehContrast.i16Red = 0;
	pDefaultSetting->ehContrast.i16Green = 0;
	pDefaultSetting->ehContrast.i16Blue = 0;
	pDefaultSetting->ehContrast.i16Gray = 0;
	pDefaultSetting->ehContrast.u32Size = sizeof (DS_ENHANCE);
	
	pDefaultSetting->gmGamma.dRed = 1.0f;
	pDefaultSetting->gmGamma.dGreen = 1.0f;
	pDefaultSetting->gmGamma.dBlue = 1.0f;
	pDefaultSetting->gmGamma.dGray = 1.0f;
	pDefaultSetting->gmGamma.u32Size = sizeof(DS_GAMMA);
	
	pDefaultSetting->pfPixelFlavor = DSPF_BLACKISZERO;
	pDefaultSetting->cpCompress = DSCP_NONE;
	pDefaultSetting->isColorBalance = 0;
	
	myPrivate->dsDescreen = DSDE_NONE;
	myPrivate->ftFilter = DSFT_NONE;
	myPrivate->isColorMatch = 0;
	theGlobal.m_bDSOpen = true;

	return DSRC_NOERROR;
}

DS_RETURNCODE CloseDS (void)
{
	if (! theGlobal.m_bDSOpen) return DSRC_DSNOTOPENED;
	if ( theGlobal.m_bScanning ) return DSRC_SCANNING;

	if (!SpicallClose()) {
		return DSRC_ERROR;
	}

	theGlobal.m_bDSOpen = false;
	theGlobal.Detach();
	return DSRC_NOERROR;
}


DS_RETURNCODE QueryImage(LPDS_IMAGEDESCRIPTOR pQuerySetting)
{
	// Error checking...
	if ( !theGlobal.m_bDSOpen ) return DSRC_DSNOTOPENED;
	if ( pQuerySetting->u32Size != sizeof(DS_IMAGEDESCRIPTOR)) return DSRC_INVALIDPARAM;
	if ( theGlobal.m_bScanning) return DSRC_SCANNING;

	memcpy(&theGlobal.m_ScanJob,pQuerySetting,sizeof(DS_IMAGEDESCRIPTOR));		

	theGlobal.m_u32DestXRes = pQuerySetting->u32Resolution*pQuerySetting->u32Scaling/100;
	theGlobal.m_u32DestYRes = theGlobal.m_u32DestXRes;	
	// initialize the scan line count
	SETPARAMETERS setParameter;
	setParameter.fmArea.x1 = pQuerySetting->u32XOrigin*300/pQuerySetting->u32Resolution;
	setParameter.fmArea.x2 = (pQuerySetting->u32XOrigin + pQuerySetting->u32ImageWidth)*300/pQuerySetting->u32Resolution +
		(((pQuerySetting->u32XOrigin + pQuerySetting->u32ImageWidth)*300%pQuerySetting->u32Resolution > 0) ? 1 : 0);
	setParameter.fmArea.y1 = pQuerySetting->u32YOrigin*300/pQuerySetting->u32Resolution;
	setParameter.fmArea.y2 = (pQuerySetting->u32YOrigin + pQuerySetting->u32ImageHeight)*300/pQuerySetting->u32Resolution +
		(((pQuerySetting->u32YOrigin + pQuerySetting->u32ImageHeight)*300%pQuerySetting->u32Resolution > 0 ) ? 1 : 0);

	// make sure x1, x2, y1, y2 in scanner scan area
	//if (setParameter.fmArea.x1<0) setParameter.fmArea.x1=0;
	//if (setParameter.fmArea.x2>2550) setParameter.fmArea.x2=2550;
	//if (setParameter.fmArea.y1<0) setParameter.fmArea.y1=0;
	//if (setParameter.fmArea.y2>3507) setParameter.fmArea.y2=3507;	
	
	if (theGlobal.m_u32DestXRes < 50) {
		setParameter.wTargetDPI = 50;
	} else {
		setParameter.wTargetDPI = theGlobal.m_u32DestXRes;
	}	
	switch (pQuerySetting->smScanMode) {
		case DSSM_COLOR24 :
			setParameter.smScanMode = SM_RGB24;
			break;
		case DSSM_COLOR30:
			setParameter.smScanMode = SM_RGB30;
			break;
		case DSSM_COLOR36:
			setParameter.smScanMode = SM_RGB36;
			break;
		case DSSM_COLOR48:
			setParameter.smScanMode = SM_RGB48;
			break;
		case DSSM_GRAY:
			setParameter.smScanMode = SM_GRAY;
			break;
		case DSSM_LINEART:
			setParameter.smScanMode =SM_TEXT;
			break;
	}
		
	GAMMAINFO gammainfo;
	if (theGlobal.m_bDownloadGamma) {
		gammainfo.smScanMode = setParameter.smScanMode;
		GetGammaInfo(&gammainfo);
		if (gammainfo.ecErrorCode == EC_OK) {		
			theGlobal.CreateGammaTable( gammainfo.wInputGammaBits, 
				gammainfo.wOutputGammaBits,
				pQuerySetting->ehBrightness,
				pQuerySetting->ehContrast,
				pQuerySetting->gmGamma,
				pQuerySetting->smScanMode);
		}
	} else {
		// not down load gamma, process gamma by ourself
		int gamma_bits;
		switch (pQuerySetting->smScanMode) {
			case DSSM_COLOR24 :
				gamma_bits = 8;
				break;
			case DSSM_COLOR30:
				gamma_bits = 10;
				break;
			case DSSM_COLOR36:
				gamma_bits = 12;
				break;
			case DSSM_COLOR48:
				gamma_bits = 16;
				break;
			case DSSM_GRAY:
				gamma_bits = 8;
				break;
			case DSSM_LINEART:
				gamma_bits = 0;
				break;
		}
		if (gamma_bits > 0) {
			theGlobal.CreateGammaTable( gamma_bits, gamma_bits,
				pQuerySetting->ehBrightness,
				pQuerySetting->ehContrast,
				pQuerySetting->gmGamma,
				pQuerySetting->smScanMode);
		}
	}


	if (pQuerySetting->smScanMode == DSSM_LINEART) {
		setParameter.wLinearThreshold =
			255 - ((long)(pQuerySetting->ehBrightness.i16Gray+1000))*255/2000;
	} else {
		setParameter.wLinearThreshold = 128;
	}
	if (pQuerySetting->pfPixelFlavor == DSPF_BLACKISZERO) {
		setParameter.pfPixelFlavor = PF_BlackIs0;
	} else {
		setParameter.pfPixelFlavor = PF_WhiteIs0;
	}
	if (pQuerySetting->isFromADF)
		setParameter.ssScanSource = SS_ADF;
	else 
		switch (pQuerySetting->lsLightSource) {
			case DSLS_REFLECTIVE:
				setParameter.ssScanSource = SS_Reflective;
				break;
			case DSLS_POSITIVE:
				setParameter.ssScanSource = SS_Positive;
				break;
			case DSLS_NEGATIVE:
				setParameter.ssScanSource = SS_Negative;
				break;
			default:
				setParameter.ssScanSource = SS_Positive;
		}
		// if isFromADF == true, scan source set to ADF
		if (pQuerySetting->isFromADF)
			setParameter.ssScanSource = SS_ADF;
	if (theGlobal.m_bDownloadGamma == true) {
		if (setParameter.smScanMode == SM_TEXT)
			setParameter.pGammaTable = NULL;
		else 
			setParameter.pGammaTable = theGlobal.m_pEnhance;
	} else 
		setParameter.pGammaTable = NULL;	
		
	// set parameter
	if (!SetParameters(&setParameter)) {
		ShowUp("SetParameter fail.\n");
		return DSRC_ERROR;
	}
	if (setParameter.ecErrorCode != EC_OK) {
		ShowUp("SetParameter fail.\n");
		return DSRC_ERROR;
	}
	
	// get parameter
	GETPARAMETERS getParameter;
	if (!GetParameters(&getParameter)||(getParameter.ecErrorCode != EC_OK))
	{
		if(setParameter.wTargetDPI < 50 )
		{
			setParameter.wTargetDPI = 50;
			if (!SetParameters(&setParameter)||(setParameter.ecErrorCode != EC_OK)) 
			{
				ShowUp("GetParameter fail.\n");
				return DSRC_ERROR;
			}
			if (!GetParameters(&getParameter)||(getParameter.ecErrorCode != EC_OK)) 
			{
				ShowUp("GetParameter fail.\n");
				return DSRC_ERROR;
			}
		}
		else{
			ShowUp("GetParameter fail.\n");
			return DSRC_ERROR;
		}
	}
	ShowUp("getParameter.wSourceXDPI == %d.\n",getParameter.wSourceXDPI);
	//set the source width & height for reduction
	theGlobal.m_dwSrcLength = getParameter.dwLength;
	theGlobal.m_dwLineByte = getParameter.dwLineByteWidth;
	
	
	// set the source & dest resolution for interpolation or reduction
	theGlobal.m_u32SrcXRes = getParameter.wSourceXDPI;
	theGlobal.m_u32SrcYRes = getParameter.wSourceYDPI;

	if (theGlobal.m_u32DestXRes == getParameter.wSourceXDPI &&
		theGlobal.m_u32DestYRes == getParameter.wSourceYDPI) 
		theGlobal.m_rsResize = SMRS_NONE;
	else if(theGlobal.m_u32DestXRes > getParameter.wSourceXDPI ||
		theGlobal.m_u32DestYRes > getParameter.wSourceYDPI) 
		theGlobal.m_rsResize = SMRS_INTERPOLATION;
	else if(theGlobal.m_u32DestXRes < getParameter.wSourceXDPI ||
		theGlobal.m_u32DestYRes < getParameter.wSourceYDPI) 
		theGlobal.m_rsResize = SMRS_REDUCTION;
	else
		return DSRC_ERROR;
		
	if (!theGlobal.m_pHRuler)
		theGlobal.m_pHRuler = new CFastRuler;
	if (!theGlobal.m_pVRuler)
		theGlobal.m_pVRuler = new CFastRuler;
		
	theGlobal.m_nImageWidth = pQuerySetting->u32ImageWidth*pQuerySetting->u32Scaling / 100;
	theGlobal.m_nImageHeight = pQuerySetting->u32ImageHeight*pQuerySetting->u32Scaling / 100;
	
	DWORD PixelPerLine;
	switch (pQuerySetting->smScanMode) {
		case DSSM_COLOR24 :
			PixelPerLine = getParameter.dwLineByteWidth / 3;
			break;
		case DSSM_COLOR30:
			PixelPerLine = getParameter.dwLineByteWidth / 6;
			break;
		case DSSM_COLOR36:
			PixelPerLine = getParameter.dwLineByteWidth / 6;
			break;
		case DSSM_COLOR48:
			PixelPerLine = getParameter.dwLineByteWidth / 6;
			break;
		case DSSM_GRAY:
			PixelPerLine = getParameter.dwLineByteWidth;
			break;
		case DSSM_LINEART:
			PixelPerLine = getParameter.dwLineByteWidth * 8;
			break;
	}
	
	theGlobal.m_nHBound = theGlobal.m_pHRuler->SetRange(
		theGlobal.m_nImageWidth,
		PixelPerLine);
		
	theGlobal.m_nVBound = theGlobal.m_pVRuler->SetRange(
		theGlobal.m_nImageHeight,
		getParameter.dwLength);
	
	theGlobal.m_nBound2 = theGlobal.m_nVBound * theGlobal.m_nHBound;
	theGlobal.m_nBound = max (theGlobal.m_nVBound, theGlobal.m_nHBound);
	theGlobal.m_nHalfBound = (theGlobal.m_nBound+1)/2;
	// start scan
	COMMON common;
	if (!StartScan(&common)) {
		ShowUp("StartScan fail.\n");
		return DSRC_ERROR;
	}
	if (common.ecErrorCode != EC_OK) {
		ShowUp("StartScan fail.\n");
		return DSRC_ERROR;
	}
//	theGlobal.m_dwScanLine = 0;
	theGlobal.m_nLinesTransfered = 0;
	theGlobal.m_pSourceBuffer = new CSourceBuffer(&theGlobal);
	theGlobal.m_pSourceBuffer->InitializeBuffer(theGlobal.m_dwLineByte,getParameter.dwLength);
	theGlobal.m_bScanning = TRUE;

	return DSRC_NOERROR;
}

DS_RETURNCODE QueryBlock(LPDS_BLOCKDESCRIPTOR pImageBlock)
{
	// error check.
	if (theGlobal.m_bDSOpen == FALSE) {
		return DSRC_DSNOTOPENED;
	}
	if (theGlobal.m_bScanning == FALSE) {
		 return DSRC_NOQUERYIMAGE;
	}
	if (pImageBlock->u32Size != sizeof (DS_BLOCKDESCRIPTOR) ) {
		return DSRC_INVALIDPARAM;
	}
	if (theGlobal.m_pSourceBuffer->DataLineCount() == 0) // < pImageBlock->u16BufferLines)
		theGlobal.m_pSourceBuffer->ReadImageToBuffer();
	
	switch(theGlobal.m_rsResize)
	{
	case SMRS_NONE:	// source dpi == target dpi ....
//	if (!theGlobal.m_bInterpProcess) {
		int i;
		for ( i = 0; i < pImageBlock->u16BufferLines; i++) {
			bool success;
			char* buf;
			int size;			
			buf = new char[theGlobal.m_dwLineByte];
			success = theGlobal.m_pSourceBuffer->GetOneLine( buf);
			//			(char*) (pImageBlock->pBuffer)+theGlobal.m_dwLineByte*i);
			if (!success) {
				delete buf;
				break;
			}
			switch (theGlobal.m_ScanJob.smScanMode) {
				case DSSM_LINEART:
					size = theGlobal.m_nImageWidth / 8 + (theGlobal.m_nImageWidth % 8 ? 1 : 0);
					break;
				case DSSM_GRAY:
					size = theGlobal.m_nImageWidth ;
					break;
				case DSSM_COLOR24:
					size = theGlobal.m_nImageWidth * 3;
					break;
				default:
					size = theGlobal.m_nImageWidth * 6;
					break;
			}

			//size = theGlobal.m_dwLineByte;
			memcpy((char*) (pImageBlock->pBuffer)+size*i,buf,size);
			delete buf;		
		}
		pImageBlock->u16XferLines = i;
		theGlobal.m_nLinesTransfered += i;

		if (theGlobal.m_nLinesTransfered < theGlobal.m_nImageHeight) 
			return DSRC_SCANMORE;
		else
			return DSRC_SCANDONE;
		break;


	case SMRS_INTERPOLATION:	// source dpi < target dpi ....	
		switch (theGlobal.m_ScanJob.smScanMode) {
			case DSSM_LINEART:
				theGlobal.BltTextLines(pImageBlock);
				break;
			case DSSM_GRAY:
				theGlobal.BltGrayLines(pImageBlock);
				break;
			case DSSM_COLOR24:
				theGlobal.BltRGBLines(pImageBlock);
				break;
			default:
				break;
		}
		theGlobal.m_nLinesTransfered += pImageBlock->u16XferLines;
			
		if (theGlobal.m_nLinesTransfered < theGlobal.m_nImageHeight) 
			return DSRC_SCANMORE;
		else
			return DSRC_SCANDONE;
		break;	
	case SMRS_REDUCTION:	// source dpi > target dpi ....
		switch (theGlobal.m_ScanJob.smScanMode) {
			case DSSM_LINEART:
				theGlobal.DecTextLines(pImageBlock);
				break;
			case DSSM_GRAY:
				theGlobal.DecGrayLines(pImageBlock);
				break;
			case DSSM_COLOR24:
				theGlobal.DecRGBLines(pImageBlock);
				break;
			default:
				break;
		}
		theGlobal.m_nLinesTransfered += pImageBlock->u16XferLines;	
#ifdef PROGRESS_BAR
	theGlobal.m_pProgress->SetStat(theGlobal.m_nLinesTransfered);
#endif	
		if (theGlobal.m_nLinesTransfered < theGlobal.m_nImageHeight) 
			return DSRC_SCANMORE;
		else
			return DSRC_SCANDONE;
		break;
	default:
		return DSRC_ERROR;
		break;
			
	}
	return DSRC_SCANDONE;
	
}

DS_RETURNCODE StopQuery()
{
	//* Debug Code
	CScanModuleGlobal* pTheApp = &theGlobal;
	//*/

	if (theGlobal.m_bDSOpen == FALSE) return DSRC_DSNOTOPENED;

#ifdef PROGRESS_BAR
	if (theGlobal.m_pProgress != NULL) {
		delete theGlobal.m_pProgress;
		theGlobal.m_pProgress = NULL;
	}
#endif


	COMMON common;
	if (theGlobal.m_pSourceBuffer) {
		delete theGlobal.m_pSourceBuffer;
		theGlobal.m_pSourceBuffer = NULL;
	}
	if (theGlobal.m_pUpperSrcLine) {
		free(theGlobal.m_pUpperSrcLine);
		theGlobal.m_pUpperSrcLine = NULL;
	}
	if (theGlobal.m_pLowerSrcLine) {
		free(theGlobal.m_pLowerSrcLine);
		theGlobal.m_pLowerSrcLine = NULL;
	}
	if (theGlobal.m_bScanning) {
		if (!StopScan(&common)) {
			return DSRC_ERROR;
		}
		if (common.ecErrorCode != EC_OK) {
			return DSRC_ERROR;
		}		
	} 
	CarriageHome(&common);			
	return DSRC_SCANDONE;
	
	return DSRC_ERROR;
}

