/* BearPaw1200CU panel

   Copyright (C) 2001 Mustek. 
   Maintained by Tom Wang and Alex Lin
   <tom.wang@mustek.com.tw, alex_lin@mustek.com.tw>

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston,
   MA 02111-1307, USA.
*/


#include <stdlib.h>
#include <math.h>
#include "ScanModuleGlobal.h"
#include "SourceBuffer.h"
#include "Fraction.h"
#include "common.h"

CScanModuleGlobal::CScanModuleGlobal()
{
    // constructor is not executed... all initialization should be done in attach
}

CScanModuleGlobal::~CScanModuleGlobal()
{

}

void CScanModuleGlobal::Attach()
{
	// public members initialization
	m_bDSOpen = FALSE;
	m_bScanning = FALSE;
	m_pEnhance = NULL;
	m_pSourceBuffer = NULL;
	m_pLowerSrcLine = NULL;
	m_pUpperSrcLine = NULL;
	
	m_rsResize = SMRS_NONE;
	m_bDownSampleProcess = false;
	m_bDownloadGamma = false;
	// private members initialization
	m_bGetGammaMaps = false;
}

void CScanModuleGlobal::Detach()
{
	if (m_pEnhance) {
		free ( m_pEnhance);
		m_pEnhance = NULL;
	}
	if (m_pHRuler) {
		delete m_pHRuler;
		m_pHRuler = NULL;
	}
	if (m_pVRuler) {
		delete m_pVRuler;
		m_pVRuler = NULL;
	}
	if (m_pSourceBuffer) {
		delete m_pSourceBuffer;
		m_pSourceBuffer = NULL;
	}
	if (m_pLowerSrcLine != NULL) {
		free (m_pLowerSrcLine);
		m_pLowerSrcLine = NULL;
	}
	if (m_pUpperSrcLine != NULL) {
		free (m_pUpperSrcLine);
		m_pUpperSrcLine = NULL;
	}
}

void CScanModuleGlobal::GenerateGammaCode(VRGBE Color, int input_level, int output_level,
	WORD* lpTab)
{	
	WORD LUT[32];
	float *lpCode;
	DWORD i,j,iStart,iEnd;
	WORD x0, y0;
	float fOffset, ftemp;
	BOOL bIsEnd,bGamMap = FALSE;
	
	lpCode = NULL;
	
	x0 = (WORD)(input_level-1);
	y0 = (WORD)(output_level-1);
	
	if (20 < y0 && y0 < 65536 && 20 < x0 && x0 < 65536
		&& 1 <= Color.Gam && Color.Gam <= 79)
//		&& -127 <= Color.Con && Color.Con <= 127
//		&& -127 <= Color.Bri && Color.Bri <= 127) 
	{
		if (m_bGetGammaMaps)
			;//bGamMap = GetPSPGammaMaps(Color.Gam,LUT);
		else
			bGamMap = FALSE;
		
		lpCode = (float*)malloc(sizeof(float)*input_level);
		// Contrast here...
		if (Color.Con >= 0) {
			if (Color.Con > 0)
				fOffset = ((float)(Color.Con+1) * input_level / 256) -1;
			else
				fOffset = 0;
			iStart = (DWORD)fOffset;
			iEnd = (DWORD)(x0 - (DWORD)fOffset);
			
			for (i = 0; i <= x0; i++) {
				if (i <= iStart) 
					lpCode[i] = 0;
				else if ((i>iStart) && (i<iEnd))
					lpCode[i] = (float) ((float)x0* (i-fOffset) / (x0 - 2 *fOffset));
				else if (i >= iEnd)
					lpCode[i] = (float)x0;
			}
		} else {
			fOffset = ((float)(Color.Con-1) * input_level  / (-256)) -1;
			iStart = 0;
			iEnd = x0;
			if (Color.Con == -127)
				for (i =0; i <= x0; i++) lpCode[i] = (float) fOffset;
			else 
				for (i =0; i <= x0; i++)
					lpCode[i] = (float)(fOffset +(float)i * (x0-2*fOffset) / x0);
		} // end of contrast 
		// apply gamma factor here in the future... by Tom
		
		// apply brightness here
		if (Color.Bri > 0)
			fOffset = ((float)(Color.Bri+1) * input_level / 256) - 1;
		else if (Color.Bri < 0)
			fOffset = ((float)(Color.Bri-1) * input_level / 256) + 1;
		else
			fOffset = 0;
			
		bIsEnd = false;
		for ( i =0; i <= x0; i++ ) {
			ftemp = fOffset + lpCode[i];
			if (ftemp <1) {
				lpCode[i] = 0;
				iStart = i+1;
			} else if (ftemp >= x0) {
				lpCode[i] = (float) x0;
				if (!bIsEnd) {
					if ( i > 1) {
						iEnd = (i-1);
						bIsEnd = true;
					}
				}
			} else 
				lpCode[i] = (float)ftemp;
		}
		// apply gamma factor here
		fOffset = (float) Color.Gam;
		for ( i = iStart; i <= iEnd; i++) {
			if (iEnd > iStart) {
				ftemp = lpCode[iStart] + pow( ((double) i - iStart)/(iEnd - iStart),
					10.0 / fOffset) * (lpCode[iEnd] - lpCode[iStart]);
				lpCode[i] = ftemp;
			}
		}
		int   inBits,OutBits,itempBits;
		DWORD modnum;
		BOOL  InGreatEQOut;

		inBits = 0;
		modnum = input_level;

		while(modnum >= 2) {
			modnum /= 2;
			inBits++;  
		} 

		OutBits = 0;
		modnum = output_level;
		while(modnum >= 2){
			modnum /= 2;
			OutBits++;  
		}
		
		//Get Photoshop gamma mapping
		float  fvalue, ffoward=1.0, ffowardGam,fGam;
		int    imultiply=0; 
		DWORD  wstep, wdelta,fMax;
		BOOL   bNext=FALSE; 
   
		if(m_bGetGammaMaps){
			if(inBits > 8)
				imultiply = inBits - 8;
			else{
				bGamMap = FALSE;
				for(i=0;i<32;i++)
            	lpCode[i] = (float)LUT[i];
        	}
   		}
		i=1;
		if(bGamMap){
			*(lpCode) = 0.0;
			wdelta = wstep = i << imultiply;
			ffowardGam = 0.0;
			fMax = ((DWORD)LUT[i] << imultiply);
			for(j=1; j<input_level; j++){
				if(j>=wstep){
					i++;
					wstep = i << imultiply;
					ffoward = (float)((i-1) << imultiply);
					ffowardGam += ((float) ((LUT[i-1] - LUT[i-2]) * wdelta));
					fMax = ((DWORD)LUT[i] << imultiply);
					bNext = TRUE;
					if(i > 31) break;
				}
				if(!bNext){
					fvalue = (float) ((LUT[i] - LUT[i-1]) * wdelta);
					fGam = (fvalue  / wdelta) * (j-ffoward+1) + ffowardGam;
					if(fGam <= fMax)
						lpCode[j] = fGam;
					else
						lpCode[j] = (float)fMax;
				} else {
					bNext = FALSE;
					j--;
				}
			}//end for  
		} //end Get Photoshop gamma mapping
		
		if(inBits >= OutBits){ 
			itempBits = inBits - OutBits;
			InGreatEQOut = TRUE;
		} else {
			itempBits = OutBits - inBits;
			InGreatEQOut = FALSE;
		}
		
		WORD wtemp,MaxOutLevel;
		if(InGreatEQOut){
			for(i=0; i< input_level;i++){
				if(lpCode[i] >x0)
					wtemp = x0;
				else
					wtemp = (WORD)lpCode[i];
				wtemp = wtemp >> itempBits;
				if(wtemp <= 0)
					lpTab[i] = 0;
				else if(wtemp >=  y0)
					lpTab[i] =  (WORD)y0;
				else if((wtemp > 0) && (wtemp < y0))
					lpTab[i] = wtemp;
	  		}
   		} else {
   			MaxOutLevel = y0;
   			MaxOutLevel = MaxOutLevel << itempBits;
   			for(i=0; i< input_level;i++){
				if(lpCode[i] >x0)
					wtemp = x0;
				else
					wtemp = (WORD)lpCode[i];
				wtemp = wtemp << itempBits;
				if(wtemp <= 0)
					lpTab[i] = 0;
				else if(wtemp >=  MaxOutLevel)
					lpTab[i] = MaxOutLevel;
				else if((wtemp > 0) && (wtemp < MaxOutLevel))
					lpTab[i] = wtemp;
			}
		}
		if(lpCode != NULL) {
			free(lpCode);
			lpCode = NULL;
		}
		return;
	} // end of legal value
	return;
}

void CScanModuleGlobal::CreateGammaTable(int input_gamma, int output_gamma, DS_ENHANCE brightness, 
	DS_ENHANCE contrast, DS_GAMMA gamma,DS_SCANMODE scanmode)
{
	int input_level,output_level;
	WORD* red;
	WORD* green;
	WORD* blue;
	WORD* gray;
	
	input_level = 1 << (input_gamma);
	output_level = 1 << (output_gamma);
	if (m_pEnhance) free (m_pEnhance);
	m_pEnhance = (WORD*) malloc (3*sizeof(WORD)*input_level);
	if (m_pEnhance == NULL) return;
	gray = red = &m_pEnhance[0];
	green = &m_pEnhance[input_level];
	blue = &m_pEnhance[input_level*2];
	
	VRGBE color;
	switch (scanmode) {
		case DSSM_LINEART:
		case DSSM_GRAY:
			color.Con = (char)((float)contrast.i16Gray / 1000 * 127);
			color.Bri = (char)((float)brightness.i16Gray/1000 * 127);
			color.Gam = (char)(gamma.dGray*10);
			GenerateGammaCode(color, input_level, output_level,gray);
			break;
		default:
			color.Con = (char)((float)contrast.i16Red / 1000 * 127);
			color.Bri = (char)((float)brightness.i16Red / 1000 * 127);
			color.Gam = (char)(gamma.dRed*10);
			GenerateGammaCode(color, input_level, output_level,red);
			
			color.Con = (char)((float)contrast.i16Green / 1000 * 127);
			color.Bri = (char)((float)brightness.i16Green / 1000 * 127);
			color.Gam = (char)(gamma.dGreen*10);
			GenerateGammaCode(color, input_level, output_level,green);
			
			color.Con = (char)((float)contrast.i16Blue / 1000 * 127);
			color.Bri = (char)((float)brightness.i16Blue / 1000 * 127);
			color.Gam = (char)(gamma.dBlue*10);
			GenerateGammaCode(color, input_level, output_level,blue);						
	}
}

void CScanModuleGlobal::BltRGBLines(LPDS_BLOCKDESCRIPTOR pImageBlock)
{
	LONG VRatio;
	LONG HRatio;
	BOOL isChange;
	CScanModuleGlobal* Global= this;	//Add by asion for Trace
	
	double Tmp1,Tmp2,Tmp3,Tmp4;
	WORD LineCount = pImageBlock->u16BufferLines;
	char* lpDestRows = (char*)pImageBlock->pBuffer;
	pImageBlock->u16XferLines = 0;
	if (m_pLowerSrcLine == NULL) {
		m_pLowerSrcLine = (char*) malloc (m_pSourceBuffer->m_nBytesPerLine);
		if (m_pLowerSrcLine) 
			m_pSourceBuffer->GetOneLine(m_pLowerSrcLine);
		else 
			return;
		
		if (m_pUpperSrcLine != NULL) free (m_pUpperSrcLine);
		m_pUpperSrcLine = (char*) malloc(m_pSourceBuffer->m_nBytesPerLine);
		if (m_pUpperSrcLine)
			m_pSourceBuffer->GetOneLine(m_pUpperSrcLine);
		else
			return;
	}
	
	while ((LineCount>0)&&m_pVRuler->GetNext(VRatio,isChange)){
		if (isChange){
			char* pTemp=m_pLowerSrcLine;
			m_pLowerSrcLine=m_pUpperSrcLine;
			m_pUpperSrcLine=pTemp;
			m_pSourceBuffer->GetOneLine(m_pUpperSrcLine);
		}
		char* lpDest=lpDestRows;
		if (VRatio==0){
			m_pHRuler->Reset();
			char* lpLower=m_pLowerSrcLine;
			char* lp01 = m_pLowerSrcLine+m_pSourceBuffer->m_nBytesPerLine-6;
			while (m_pHRuler->GetNextRGB(HRatio,lpLower)){
				if(lpLower>lp01)
                   			lpLower-=3;
				if (HRatio==0){
					*(lpDest++)=BYTE(*(lpLower++));
					*(lpDest++)=BYTE(*(lpLower++));
					*(lpDest++)=BYTE(*lpLower);
					lpLower-=2;
				}
				else{
					LONG HComp=m_nHBound-HRatio;
					*(lpDest++)=BYTE((LONG(BYTE(*(lpLower+3)))*HRatio+LONG(BYTE(*lpLower))*HComp)/m_nHBound);
					lpLower++;
					*(lpDest++)=BYTE((LONG(BYTE(*(lpLower+3)))*HRatio+LONG(BYTE(*lpLower))*HComp)/m_nHBound);
					lpLower++;
					*(lpDest++)=BYTE((LONG(BYTE(*(lpLower+3)))*HRatio+LONG(BYTE(*lpLower))*HComp)/m_nHBound);
					lpLower-=2;
				}
			}
		}
		else{
			m_pHRuler->Reset();
			char* lpLower=m_pLowerSrcLine;
			char* lpUpper=m_pUpperSrcLine;
		    	char* lp01 = m_pLowerSrcLine+m_pSourceBuffer->m_nBytesPerLine-6;
			char* lp02 = m_pUpperSrcLine+m_pSourceBuffer->m_nBytesPerLine-6;
			LONG VComp=m_nVBound-VRatio;
			while (m_pHRuler->GetNextRGB(HRatio,lpLower,lpUpper)){
				if(lpLower>lp01)
			                lpLower-=3;
				if(lpUpper>lp02)
                   			lpUpper-=3;
				if (HRatio==0){
					*(lpDest++)=BYTE((LONG(BYTE(*(lpUpper++)))*(VRatio)+LONG(BYTE(*(lpLower++)))*VComp)/m_nVBound);
					*(lpDest++)=BYTE((LONG(BYTE(*(lpUpper++)))*(VRatio)+LONG(BYTE(*(lpLower++)))*VComp)/m_nVBound);
					*(lpDest++)=BYTE((LONG(BYTE(*(lpUpper++)))*(VRatio)+LONG(BYTE(*(lpLower++)))*VComp)/m_nVBound);
					lpUpper-=3;
					lpLower-=3;
				}
				else{
					
					//R
					LONG HComp=m_nHBound-HRatio;
					Tmp1=BYTE(*(lpUpper+3));
					Tmp1=(double)(Tmp1*VRatio*HRatio)/m_nBound2;
					Tmp2=BYTE(*lpUpper);
					Tmp2=(double)(Tmp2*VRatio*HComp)/m_nBound2;
					Tmp3=BYTE(*(lpLower+3));
					Tmp3=(double)(Tmp3*VComp*HRatio)/m_nBound2;
					Tmp4=BYTE(*lpLower);
					Tmp4=(double)(Tmp4*VComp*HComp)/m_nBound2;
					*(lpDest++)=BYTE(Tmp1+Tmp2+Tmp3+Tmp4);
					
 			                //G
					lpUpper++;
					lpLower++;
					Tmp1=BYTE(*(lpUpper+3));
					Tmp1=(double)(Tmp1*VRatio*HRatio)/m_nBound2;
					Tmp2=BYTE(*lpUpper);
					Tmp2=(double)(Tmp2*VRatio*HComp)/m_nBound2;
					Tmp3=BYTE(*(lpLower+3));
					Tmp3=(double)(Tmp3*VComp*HRatio)/m_nBound2;
					Tmp4=BYTE(*lpLower);
					Tmp4=(double)(Tmp4*VComp*HComp)/m_nBound2;
					*(lpDest++)=BYTE(Tmp1+Tmp2+Tmp3+Tmp4);
					
					//B
					lpUpper++;
					lpLower++;
					Tmp1=BYTE(*(lpUpper+3));
					Tmp1=(double)(Tmp1*VRatio*HRatio)/m_nBound2;
					Tmp2=BYTE(*lpUpper);
					Tmp2=(double)(Tmp2*VRatio*HComp)/m_nBound2;
					Tmp3=BYTE(*(lpLower+3));
					Tmp3=(double)(Tmp3*VComp*HRatio)/m_nBound2;
					Tmp4=BYTE(*lpLower);
					Tmp4=(double)(Tmp4*VComp*HComp)/m_nBound2;
					*(lpDest++)=BYTE(Tmp1+Tmp2+Tmp3+Tmp4);
					
					lpUpper-=2;
					lpLower-=2;
				}
			}
		}
		LineCount--;
		lpDestRows+=m_nImageWidth*3;
		pImageBlock->u16XferLines++;
	}
}

void CScanModuleGlobal::BltGrayLines(LPDS_BLOCKDESCRIPTOR pImageBlock)
{
	LONG VRatio;
	LONG HRatio;
	BOOL isChange;
	double Tmp1,Tmp2,Tmp3,Tmp4;
	WORD LineCount = pImageBlock->u16BufferLines;
	char* lpDestRows = (char*)pImageBlock->pBuffer;
	pImageBlock->u16XferLines = 0;
	if (m_pLowerSrcLine == NULL) {
		m_pLowerSrcLine = (char*) malloc (m_pSourceBuffer->m_nBytesPerLine);
		if (m_pLowerSrcLine) 
			m_pSourceBuffer->GetOneLine(m_pLowerSrcLine);
		else 
			return;
		
		if (m_pUpperSrcLine != NULL) free (m_pUpperSrcLine);
		m_pUpperSrcLine = (char*) malloc(m_pSourceBuffer->m_nBytesPerLine);
		if (m_pUpperSrcLine)
			m_pSourceBuffer->GetOneLine(m_pUpperSrcLine);
		else
			return;
	}
	
	while ((LineCount>0)&&m_pVRuler->GetNext(VRatio,isChange)){
		if (isChange){
			char* pTemp=m_pLowerSrcLine;
			m_pLowerSrcLine=m_pUpperSrcLine;
			m_pUpperSrcLine=pTemp;
			m_pSourceBuffer->GetOneLine(m_pUpperSrcLine);
		}
		char* lpDest=lpDestRows;
		if (VRatio==0){
			m_pHRuler->Reset();
			char* lpLower=m_pLowerSrcLine;
			char* lp01 = m_pLowerSrcLine+m_pSourceBuffer->m_nBytesPerLine-2;
			while (m_pHRuler->GetNextGray(HRatio,lpLower)){
				if(lpLower>lp01)
                   lpLower--;
				if (HRatio==0){
					*(lpDest++)=BYTE(*lpLower);
				}
				else{
					*(lpDest++)=BYTE((LONG(BYTE(*(lpLower+1)))*(HRatio)+LONG(BYTE(*lpLower))*(m_nHBound-HRatio))/m_nHBound);
				}
			}
		}
		else{
			m_pHRuler->Reset();
			char* lpLower=m_pLowerSrcLine;
			char* lpUpper=m_pUpperSrcLine;
			char* lp01 = m_pLowerSrcLine+m_pSourceBuffer->m_nBytesPerLine-2;
			char* lp02 = m_pUpperSrcLine+m_pSourceBuffer->m_nBytesPerLine-2;
			while (m_pHRuler->GetNextGray(HRatio,lpLower,lpUpper)){
				if(lpLower>lp01)
                   			lpLower-=3;
				if(lpUpper>lp02)
                   			lpUpper-=3;
				if (HRatio==0){
					*(lpDest++)=BYTE((LONG(BYTE(*(lpUpper++)))*(VRatio)+LONG(BYTE(*(lpLower++)))*(m_nVBound-VRatio))/m_nVBound);
					//*(lpDest++)=BYTE((LONG(BYTE(*(lpUpper++)))*(VRatio)+LONG(BYTE(*(lpLower++)))*VComp)/m_nVBound);
					lpUpper--;
					lpLower--;
				}
				else{
                    			LONG VComp=m_nVBound-VRatio;
					LONG HComp=m_nHBound-HRatio;
					Tmp1=BYTE(*(lpUpper+1));
					Tmp1=Tmp1*(VRatio*HRatio)/m_nBound2;
					Tmp2=BYTE(*lpUpper);
					Tmp2=Tmp2*(VRatio*HComp)/m_nBound2;
					Tmp3=BYTE(*(lpLower+1));
					Tmp3=Tmp3*(VComp*HRatio)/m_nBound2;
					Tmp4=BYTE(*lpLower);
					Tmp4=Tmp4*(VComp*HComp)/m_nBound2;
					
					*(lpDest++)=BYTE(Tmp1+Tmp2+Tmp3+Tmp4);
				}
			}
		}
		LineCount--;
		lpDestRows+=m_nImageWidth;
		pImageBlock->u16XferLines++;		
	}
}


void CScanModuleGlobal::BltTextLines(LPDS_BLOCKDESCRIPTOR pImageBlock)
{
	LONG VRatio;                  
	BOOL isChange;
	WORD LineCount = pImageBlock->u16BufferLines;
	char* lpDestRows = (char*)pImageBlock->pBuffer;
	pImageBlock->u16XferLines = 0;
	long BytesPerRow;
	
	BytesPerRow = m_nImageWidth / 8 + ((m_nImageWidth % 8 == 0) ? 0 : 1);
	
	if (m_pLowerSrcLine == NULL) {
		m_pLowerSrcLine = (char*) malloc (BytesPerRow);
		if (m_pLowerSrcLine) {
			GetTextOneLine(m_pLowerSrcLine);
		}
		else 
			return;
		
		if (m_pUpperSrcLine != NULL) free (m_pUpperSrcLine);
		m_pUpperSrcLine = (char*) malloc(BytesPerRow);
		if (m_pUpperSrcLine) {
			GetTextOneLine(m_pUpperSrcLine);
		}
		else
			return;
	}
		
	while (LineCount>0&&m_pVRuler->GetNext(VRatio,isChange)){
		if (isChange){
			char* pTemp=m_pLowerSrcLine;
			m_pLowerSrcLine=m_pUpperSrcLine;
			m_pUpperSrcLine=pTemp;
			GetTextOneLine(m_pUpperSrcLine);			
		}
		char* lpDest=lpDestRows;
		if (VRatio>=m_nHalfBound){
			for (DWORD i=0;i<BytesPerRow;i++)
				*(lpDest++)=m_pUpperSrcLine[i];
		}
		else{
			for (DWORD i=0;i<BytesPerRow;i++)
				*(lpDest++)=m_pLowerSrcLine[i];
		}
		LineCount--;
		lpDestRows+=BytesPerRow;
		pImageBlock->u16XferLines++;		
	}
}
void CScanModuleGlobal::DecRGBLines(LPDS_BLOCKDESCRIPTOR pImageBlock)
{
	static CFraction frHRatio(m_nImageWidth,m_pSourceBuffer->m_nBytesPerLine/3);
	static CFraction frVRatio(m_nImageHeight,m_dwSrcLength);
	static CFraction frHTmp = frHRatio;
	static CFraction frVTmp = frVRatio;

	static BOOL isReset = false;		
	if(isReset)
	{
		frHRatio.SetValue(m_nImageWidth,m_pSourceBuffer->m_nBytesPerLine/3,true);
		frVRatio.SetValue(m_nImageHeight,m_dwSrcLength,true);
		frHTmp = frHRatio;
		frVTmp = frVRatio;
		isReset = false;
	}
	
	//const CFraction frCompareOne( GetDenominator(),GetDenominator() );
	if( (frHRatio>=1) || (frVRatio>=1) ) //if Ratio>1 need to do BltRGBLines() not DecRGBLines()
		return;
	long LineCount = pImageBlock->u16BufferLines;
	long PointCount = m_pSourceBuffer->m_nBytesPerLine / 3;
	long lGetCount;
	
	char* lpDestRows = (char*)pImageBlock->pBuffer;
	pImageBlock->u16XferLines = 0;
	if (m_pLowerSrcLine == NULL) {
		m_pLowerSrcLine = (char*) malloc (m_pSourceBuffer->m_nBytesPerLine);
		if (m_pLowerSrcLine) 
			if (!m_pSourceBuffer->GetOneLine(m_pLowerSrcLine)) return;
		else 
			return;
	}
	char* lpDest=lpDestRows;
	do
	{
		char* lpLower=m_pLowerSrcLine;
		
		if (frVTmp >= 1)	{	//Use this line
			frVTmp--;
			lGetCount = 0;
			frHTmp = frHRatio;
			while(PointCount-- > 0)	{
				if(frHTmp >= 1)	{	//Use this pixel
					frHTmp--;
					*(lpDest  )=BYTE(*(lpLower  ));
					*(lpDest+1)=BYTE(*(lpLower+1));
					*(lpDest+2)=BYTE(*(lpLower+2));
					lpDest += 3;
					lGetCount++;
				}
				lpLower += 3;
				frHTmp += frHRatio;
			}
			PointCount = m_pSourceBuffer->m_nBytesPerLine / 3;
			pImageBlock->u16XferLines++;
			LineCount--;
		}
		if (!m_pSourceBuffer->GetOneLine(m_pLowerSrcLine)) break;
		frVTmp += frVRatio;
	}while (LineCount > 0);

	if( (m_nLinesTransfered+pImageBlock->u16XferLines) >= m_nImageHeight )
		isReset = true;
	
}

void CScanModuleGlobal::DecGrayLines(LPDS_BLOCKDESCRIPTOR pImageBlock)
{
	static CFraction frHRatio(m_nImageWidth,m_pSourceBuffer->m_nBytesPerLine);
	static CFraction frVRatio(m_nImageHeight,m_dwSrcLength);
	frHRatio.Reduction();
	frVRatio.Reduction();

	static CFraction frHTmp = frHRatio;
	static CFraction frVTmp = frVRatio;
	static BOOL isReset = false;
		
	if(isReset)
	{
		frHRatio.SetValue(m_nImageWidth,m_pSourceBuffer->m_nBytesPerLine, TRUE);
		frVRatio.SetValue(m_nImageHeight,m_dwSrcLength, TRUE);
		frHTmp = frHRatio;
		frVTmp = frVRatio;
	}
	if( (frHRatio>=1) || (frVRatio>=1) ) //if Ratio>1 need to do BltRGBLines() not DecRGBLines()
		return;
	long LineCount = pImageBlock->u16BufferLines;
	long PointCount = m_pSourceBuffer->m_nBytesPerLine;
	
	char* lpDestRows = (char*)pImageBlock->pBuffer;
	pImageBlock->u16XferLines = 0;
	if (m_pLowerSrcLine == NULL) {
		m_pLowerSrcLine = (char*) malloc (m_pSourceBuffer->m_nBytesPerLine);
		if (m_pLowerSrcLine) 
			if (!m_pSourceBuffer->GetOneLine(m_pLowerSrcLine)) return;
		else 
			return;
	}
	char* lpDest=lpDestRows;
	do
	{
		char* lpLower=m_pLowerSrcLine;
		
		if (frVTmp >= 1)	{	//Skip this line
			frVTmp--;
			while(PointCount-- > 0)	{
				if(frHTmp >= 1)	{	//Skip this pixel
					frHTmp--;
					*(lpDest  )=BYTE(*(lpLower  ));
					//*(lpDest+1)=BYTE(*(lpLower+1));
					//*(lpDest+2)=BYTE(*(lpLower+2));
					lpDest++;
				}
				lpLower++;
				frHTmp += frHRatio;
			}
			PointCount = m_pSourceBuffer->m_nBytesPerLine;
			pImageBlock->u16XferLines++;
			LineCount--;
		}
		if (!m_pSourceBuffer->GetOneLine(m_pLowerSrcLine)) break;
		frVTmp += frVRatio;
	}while (LineCount > 0);
	
	if( (m_nLinesTransfered+pImageBlock->u16XferLines) >= m_nImageHeight )
		isReset = true;
}


void CScanModuleGlobal::DecTextLines(LPDS_BLOCKDESCRIPTOR pImageBlock) //unfinish, now is same of DecGrayLines()
{
	static CFraction frHRatio(m_nImageWidth,m_pSourceBuffer->m_nBytesPerLine);
	static CFraction frVRatio(m_nImageHeight,m_dwSrcLength);
	frHRatio.Reduction();
	frVRatio.Reduction();
	static CFraction frHTmp = frHRatio;
	static CFraction frVTmp = frVRatio;
	static BOOL isReset = false;
		
	if(isReset)
	{
		frHRatio.SetValue(m_nImageWidth,m_pSourceBuffer->m_nBytesPerLine,TRUE);
		frVRatio.SetValue(m_nImageHeight,m_dwSrcLength,TRUE);
		frHTmp = frHRatio;
		frVTmp = frVRatio;
	}
	if( (frHRatio>=1) || (frVRatio>=1) ) //if Ratio>1 need to do BltRGBLines() not DecRGBLines()
		return;
	long LineCount = pImageBlock->u16BufferLines;
	long PointCount = m_pSourceBuffer->m_nBytesPerLine;
	
	char* lpDestRows = (char*)pImageBlock->pBuffer;
	pImageBlock->u16XferLines = 0;
	if (m_pLowerSrcLine == NULL) {
		m_pLowerSrcLine = (char*) malloc (m_pSourceBuffer->m_nBytesPerLine);
		if (m_pLowerSrcLine) 
			if (!m_pSourceBuffer->GetOneLine(m_pLowerSrcLine)) return;
		else 
			return;
	}
	char* lpDest=lpDestRows;
	do
	{
		char* lpLower=m_pLowerSrcLine;
		
		if (frVTmp >= 1)	{	//Skip this line
			frVTmp--;
			while(PointCount-- > 0)	{
				if(frHTmp >= 1)	{	//Skip this pixel
					frHTmp--;
					*(lpDest  )=BYTE(*(lpLower  ));
					//*(lpDest+1)=BYTE(*(lpLower+1));
					//*(lpDest+2)=BYTE(*(lpLower+2));
					lpDest++;
				}
				lpLower++;
				frHTmp += frHRatio;
			}
			PointCount = m_pSourceBuffer->m_nBytesPerLine;
			pImageBlock->u16XferLines++;
			LineCount--;
		}
		if (!m_pSourceBuffer->GetOneLine(m_pLowerSrcLine)) break;
		frVTmp += frVRatio;
	}while (LineCount > 0);
	
	if( (m_nLinesTransfered+pImageBlock->u16XferLines) >= m_nImageHeight )
		isReset = true;
	
}

void CScanModuleGlobal::GetTextOneLine(char* pBuffer)
{
//	m_pSourceBuffer->GetOneLine(pBuffer);
//	return;
	
	char* pTextSrc;
	pTextSrc = (char*) malloc (m_dwLineByte);
	if (pTextSrc == NULL) return;
	m_pSourceBuffer->GetOneLine(pTextSrc);
	BYTE BitsFilter[8];
	int src_proc, dest_proc;
	
	src_proc = dest_proc = 0;
	
	BitsFilter[0] = 0x80;
	BitsFilter[1] = 0x40;
	BitsFilter[2] = 0x20;
	BitsFilter[3] = 0x10;
	BitsFilter[4] = 0x08;
	BitsFilter[5] = 0x04;
	BitsFilter[6] = 0x02;
	BitsFilter[7] = 0x01;

	m_pHRuler->Reset();
	char* pSrc=pTextSrc;
	BYTE SrcByte=*pSrc;
	BYTE DestByte=0;
	BYTE SrcLeftBit=(BYTE)((SrcByte&BitsFilter[0])!=0);
	BYTE SrcRightBit=(BYTE)((SrcByte&BitsFilter[1])!=0);
	WORD j=2;

	DWORD Width = m_nImageWidth;	
	LONG HRatio;
	BOOL isChange;
	
	//Folowing codes are writed by considering performance.
	while (Width >=8) {
		DestByte=0;
		//Bit 0
		m_pHRuler->GetNext(HRatio,isChange);
		if (isChange){
			if (j>=8) {
				j=0;
				pSrc++;
				SrcByte=*pSrc;
				src_proc++;
			}
			SrcLeftBit=SrcRightBit;
			SrcRightBit=(BYTE)((SrcByte&BitsFilter[j])!=0);
			j++;
		}
		if (HRatio>=m_nHalfBound)
			DestByte=SrcRightBit;
		else 
			DestByte=SrcLeftBit;
		//Bit 1
		m_pHRuler->GetNext(HRatio,isChange);
		if (isChange){
			if (j>=8) {
				j=0;
				pSrc++;
				SrcByte=*pSrc;
							src_proc++;
	
			}
			SrcLeftBit=SrcRightBit;
			SrcRightBit=(BYTE)((SrcByte&BitsFilter[j])!=0);
			j++;
		}
		DestByte<<=1;
		if (HRatio>=m_nHalfBound)
			DestByte|=SrcRightBit;
		else 
			DestByte|=SrcLeftBit;
		//Bit 2
		m_pHRuler->GetNext(HRatio,isChange);
		if (isChange){
			if (j>=8) {
				j=0;
				pSrc++;
				SrcByte=*pSrc;
								src_proc++;

			}
			SrcLeftBit=SrcRightBit;
			SrcRightBit=(BYTE)((SrcByte&BitsFilter[j])!=0);
			j++;
		}
		DestByte<<=1;
		if (HRatio>=m_nHalfBound)
			DestByte|=SrcRightBit;
		else 
			DestByte|=SrcLeftBit;
		//Bit 3
		m_pHRuler->GetNext(HRatio,isChange);
		if (isChange){
			if (j>=8) {
				j=0;
				pSrc++;
				SrcByte=*pSrc;
				src_proc++;
				
			}
			SrcLeftBit=SrcRightBit;
			SrcRightBit=(BYTE)((SrcByte&BitsFilter[j])!=0);
			j++;
		}
		DestByte<<=1;
		if (HRatio>=m_nHalfBound)
			DestByte|=SrcRightBit;
		else 
			DestByte|=SrcLeftBit;
		//Bit 4
		m_pHRuler->GetNext(HRatio,isChange);
		if (isChange){
			if (j>=8) {
				j=0;
				pSrc++;
				SrcByte=*pSrc;
							src_proc++;

			}
			SrcLeftBit=SrcRightBit;
			SrcRightBit=(BYTE)((SrcByte&BitsFilter[j])!=0);
			j++;
		}
		DestByte<<=1;
		if (HRatio>=m_nHalfBound)
			DestByte|=SrcRightBit;
		else 
			DestByte|=SrcLeftBit;
		//Bit 5
		m_pHRuler->GetNext(HRatio,isChange);
		if (isChange){
			if (j>=8) {
				j=0;
				pSrc++;
				SrcByte=*pSrc;
								src_proc++;

			}
			SrcLeftBit=SrcRightBit;
			SrcRightBit=(BYTE)((SrcByte&BitsFilter[j])!=0);
			j++;
		}
		DestByte<<=1;
		if (HRatio>=m_nHalfBound)
			DestByte|=SrcRightBit;
		else 
			DestByte|=SrcLeftBit;
		//Bit 6
		m_pHRuler->GetNext(HRatio,isChange);
		if (isChange){
			if (j>=8) {
				j=0;
				pSrc++;
				SrcByte=*pSrc;
								src_proc++;

			}
			SrcLeftBit=SrcRightBit;
			SrcRightBit=(BYTE)((SrcByte&BitsFilter[j])!=0);
			j++;
		}
		DestByte<<=1;
		if (HRatio>=m_nHalfBound)
			DestByte|=SrcRightBit;
		else 
			DestByte|=SrcLeftBit;
		//Bit 7
		m_pHRuler->GetNext(HRatio,isChange);
		if (isChange){
			if (j>=8) {
				j=0;
				pSrc++;
				SrcByte=*pSrc;
				src_proc++;

			}
			SrcLeftBit=SrcRightBit;
			SrcRightBit=(BYTE)((SrcByte&BitsFilter[j])!=0);
			j++;
		}
		DestByte<<=1;
		if (HRatio>=m_nHalfBound)
			DestByte|=SrcRightBit;
		else 
			DestByte|=SrcLeftBit;
		Width-=8;
		*(pBuffer++)=DestByte;
		dest_proc++;
	}
	WORD i = 8;
	if (Width>0) {
		DestByte=0;
		while (m_pHRuler->GetNext(HRatio,isChange)) {
			if (isChange) {
				if (j>=8) {
					j=0;
					pSrc++;
					SrcByte=*pSrc;
				}
				SrcLeftBit=SrcRightBit;
				SrcRightBit=(BYTE)((SrcByte&BitsFilter[j])!=0);
				j++;
			}
			DestByte<<=1;
			if (HRatio>=m_nHalfBound)
				DestByte|=SrcRightBit;
			else 
				DestByte|=SrcLeftBit;
			i--;
		}
		DestByte <<= i;
		*pBuffer=DestByte; 
	}
	free (pTextSrc); 
}
