/* BearPaw1200CU panel

   Copyright (C) 2001 Mustek. 
   Maintained by Tom Wang and Alex Lin
   <tom.wang@mustek.com.tw, alex_lin@mustek.com.tw>

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston,
   MA 02111-1307, USA.
*/


#include "Filter.h"
#include "SourceBuffer.h"
#include "SpExport.h"
#include "ScanModuleInternal.h"

CSourceBuffer::CSourceBuffer(CScanModuleGlobal* ptheGlobal)
{
	m_pSourceData = NULL;
	m_pGlobal = ptheGlobal; 
}

CSourceBuffer::~CSourceBuffer()
{
	if (m_pSourceData != NULL) {
		free(m_pSourceData);
		m_pSourceData = NULL;
	}
}

void CSourceBuffer::InitializeBuffer(long BytesPerLine,long length)
{
	long max_free = 512*1024;  // Let Maxfree memory be 512K ram
	bool done = false;
	max_free = max_free / 4;
	if (max_free > BytesPerLine * length) max_free = BytesPerLine * length;
	while (!done && max_free > BytesPerLine) {
		m_pSourceData = (char*) malloc(max_free*2); 
		if (m_pSourceData == NULL) {
			max_free /= 2;
			continue;
		}
		m_nTotalLines = max_free / BytesPerLine;
		m_nCurrentLine = 0;
		m_nEndLine = 0;
		m_nDataLength = max_free;
		m_nBytesPerLine = BytesPerLine;
		m_nSourceLength = length;
		done = true;
	}
}

long CSourceBuffer::ReadImageToBuffer()
{
	// first. swap the current image to upper of buffer
	char* pDest, *pSrc;
	
	if (m_pGlobal == NULL) return 0;
	
	long MaxLineNumber = m_nSourceLength;
	pDest = m_pSourceData;
	pSrc = pDest + m_nCurrentLine * m_nBytesPerLine;
	
	long i;
	for (i = m_nCurrentLine ; i < m_nEndLine; i ++) {
		memcpy(pDest, pSrc, m_nBytesPerLine);
		pDest += m_nBytesPerLine;
		pSrc += m_nBytesPerLine;
	}
	m_nEndLine = m_nEndLine - m_nCurrentLine;
	m_nCurrentLine = 0;
	long actual_line;
	if (m_nTotalLines - m_nEndLine > MaxLineNumber)
		actual_line = MaxLineNumber;
	else
		actual_line = m_nTotalLines - m_nEndLine;
		
	IMAGEROWS image_row;	
	
	image_row.ecErrorCode = 0;
	image_row.roRgbOrder = RO_RGB;
	image_row.wWantedLineNum = actual_line;
	image_row.pBuffer = (LPBYTE) (m_pSourceData + m_nBytesPerLine * m_nEndLine);
	if (!ReadScannedData(&image_row)) {
		ShowUp("Failed to GetRows 1");
		return 0;
	}
	if (image_row.ecErrorCode != EC_OK) {
		ShowUp("Failed to GetRows 2, Error code = %d", image_row.ecErrorCode);
		return 0;
	}

	m_nSourceLength -= image_row.wXferedLineNum;
	m_nEndLine = m_nEndLine + image_row.wXferedLineNum;
	DS_SCANMODE current_mode;
	current_mode = m_pGlobal->m_ScanJob.smScanMode;

	// Gamma table mapping
	if ( current_mode != DSSM_LINEART && !m_pGlobal->m_bDownloadGamma) {
		long table_size;
		long pixel_count;
		long i;
		switch (current_mode) {
			case DSSM_LINEART:
				break;
			case DSSM_GRAY:
				table_size = 1 << 8;
				pixel_count = image_row.wXferedLineNum*m_nBytesPerLine;
				for ( i = 0 ; i < pixel_count; i++)
					image_row.pBuffer[i] = m_pGlobal->m_pEnhance[image_row.pBuffer[i]];
				break;
			case DSSM_COLOR24:
				table_size = 1 << 8;
				pixel_count = image_row.wXferedLineNum*m_nBytesPerLine / 3;
				for ( i = 0 ; i < pixel_count; i++) {
					// R
					image_row.pBuffer[i*3] = m_pGlobal->m_pEnhance[image_row.pBuffer[i*3]];
					// G
					image_row.pBuffer[i*3+1] = m_pGlobal->m_pEnhance[image_row.pBuffer[i*3+1]+256];
					// B
					image_row.pBuffer[i*3+2] = m_pGlobal->m_pEnhance[image_row.pBuffer[i*3+2]+512];	
				}
		}
		
	}	
	
	SM_PRIVATE* myPrivate;
	myPrivate = (SM_PRIVATE*)&(m_pGlobal->m_ScanJob.cScanModulePrivate);

	//Process for Negative
	DS_LIGHTSOURCE current_LightSource;
	current_LightSource = m_pGlobal->m_ScanJob.lsLightSource;
	
	CFilter* pFilter = new CFilter();
	if(current_LightSource == DSLS_NEGATIVE) {
		pFilter->Invert((char*)image_row.pBuffer,current_mode,image_row.wXferedLineNum,m_nBytesPerLine);
		pFilter->AutoLevel((char*)image_row.pBuffer,current_mode,image_row.wXferedLineNum,m_nBytesPerLine);
		pFilter->QBetChange((char*)image_row.pBuffer,current_mode,image_row.wXferedLineNum,m_nBytesPerLine);
	}
	//Process for filter setting
	switch (myPrivate->ftFilter) {
		case DSFT_BLUR:
			pFilter->Blur((char*)image_row.pBuffer,current_mode,image_row.wXferedLineNum,m_nBytesPerLine);
			break;
		case DSFT_BLURMORE:
			pFilter->Blur((char*)image_row.pBuffer,current_mode,image_row.wXferedLineNum,m_nBytesPerLine);
			pFilter->Blur((char*)image_row.pBuffer,current_mode,image_row.wXferedLineNum,m_nBytesPerLine);
			break;
		case DSFT_SHARPEN:
			pFilter->Sharpen((char*)image_row.pBuffer,current_mode,image_row.wXferedLineNum,m_nBytesPerLine);
			break;
		case DSFT_SHARPENMORE:
			pFilter->Sharpen((char*)image_row.pBuffer,current_mode,image_row.wXferedLineNum,m_nBytesPerLine);
			pFilter->Sharpen((char*)image_row.pBuffer,current_mode,image_row.wXferedLineNum,m_nBytesPerLine);
			break;
		case DSFT_INVERT:
			pFilter->Invert((char*)image_row.pBuffer,current_mode,image_row.wXferedLineNum,m_nBytesPerLine);
			break;
		case DSFT_FLIP:
			pFilter->Flip((char*)image_row.pBuffer,current_mode,image_row.wXferedLineNum,m_nBytesPerLine);
			break;
		case DSFT_EMBOSS:
			pFilter->Emboss((char*)image_row.pBuffer,current_mode,image_row.wXferedLineNum,m_nBytesPerLine);
			break;
		case DSFT_UNSHARPENMASK:
			pFilter->UnSharpMask((char*)image_row.pBuffer,current_mode,image_row.wXferedLineNum,m_nBytesPerLine);
			break;
		default:
			break;
	}
	// demoire here
	switch (myPrivate->dsDescreen) {
		case DSDE_NONE:
			break;
		case DSDE_NEWSPAPER:
		case DSDE_MAGAZINE:
		case DSDE_ARTPAINT:
		default:
			pFilter->DeMoire((char*)image_row.pBuffer,current_mode, image_row.wXferedLineNum,m_nBytesPerLine);
			break;	
	}
	delete pFilter;
	return image_row.wXferedLineNum;
}

long CSourceBuffer::DataLineCount()
{
	return (m_nEndLine-m_nCurrentLine);
}

bool CSourceBuffer::GetOneLine(char* data)
{
	if (m_nCurrentLine >= m_nEndLine) {
		if (ReadImageToBuffer()==0) return false;
	}
	memcpy(data, (char*)(m_pSourceData+m_nCurrentLine*m_nBytesPerLine),m_nBytesPerLine);
	m_nCurrentLine++;
	return true;
 }

bool CSourceBuffer::MoreData()
{
	if (m_nCurrentLine < m_nEndLine) return true;
	if (m_nSourceLength > 0) return true;
	if (!m_pGlobal->m_pVRuler->EndRuler()) return true;
	return false;
}










