/* BearPaw1200CU panel

   Copyright (C) 2001 Mustek. 
   Maintained by Tom Wang and Alex Lin
   <tom.wang@mustek.com.tw, alex_lin@mustek.com.tw>

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston,
   MA 02111-1307, USA.
*/


#ifndef _SpExport_
#define _SpExport_

#include "BaseTypedef.h"

typedef WORD ERRORCODE, *LPERRORCODE;
#define EC_OK		0x00
#define EC_FAILED	0x01
#define	ERR_SPI_NONE	0
#define	ERR_SPI_VERSION	1
#define	ERR_SPI_INTERFACE	2
#define	ERR_SPI_PORTADDR	3
#define	ERR_SPI_PORTMODE	4
#define	ERR_SPI_SLIDEKIT	5
#define	ERR_SPI_ADF		6
#define	ERR_SPI_ISDOWNLOADGAMMA	7
#define	ERR_SPI_INPUTGAMMA	8
#define	ERR_SPI_OUTPUTGAMMA	9
#define	ERR_SPI_ISTRIGPRESCAN	10
#define	ERR_SPI_ISTRIGSCAN	11
#define	ERR_SPI_ISTRIGAP	12
#define	ERR_SPI_FRAME		13
#define	ERR_SPI_SCANMODE	14
#define	ERR_SPI_LINEARTTHRESHOLD	15
#define	ERR_SPI_PIXELFLAVOR	16
#define	ERR_SPI_SCANSOURCE	17
#define	ERR_SPI_GAMMATABLE	18
#define	ERR_SPI_CALIBRATION	19
#define	ERR_SPI_RB		20
#define	ERR_SPI_RT		21
#define	ERR_SPI_POWERDELAY	22
#define	ERR_SPI_EXPOSETIME	23
#define	ERR_SPI_DELAYTIME	24
#define	ERR_SPI_ALLOCATEMEM	25
#define	ERR_SPI_CARRIAGE	26
#define	ERR_SPI_STOPSCAN	27
#define	ERR_SPI_CLOSE		28
#define	ERR_SPI_READIMAGE	29
#define	ERR_SPI_REVERSE_RGB	30
#define	ERR_SPI_POWERON		31
#define	ERR_SPI_POWEROFF	32
#define	ERR_SPI_TRIGPRESCAN	33
#define	ERR_SPI_TRIGSCAN	34
#define	ERR_SPI_TRIGAP		35
#define	ERR_SPI_ADFPAPER	36
#define	ERR_SPI_NOTCONNECT	37
#define ERR_SPI_PRINTING	38
#define	ERR_SPI_PROGRAMMING	39
#define	ERR_SPI_HARDWARE	40
#define	ERR_SPI_NONSTI		41
#define ERR_SPI_CANCELSCAN	42

typedef WORD IOINTERFACE, *LPIOINTERFACE;
#define II_PARALLEL	0x00
#define II_SCSI		0x01
#define II_USB		0x02
typedef WORD PARALLELMODE, *LPPARALLELMODE;
#define PM_NIBBLE	0x00
#define PM_UNIDIR	0x01
#define PM_BIDIR	0x02
#define PM_EPP		0x03
#define PM_ECP		0x04
typedef BYTE SPICALLBOOL, *LPSPICALLBOOL;
#define SB_FALSE	0x00
#define SB_TRUE		0x01
typedef WORD SCANMODE, *LPSCANMODE;
#define SM_TEXT		0x00
#define SM_GRAY		0x01
#define SM_RGB24	0x02
#define SM_GRAY10	0x03
#define SM_RGB30	0x04
#define SM_GRAY12	0x05
#define SM_RGB36	0x06
#define SM_GRAY14	0x07
#define SM_RGB42	0x08
#define SM_GRAY16	0x09
#define SM_RGB48	0x0a
typedef BYTE PIXELFLAVOR, *LPPIXELFLAVOR;
#define PF_BlackIs0	0x00
#define PF_WhiteIs0	0x01
typedef BYTE SCANSOURCE, *LPSCANSOURCE;
#define SS_Reflective	0x00
#define SS_Positive	0x01
#define SS_Negative	0x02
#define SS_ADF		0x03
typedef WORD RGBORDER, *LPRGBORDER;
#define RO_RGB		0x00
#define RO_BGR		0x01
typedef WORD TESTID, *LPTESTID;
#define TI_INFORMATION	0x01
#define TI_PASSTHROUGH	0x02
#define TI_COVERSENSOR	0x03
#define TI_BUTTON	0x04
#define TI_LAMPOFF	0x05
#define TI_LAMPON	0x06
typedef WORD COVERSTATUS, *LPCOVERSTATUS;
#define CS_COVEROPEN	0x02
#define CS_COVERCLOSE	0x03

typedef struct tagFRAME {
	WORD x1;
	WORD y1;
	WORD x2;
	WORD y2;
} FRAME ,*LPFRAME;

typedef struct tagDEVICEINFO {
	ERRORCODE ecErrorCode;
	IOINTERFACE iiInterface;
	WORD wPortAddr;
	PARALLELMODE pmParallelMode; // for Parall use only
	WORD wSizeOfInfo;
	LPCSTR pInfo;
	SPICALLBOOL sbSlideKitInstalled;
	SPICALLBOOL sbADFInstalled;
	SPICALLBOOL sbDownLoadGamma;
	BYTE bInputGammaBits;
	BYTE bOutputGammaBits;
	SPICALLBOOL sbCanTriggerPrescan;
	SPICALLBOOL sbCanTriggerscan;
	SPICALLBOOL sbCanTriggerAP;
} DEVICEINFO, *LPDEVICEINFO;

typedef struct tagGAMMAINFO {
	ERRORCODE ecErrorCode;
	SCANMODE smScanMode;
	WORD wInputGammaBits;
	WORD wOutputGammaBits;
} GAMMAINFO, *LPGAMMAINFO;

typedef struct tagSETPARAMETERS {
	ERRORCODE ecErrorCode;
	FRAME fmArea;
	WORD wTargetDPI;
	SCANMODE smScanMode;
	WORD wLinearThreshold;		// threshold for Lineart mode
	PIXELFLAVOR pfPixelFlavor;
	SCANSOURCE ssScanSource;
	LPWORD pGammaTable;
} SETPARAMETERS, *LPSETPARAMETERS;

typedef struct tagGETPARAMETERS {
	ERRORCODE ecErrorCode;
	WORD wSourceXDPI;
	WORD wSourceYDPI;
	DWORD dwLineByteWidth;
	DWORD dwLength;
} GETPARAMETERS, *LPGETPARAMETERS;

typedef struct tagIMAGEROWS {
	ERRORCODE ecErrorCode;
	RGBORDER roRgbOrder;
	WORD wWantedLineNum;
	WORD wXferedLineNum;
	LPBYTE pBuffer;
} IMAGEROWS, *LPIMAGEROWS;

typedef struct tagCOMMON {
	ERRORCODE ecErrorCode;
} COMMON, *LPCOMMON;

typedef struct tagSTARTSCAN {
	ERRORCODE ecErrorCode;
} STARTSCAN, *LPSTARTSCAN;

typedef struct  tagLAMPCONTROL {
	ERRORCODE ecErrorCode;
	SPICALLBOOL sbTurnOn;
} LAMPCONTROL, *LPLAMPCONTROL;

typedef struct  tagDEVICESTATUS {
	ERRORCODE ecErrorCode;
	SPICALLBOOL sbTriggerPrescan;
	SPICALLBOOL sbTriggerScan;
	SPICALLBOOL sbTriggerAP;
	SPICALLBOOL sbPaperLoadedInADF;
} DEVICESTATUS, *LPDEVICESTATUS;

typedef struct tagVERSION {
	ERRORCODE ecErrorCode;
	BYTE bMajor;
	BYTE bMinor;
} VERSION, *LPVERSION;

#ifdef __cplusplus
extern "C" {
#endif

BOOL SpicallOpen();
BOOL SpicallClose();
BOOL GetVersionInfo(LPVERSION pVersion);
BOOL GetDeviceInfo(LPDEVICEINFO pDeviceInfo);
BOOL GetDeviceStatus(LPDEVICESTATUS pDeviceStatus);
BOOL PowerControl(LPLAMPCONTROL pLampControl);
BOOL CarriageHome(LPCOMMON pCommon);
BOOL GetGammaInfo(LPGAMMAINFO pGammaInfo);
BOOL SetParameters(LPSETPARAMETERS pSetParameters);
BOOL GetParameters(LPGETPARAMETERS pGetParameters);
BOOL StartScan(LPCOMMON pCommon);
BOOL ReadScannedData(LPIMAGEROWS pImageRows);
BOOL StopScan(LPCOMMON pCommon);

#ifdef __cplusplus
}
#endif

#endif
