/* BearPaw1200CU panel

   Copyright (C) 2001 Mustek. 
   Maintained by Tom Wang and Alex Lin
   <tom.wang@mustek.com.tw, alex_lin@mustek.com.tw>

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston,
   MA 02111-1307, USA.
*/


/*****************************************************************/
/*                                                               */ 
/*	Common module for macintosh header file                      */
/*  Used for TWAIN module, scan module and UI module             */
/*  Created by Tom 88/4/10                                       */
/*                                                               */ 
/*****************************************************************/

#ifndef __COMMON_MODULE___
#define __COMMON_MODULE___

/*****************************************************************/
/*                                                               */ 
/*	Basic data type definitions                                  */
/*                                                               */ 
/*****************************************************************/
typedef long DS_RETURNCODE, *LPDS_RETURNCODE;
typedef short DS_INT16, *LPDS_INT16;
typedef unsigned short DS_UINT16, *LPDS_UINT16;
typedef long DS_INT32, *LPDS_INT32;
typedef unsigned long DS_UINT32, *LPDS_UINT32;
typedef double DS_DOUBLE, *LPDS_DOUBLE;
typedef unsigned long DS_LANGUAGE, *LPDS_LANGUAGE;

typedef unsigned long DS_SCANMODE, *LPDS_SCANMODE;
typedef unsigned long DS_LIGHTSOURCE, *LPDS_LIGHTSOURCE;
typedef unsigned long DS_PIXELFLAVOR, *LPDS_PIXELFLAVOR;
typedef unsigned long DS_COMPRESS, *LPDS_COMPRESS;
typedef unsigned long DS_SCANNERTYPE, *LPDS_SCANNERTYPE;
typedef unsigned long SM_RESIZE,*LPSM_RESIZE;

/*****************************************************************/
/*                                                               */ 
/*	Constants definitions                                        */
/*                                                               */ 
/*****************************************************************/
// Return Codes definitions
const DS_RETURNCODE DSRC_NOERROR = 1;
const DS_RETURNCODE DSRC_SCANCANCEL = 2;
const DS_RETURNCODE DSRC_SCANDONE = 3;
const DS_RETURNCODE DSRC_SCANMORE = 4;


const DS_RETURNCODE DSRC_ERROR = 0;
const DS_RETURNCODE DSRC_INVALIDPARAM = -1;
const DS_RETURNCODE DSRC_NODRIVER = -2;
const DS_RETURNCODE DSRC_DSNOTOPENED = -3;
const DS_RETURNCODE DSRC_SCANNING = -4;
const DS_RETURNCODE DSRC_NOUI = -5;
const DS_RETURNCODE DSRC_UINOTRUNNING = -6;
const DS_RETURNCODE DSRC_NOQUERYIMAGE = -7;
const DS_RETURNCODE DSRC_NOADF = -8;
const DS_RETURNCODE DSRC_ADFPAPERNOTLOAD = -9;

// Constants for DS_SCANNERTYPE
const DS_SCANNERTYPE DSST_UNKNOWN = 0x00;

const DS_SCANNERTYPE DSST_FLATBED = 0x01;

const DS_SCANNERTYPE DSST_SHEETFED = 0x02;

const DS_SCANNERTYPE DSST_HANDHELD = 0x04;


// Constants for DS_COMPRESS
const DS_COMPRESS DSCP_NONE = 0;

// Constants for DS_PIXELFLAVOR
const DS_PIXELFLAVOR DSPF_BLACKISZERO = 0; // 0: Black, 1: White
const DS_PIXELFLAVOR DSPF_WHITEISZERO = 1; // 0: White, 1: Black

// Constants for DS_LIGHTSOURCE

const DS_LIGHTSOURCE DSLS_NONE = 0x0000;
const DS_LIGHTSOURCE DSLS_REFLECTIVE = 0x0001;
const DS_LIGHTSOURCE DSLS_POSITIVE = 0x0002;
const DS_LIGHTSOURCE DSLS_NEGATIVE = 0x0004;
const DS_LIGHTSOURCE DSLS_XRAY = 0x0008;

// Constants for DS_SCANMODE
const DS_SCANMODE	DSSM_NONE = 0x0000;

const DS_SCANMODE	DSSM_LINEART = 0x0001;

const DS_SCANMODE	DSSM_GRAY = 0x0002;
const DS_SCANMODE	DSSM_COLOR24 = 0x0004;
const DS_SCANMODE	DSSM_COLOR30 = 0x0008;
const DS_SCANMODE	DSSM_COLOR36 = 0x0010;
const DS_SCANMODE	DSSM_COLOR48 = 0x0020;

// constants for DS_LANGUAGE

const DS_LANGUAGE	DSLN_NONE = 0;
const DS_LANGUAGE	DSLN_ENGLISH = 1;
const DS_LANGUAGE	DSLN_GERMAN = 2;
const DS_LANGUAGE	DSLN_ITALY = 3;
const DS_LANGUAGE	DSLN_FRENCH = 4;
const DS_LANGUAGE	DSLN_SPANISH = 5;
const DS_LANGUAGE	DSLN_DUTCH = 6;
const DS_LANGUAGE	DSLN_PROTUGUESE = 7;
const DS_LANGUAGE	DSLN_SWEDISH = 8;
const DS_LANGUAGE	DSLN_DANISH = 9;
const DS_LANGUAGE	DSLN_CHINESE = 10;
const DS_LANGUAGE	DSLN_SIMPLE = 11;
const DS_LANGUAGE	DSLN_KOREAN	= 12;
const DS_LANGUAGE	DSLN_JAPANESE = 13;

//constants for SM_RESIZE
const SM_RESIZE		SMRS_NONE = 0;
const SM_RESIZE		SMRS_INTERPOLATION = 1;
const SM_RESIZE		SMRS_REDUCTION = 2;


/*****************************************************************/
/*                                                               */ 
/*	Structure definitions for Twain module and Scan module       */
/*                                                               */ 
/*****************************************************************/
typedef struct {
	DS_UINT32	u32Size;
	DS_INT16	i16Red;
	DS_INT16	i16Green;
	DS_INT16	i16Blue;
	DS_INT16	i16Gray;
} DS_ENHANCE, *LPDS_ENHANCE;

typedef struct {
	DS_UINT32	u32Size;
	DS_DOUBLE	dRed;
	DS_DOUBLE	dGreen;
	DS_DOUBLE	dBlue;
	DS_DOUBLE	dGray;
} DS_GAMMA, *LPDS_GAMMA;

typedef struct {
	DS_UINT32		u32Size;
	DS_UINT32		u32Resolution;
	DS_UINT32		u32XOrigin;
	DS_UINT32		u32YOrigin;
	DS_UINT32   		u32ImageWidth;
	DS_UINT32		u32ImageHeight;
	DS_SCANMODE		smScanMode;
	bool			isFromADF;
	DS_LIGHTSOURCE		lsLightSource;
	DS_UINT32		u32Scaling;
	DS_ENHANCE		ehBrightness;
	DS_ENHANCE		ehContrast;
	DS_GAMMA		gmGamma;
	DS_PIXELFLAVOR		pfPixelFlavor;
	DS_COMPRESS		cpCompress;
	bool			isColorBalance;
	char			cScanModulePrivate[64];
} DS_IMAGEDESCRIPTOR, *LPDS_IMAGEDESCRIPTOR;

typedef struct {
	DS_UINT32	u32Size;
	DS_UINT16	u16BufferLines;
	DS_UINT16	u16XferLines;
	void*		pBuffer;
} DS_BLOCKDESCRIPTOR, *LPDS_BLOCKDESCRIPTOR;


#endif










