/* BearPaw1200CU panel

   Copyright (C) 2001 Mustek. 
   Maintained by Tom Wang and Alex Lin
   <tom.wang@mustek.com.tw, alex_lin@mustek.com.tw>

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston,
   MA 02111-1307, USA.
*/


/**
 * fileselect.c
 * 
 * Simplifies use of the GtkFileSelection dialog
 */

#include <gnome.h>

/* Declare pointers to the dialog and the selected filename string */
GtkWidget *dialog;
gchar *selectedfile;

/**
 * CloseTheDialog ()
 *  
 * Callback triggered when dialog closes down. 
 */
void CloseTheDialog ( GtkWidget * widget, gpointer data )
{
   gtk_main_quit ();
}

/** 
 * OKClicked () 
 * 
 * Callback triggered when the user clicks on the OK button in 
 * the dialog. It duplicates the filename selected within the dialog and
 * stores a pointer to the duplicated string in 'selectedfile', ready to 
 * be returned 
 */
void OKClicked ( GtkButton * button, gpointer data )
{
   /* We need to duplicate the filename selected, since the widget is
    * about to be destroyed, and can take the string with it */
   selectedfile = g_strdup 
    ( gtk_file_selection_get_filename ( GTK_FILE_SELECTION ( dialog ) ) );
   gtk_widget_destroy ( dialog );
}

/**
 * CancelClicked()
 * 
 * Callback triggered when the user clicks on the Cancel button in the 
 * dialog. Does nothing more than destroy the dialog. 
 */
void CancelClicked ( GtkButton * button, gpointer data )
{
   gtk_widget_destroy ( dialog );
}

/**
 * ChooseFile()
 * 
 * Pops open the GtkFileSelection dialog, and returns the name of 
 * the file selected when the dialog closes down, assuming the user actually
 * chooses something. Otherwise, it just returns FALSE. 
 */
gchar * ChooseFile (int iDoWhat)
{

   /* Reset filename to NULL to prevent false returns */
   selectedfile = NULL;

   /* Create the dialog */
   switch (iDoWhat )
     {
     case 0:  //Open
       dialog = gtk_file_selection_new ( "Choose an image to load" );
       break;
     case 1:  //Scan
       dialog = gtk_file_selection_new ( "Save file as" );
       break;
     }

   /** Connect up the signal handlers for the default buttons on the
     * dialog **/
   gtk_signal_connect ( GTK_OBJECT (
     GTK_FILE_SELECTION ( dialog )->ok_button ), "clicked",
     GTK_SIGNAL_FUNC ( OKClicked ), NULL);

   gtk_signal_connect ( GTK_OBJECT (
     GTK_FILE_SELECTION ( dialog )->cancel_button), "clicked",
     GTK_SIGNAL_FUNC ( CancelClicked ), NULL );

   gtk_signal_connect ( GTK_OBJECT ( dialog ), "destroy",
     GTK_SIGNAL_FUNC ( CloseTheDialog ), NULL );

   /* Show the dialog, then start up the gtk_main() loop again. The dialog
    * quits this */
   gtk_widget_show (dialog);
   gtk_window_set_modal ( GTK_WINDOW ( dialog ), TRUE );

   gtk_main ();
   return selectedfile;
}
