/* BearPaw1200CU panel

   Copyright (C) 2001 Mustek. 
   Maintained by Tom Wang and Alex Lin
   <tom.wang@mustek.com.tw, alex_lin@mustek.com.tw>

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston,
   MA 02111-1307, USA.
*/


#include <unistd.h>
#include <sys/types.h>
#include <pwd.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <dirent.h>
#include <sys/stat.h>
#include <gtk/gtk.h>
#include <fcntl.h>


#include "g_procedure.h"
#include "progresswnd.h"
#include "ImageWindow.h"


/** GET USER'S HOME DIRECTORY
 *
 **/
void Get1200UBPath(char* oo)
{
  uid_t uid;
  struct passwd *pw=0;
  
  uid=getuid();
  pw=getpwuid(uid);
  strcpy(oo, pw->pw_dir);
  strcat(oo, "/.1200UB");

  return;
}


/** GET EXECUTABLE FILE PATH
 *
 **/
void GetExePath(char *oo)
{
    char link[256];
    char my_path[256];
    char * lpmy_path;
    pid_t my_pid;

    
    lpmy_path = my_path;

    my_pid = getpid();
    sprintf(link, "/proc/%d/exe",my_pid);
    if (!realpath(link,my_path)) {
        printf("Failed to get my executable path, /proc not mounted?\n");
	oo[0] = '\0';
	
	return;	
    }

    
    lpmy_path = lpmy_path + strlen(my_path) - 1;
    for( ; *lpmy_path != '/' ; lpmy_path--)
    {
         *lpmy_path = '\0';
    }
    
    strcpy(oo, my_path);
        
}


/** THE SETTING FILE DOESN'T EXIST. CREATE IT.
 *
 **/
void InitSettingFile()
{

  
  FILE * lpfp = NULL;
  char chFile[256];

  Get1200UBPath(chFile);
  strcat(chFile, "/setting");
  if( ( lpfp = fopen( chFile, "r" ) ) != NULL )
  {
       fclose(lpfp);
       return;
  }
  
  if( ( lpfp = fopen( chFile, "w" ) ) != NULL )
  {
      /**===============Scan==============**/
      fputs( "[Scan]\n", lpfp );
      fputs( "ScanMode=", lpfp );
      fputs( "Color24\n", lpfp );
      fputs( "Resolution=", lpfp );
      fputs( "Normal\n", lpfp );
      fputs( "ProgramToRun=", lpfp );
      fputs( "Panel\n", lpfp );
      fputs( "\n", lpfp );  // Separator

      /**========== Preview Scan =========**/
      fputs( "[Preview]\n", lpfp );
      fputs( "ScanPreview=TRUE\n", lpfp );
      fputs( "\n", lpfp ); // Separator
      
      /**===============Mail==============**/
      fputs( "[Mail]\n", lpfp );
      fputs( "ScanMode=", lpfp );
      fputs( "Color24\n", lpfp );
      //      fputs( "MailProgram=", lpfp );
      //      fputs( "Sendmail\n", lpfp );
      
      fclose(lpfp);
      return;
  }
  else
  {
      printf("Create setting file failed.\n");
  }  
}


/** GET OPTION FROM SETTING FILE
 *
 **/
char * Option( char * lpchTag, char * lpchTarItem )
{
 
  char * lpchOption = NULL;
  char * lpchSrcItem = NULL;
  char chLine[256];
  char * lpchLine = NULL;
  FILE * lpfp = NULL;
  char chFile[256];
  gboolean FoundTag;
  char * p;
  

  Get1200UBPath(chFile);
  strcat(chFile, "/setting");

  if( ( lpfp = fopen( chFile, "r" ) ) == NULL ) 
  {
      printf("The setting file doesn't exist\n");
      
      return NULL;
  }

  FoundTag = FALSE;
  while(!feof(lpfp))
  {
      fgets(chLine, 256, lpfp );
      if(!feof(lpfp))
      {
	  Trim(chLine);

	  if(FoundTag == FALSE) 
          { 
	      if(!strcmp(lpchTag, chLine)) { FoundTag = TRUE; } 
          }
	  else if(FoundTag == TRUE)
	  {
	      lpchLine = strtok(chLine, "=");
	      lpchSrcItem = lpchLine;
	      do {
		   lpchLine = strtok(NULL, "=");
		   if(lpchLine) { lpchOption = lpchLine; }
	      } while(lpchLine);
	      
	      if (lpchSrcItem != NULL && lpchOption != NULL)
	      {
		  if(!strcmp(lpchSrcItem, lpchTarItem))
		  {
		      p = (char*) malloc(strlen(lpchOption)+1);
                      strcpy(p, lpchOption);
		      fclose(lpfp);
		      
		      return p;
		  }
	      }

	      lpchLine = NULL;
	      lpchOption = NULL;
	  }
      }
  }
  fclose( lpfp );
  return NULL;
}

void  Trim( char * SrcStr )
{
  int i,j,k;

  i = j = k = 0;
  for( ; *(SrcStr+i) == ' ' ; i++ ) {;}
  for( j=strlen(SrcStr)-1 ; *(SrcStr+j) == ' '|| *(SrcStr+j)=='\n' ; j-- ) {;}
  for( k=i ; k<=j ; k++ )
      *(SrcStr+k-i) = *(SrcStr+k);
  
  *(SrcStr+j-i+1) = '\0';

  return;
}


void Create1200UBDir()
{

  DIR * dp;
  char chhome[256];

  Get1200UBPath(chhome);
  
  if((dp = opendir( chhome )) == NULL)
  {
      mkdir(chhome, S_IRUSR|S_IWUSR|S_IXUSR);
  }
}


char * GetImageQuality( char * Param )
{
  char * p = 0;
  
  if(Param == NULL)
  {
    printf("argument is null, please see the procedure TransParam\n");
    return 0;
  }
  
  p = malloc(30);
  if(!strcmp(Param, "Color24")) {
     strcpy(p,"color");
  }
  else if(!strcmp(Param, "Gray8")) {
     strcpy(p, "gray");
  }
  else if(!strcmp(Param, "Lineart")) {
     strcpy(p, "lineart");
  }
  else if(!strcmp(Param, "Normal")) {
     strcpy(p, "100");
  }
  else if(!strcmp(Param, "Fine")) {
     strcpy(p, "200");
  }
  else if(!strcmp(Param, "Super fine")) {
     strcpy(p, "300");
  }
  else {
      g_free(p);
      p = 0;
  }

  return p;
}


void program_to_run(char * path)
{
      pid_t pid;
      gchar * lpchOption;
      
      lpchOption = Option("[Scan]", "ProgramToRun");
      if(lpchOption != NULL)
      {
      	  if (!strcmp(lpchOption, "The GIMP"))  
	  {
	      pid = fork();
	      switch (pid)
	      {
	        case 0:
		     execlp("xterm", "xterm","-iconic", "-e", "gimp", path, NULL);
		     break;
	        case -1:
		     g_print("fork for gimp successful.\n");
		     break;
	        default:
		     g_print("fork for gimp successful. child pid = %d\n", pid);
		     break;
	      }
	  }
  	  else if(!strcmp(lpchOption, "Panel"))  {
	        ShowImage(path);
	  }
      }
      else
      {
	  ShowImage(path);
      }
      
  
      free(lpchOption);
      lpchOption = NULL;

}


int StartScanner(char chFileName[256], char tag[30])
{
  
    char * lpchOption = 0;
    char * lpchResolution = 0;
    char * lpchScanMode = 0;
    int  nResolution;

    if(!strcmp(tag, "[Fax]"))
    {
       lpchScanMode = (char*) malloc(30);
       lpchResolution = (char*) malloc(30);
       strcpy(lpchScanMode, "lineart");
       strcpy(lpchResolution, "300");
    }
    else
    {
      /* Get ScanMode. Return a string */

       lpchOption = Option(tag, "ScanMode");
       lpchScanMode = GetImageQuality(lpchOption);

       /* Get Resolution. Return a string */
       free(lpchOption);
       lpchOption = Option("[Scan]", "Resolution");
       lpchResolution = GetImageQuality(lpchOption);

    }

    /* Transfer string to integer */
    nResolution = atoi(lpchResolution);


    if(lpchScanMode && lpchResolution)
    {
       /* x1,x2,y1,y2 must be integer */
       if(( pos.x1 == pos.x2 ) || ( pos.y1 == pos.y2 ))
       { 	 
	  ProgressWnd (NULL, NULL, chFileName, lpchScanMode, lpchResolution,
		       0, (int)(nResolution*8.5), 0, (int)(nResolution*11.7) );
       }
       else
       {
	  ProgressWnd(NULL, NULL, chFileName, lpchScanMode, lpchResolution,
		      (int)(pos.x1*nResolution/25), (int)(pos.x2*nResolution/25),
		      (int)(pos.y1*nResolution/25), (int)(pos.y2*nResolution/25));
       }

      free(lpchOption);
      free(lpchResolution);
      free(lpchScanMode);
      lpchOption = 0;
      lpchResolution = 0;
      lpchScanMode = 0;

      return(1);
    }
    else
    {
	printf("Some problem occur in function of GetImageQuality\n");
	
	free(lpchOption);
	free(lpchResolution);
	free(lpchScanMode);
	lpchOption = 0;
	lpchResolution = 0;
	lpchScanMode = 0;
	
	return(0);
    }
}


/**
 *
 * MAKE SCRIPT COMMAND
 **/
int MakeScript(char * ScriptName, char * Command)
{
  
      char * lpchUsrHomeDir = NULL;
      char * lpchCommand = NULL;
      int iCountCharInScript = 0;
      int iFile = 0;
      

      lpchCommand = Command;      
      iCountCharInScript=0;
      for ( ; *lpchCommand != '\0'; lpchCommand++) {iCountCharInScript++;}   // find end of string
      
      iFile = open(ScriptName , O_WRONLY|O_CREAT, S_IRUSR|S_IWUSR|S_IXUSR);
      if(iFile)
      {
	 	  
	  // write command to script file. the third argument is size, come from end of string
	  write(iFile, Command, iCountCharInScript); 
	  close(iFile);
	
      }
      else
      {
	free(lpchUsrHomeDir);
	return(0);
      }

      free(lpchUsrHomeDir);
      return(-1);
}


/*
 *
 * CHECKED THE PANEL ALREADY RUNNING OR NOT.
 */
int PanelAlreadyRun()
{

	pid_t my_pid;
	pid_t ppid;

	
	DIR* proc_dir;
	struct dirent *next;
	char s_pid[256];
	char link[256], real[256];
	char my_path[256];
	
	my_pid = getpid();
	ppid = getppid();
	
	sprintf(link, "/proc/%d/exe",my_pid);
	sprintf(s_pid, "%d",my_pid);
	if (realpath(link,my_path) == NULL) {
		printf("Failed to get my executable path, /proc not mounted?\n"); 
		return 1;	
	}
	proc_dir = opendir("/proc/");
	if (!proc_dir) {
		printf("fail to opendir for /proc.\n");
		return 1;
	}
	while ((next=readdir(proc_dir))!=NULL){
		if (strcmp(next->d_name,"self")!= 0 && 
		    strcmp(next->d_name,s_pid)!=0) {
			sprintf(link, "/proc/%s/exe",next->d_name);
			if (realpath(link,real)!=NULL) {
				if (strcmp(real, my_path) == 0) {
				        printf("There is another panel running. Program terminated.\n");
					return 1;
				}
			}
		}
	} 
	closedir(proc_dir);
	return 0;
}



void swap(double *i, double *j)
{
    double temp;

    temp = *i;
    *i = *j;
    *j = temp;
}







