/* BearPaw1200CU panel

   Copyright (C) 2001 Mustek. 
   Maintained by Tom Wang and Alex Lin
   <tom.wang@mustek.com.tw, alex_lin@mustek.com.tw>

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston,
   MA 02111-1307, USA.
*/


/**
 * imageviewerutility.c
 *
 * Utility functions for imageviewer app. 
 **/

#include <gnome.h>

/* Set up pointers to keep track of the last window and image used */
GtkWidget *lastwindow;
GdkImlibImage *lastimage;

/** 
 * SetActiveWindow ()
 * 
 * Used when a window becomes active. Stores its pointer so 
 * the main window GUI can work with it. 
 **/
void SetActiveWindow ( GtkWidget * window, GdkImlibImage * image )
{
   lastimage = image;
   lastwindow = window;
}

/**
 * ClearActiveWindow ()
 * 
 * Used when a window deactivates. Clears the last window 
 * pointer. Normally another window comes active afterwards, 
 * so lastwindow is later checked out as NULL, there are no 
 * graphics loaded. 
 **/
void ClearActiveWindow ()
{
   lastimage = NULL;
   lastwindow = NULL;
}

/**
 * GetActiveWindow ()
 * 
 * Returns a pointer to the last active window. Called from the 
 * mainwindow GUI in order for the GUI to work with the right 
 * window. 
 **/
GtkWidget *GetActiveWindow ()
{
   return lastwindow;
}

/**
 * GetActiveImage ()
 * 
 * Returns a pointer to the last active image. 
 */
GdkImlibImage * GetActiveImage ()
{
   return lastimage;
}
