/* BearPaw1200CU panel

   Copyright (C) 2001 Mustek. 
   Maintained by Tom Wang and Alex Lin
   <tom.wang@mustek.com.tw, alex_lin@mustek.com.tw>

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston,
   MA 02111-1307, USA.
*/


/*

Copyright (c) 1998-2000  Grandtech Semiconductor Corp.

Module Name:

   LIBSCI.H

Notes:

  THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY
  KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
  IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A PARTICULAR
  PURPOSE.

  Copyright (c) 1998-2000 Grandtech Semiconductor Corporation.
  All Rights Reserved.

Author:

  C. C. Chang

Revision History:

    12/20/99 : created

--*/

#ifndef __LIBSCI_H__
#define __LIBSCI_H__
#define WIN32

#include "sci.h"

#ifdef __cplusplus
extern "C" {
#endif


#define SCANNER_DEVICE_MAX_COUNT        32
#define USB_SCANNER_FW_SIZE             8192

#define DEFAULTDPI      600

#define XMaxPixels      (12 * 1200)
#define YMaxPixels      (12 * 1200)


#define COMMAND_SW_GET_FLAG                     0x00
#define COMMAND_SW_SET_FLAG                     0x01
#define COMMAND_SET_SCAN_WINDOW                 0x20
#define COMMAND_SET_AFE_GAIN                    0x22
#define COMMAND_SET_SENSOR_CCD                  0x23
#define COMMAND_GET_SCANNER_INFO                0x23
#define COMMAND_SET_SENSOR_CIS                  0x24
#define COMMAND_MISC_CONTROL                    0x24
#define COMMAND_SET_AFE_GAIN2                   0x25
#define COMMAND_SET_LINE_TIME2                  0x26
#define COMMAND_GET_SENSOR_TIMING               0x27
#define COMMAND_SET_SENSOR_TIMING               0x28
#define COMMAND_WRITE_REGISTER                  0x2A
#define COMMAND_READ_REGISTER                   0x2B
#define COMMAND_SET_SCAN_WINDOW2                0x2C
#define COMMAND_GET_ID                          0x2E
#define COMMAND_SET_MOTOR_HOLD                  0x32
#define COMMAND_SET_MOTOR_HOME                  0x34
#define COMMAND_WAIT_SCANNER_READY              0x35
#define COMMAND_SET_END_SCAN                    0x42
#define COMMAND_SET_START_SCAN                  0x43
#define COMMAND_GET_DATA                        0x44
#define COMMAND_GENERAL_IO                      0x50
#define COMMAND_LIGHT_OFF                       0x51
#define COMMAND_GENERAL_IO_GET_ID               0x60
#define COMMAND_SET_ENDIAN                      0x66
#define COMMAND_ENABLE_FIRMWARE                 0x69
#define COMMAND_STI_GET_ONLINE_STATUS           0x70 // without Firmware
#define COMMAND_STI_DIAGNOSTIC                  0x71 // without Firmware
#define COMMAND_STI_RESET                       0x72 // without Firmware
#define COMMAND_GET_VERSION                     0x73 // without Firmware
#define COMMAND_STI_GETBUTTONSTATUS             0x74 // without Firmware
#define COMMAND_SET_EXPOSURE_TIME               0x76
#define COMMAND_SET_LINE_TIME                   0x77


#define COMMAND_GENERAL_IO_STI_GET_FW_STATUS    0x61
#define COMMAND_GENERAL_IO_STI_ONLINESTATUS     0x70
#define COMMAND_GENERAL_IO_STI_DIAGNOSTIC       0x71
#define COMMAND_GENERAL_IO_STI_RESET            0x72
#define COMMAND_GENERAL_IO_STI_GETSTATUS        0x73


#define COMMAND_MISC_CONTROL_DEBUG_MOTOR                0x0020
#define COMMAND_MISC_CONTROL_DEBUG_MOTOR_SPEED_INIT     0x0000
#define COMMAND_MISC_CONTROL_DEBUG_MOTOR_TARGET_SPEED   0x0001
#define COMMAND_MISC_CONTROL_DEBUG_MOTOR_ADD_VALUE      0x0002
#define COMMAND_MISC_CONTROL_DEBUG_MOTOR_KEEP_TIME      0x0003

#define DEC_SEED 8
#define COMPRESS_BUFFER_SIZE    16384
#define MAXIMUM_HEIGHT          7020


typedef struct _SCANNER_PARAM_HEADER {
    UCHAR  Command;
    UCHAR  SendingIndex;
} SCANNER_PARAM_HEADER, *PSCANNER_PARAM_HEADER;


typedef struct _SCANNER_SET_AFE_GAIN_PARAM {
    UCHAR  R_Offset;
    UCHAR  R_Gain;
    UCHAR  G_Offset;
    UCHAR  G_Gain;
    UCHAR  B_Offset;
    UCHAR  B_Gain;
} SCANNER_SET_AFE_GAIN_PARAM, *PSCANNER_SET_AFE_GAIN_PARAM;

// MODIFYMODIFY : Need More Verification
typedef struct _SCANNER_SET_WINDOW_PARAM {
    USHORT StartY;
    USHORT Height;
    USHORT StartX;
    USHORT Width;
#ifdef WIN32    
    USHORT ScanType      :3;
    USHORT DataBusType   :2;
    USHORT ModeReserved1 :1;
    USHORT CompressType  :1;
    USHORT RawDataFormat :1;
    USHORT TestMode      :1;
    USHORT BufferSize    :1;
    USHORT DataAttribute :1;
    USHORT BusProtocol   :3;
    USHORT DeviceType    :1;
    USHORT ModeReserved3 :1;
#else     
    USHORT ModeReserved3 :1;
    USHORT DeviceType    :1;
    USHORT BusProtocol   :3;
    USHORT DataAttribute :1;
    USHORT BufferSize    :1;
    USHORT TestMode      :1;
    USHORT RawDataFormat :1;
    USHORT CompressType  :1;
    USHORT ModeReserved1 :1;
    USHORT DataBusType   :2;
    USHORT ScanType      :3;
#endif    
    USHORT DPI;
    USHORT PackCnt;
    USHORT LineCnt;
    USHORT HeightCnt;
#ifdef WIN32    
    UCHAR  StillAfterScan   :1;
    UCHAR  StillBeforeScan  :1;
    UCHAR  StillOnScan      :1;
    UCHAR  BackingIfBufFull :1;
    UCHAR  MotorReserved    :4;
    UCHAR  LampOffOnScan    :1;
    UCHAR  AutoBandwidth    :1;
    UCHAR  MiscReserved     :6;
#else
    UCHAR  MiscReserved     :6;
    UCHAR  AutoBandwidth    :1;
    UCHAR  LampOffOnScan    :1;
    UCHAR  MotorReserved    :4;
    UCHAR  BackingIfBufFull :1;
    UCHAR  StillOnScan      :1;
    UCHAR  StillBeforeScan  :1;
    UCHAR  StillAfterScan   :1;    
#endif
    USHORT YDPI;
    USHORT MotorBackwardStepsIfBufferFull;
} SCANNER_SET_WINDOW_PARAM, *PSCANNER_SET_WINDOW_PARAM;


typedef struct _SCANNER_SET_START_SCAN_PARAM {
    USHORT Reserved;
} SCANNER_SET_START_SCAN_PARAM, *PSCANNER_SET_START_SCAN_PARAM;


typedef struct _SCANNER_SET_END_SCAN_PARAM {
    USHORT Reserved;
} SCANNER_SET_END_SCAN_PARAM, *PSCANNER_SET_END_SCAN_PARAM;


typedef struct _SCANNER_GENERAL_IO_PARAM {
    UCHAR  SubCommand;
    UCHAR  SubData[5];
    UCHAR  Reserved[64 - 8];
} SCANNER_GENERAL_IO_PARAM, *PSCANNER_GENERAL_IO_PARAM;

typedef struct _SCANNER_MISC_CONTROL_PARAM {
//  UCHAR  SubCommandB0;
//  UCHAR  SubCommandB1;
//  UCHAR  FunctionB0;
//  UCHAR  FunctionB1;
    UCHAR  Parameter[64 - 2 - 4];
} SCANNER_MISC_CONTROL_PARAM, *PSCANNER_MISC_CONTROL_PARAM;

typedef struct _SCANNER_SEND_USER_COMMAND_PARAM {
    UCHAR  InputData[64 - 2];
} SCANNER_SEND_USER_COMMAND_PARAM, *PSCANNER_SEND_USER_COMMAND_PARAM;


typedef struct _SCANNER_SET_SENSOR_PARAM {
    USHORT Reserved;
} SCANNER_SET_SENSOR_PARAM, *PSCANNER_SET_SENSOR_PARAM;


typedef struct _SCANNER_SET_LAMP_ONOFF_PARAM {
    USHORT Reserved;
} SCANNER_SET_LAMP_ONOFF_PARAM, *PSCANNER_SET_LAMP_ONOFF_PARAM;


typedef struct _SCANNER_DOWNLOAD_FIRMWARE_PARAM {
    PUCHAR Buffer;
} SCANNER_DOWNLOAD_FIRMWARE_PARAM, *PSCANNER_DOWNLOAD_FIRMWARE_PARAM;


typedef struct _SCANNER_GET_FW_INFO_PARAM {
    USHORT Reserved;
} SCANNER_GET_FW_INFO_PARAM, *PSCANNER_GET_FW_INFO_PARAM;


typedef struct _SCANNER_SET_EXPOSURE_TIME_PARAM {
    UCHAR RedStartB0;
    UCHAR RedStartB1;
    UCHAR RedStopB0;
    UCHAR RedStopB1;
    UCHAR GreenStartB0;
    UCHAR GreenStartB1;
    UCHAR GreenStopB0;
    UCHAR GreenStopB1;
    UCHAR BlueStartB0;
    UCHAR BlueStartB1;
    UCHAR BlueStopB0;
    UCHAR BlueStopB1;
} SCANNER_SET_EXPOSURE_TIME_PARAM, *PSCANNER_SET_EXPOSURE_TIME_PARAM;

typedef struct _SCANNER_SET_LINE_TIME_PARAM {
    UCHAR LineTimeB0;
    UCHAR LineTimeB1;
    UCHAR LineTimeB2;
    UCHAR LineTimeB3;
} SCANNER_SET_LINE_TIME_PARAM, *PSCANNER_SET_LINE_TIME_PARAM;

typedef struct _SCANNER_SET_LINE_TIME2_PARAM {
    UCHAR LineTimeB0;
    UCHAR LineTimeB1;
    UCHAR LineTimeB2;
    UCHAR LineTimeB3;
} SCANNER_SET_LINE_TIME2_PARAM, *PSCANNER_SET_LINE_TIME2_PARAM;

typedef struct _SCANNER_SET_AFE_GAIN2_PARAM {
    UCHAR  R_OffsetB0;
    UCHAR  R_OffsetB1;
    UCHAR  R_GainB0;
    UCHAR  R_GainB1;
    UCHAR  G_OffsetB0;
    UCHAR  G_OffsetB1;
    UCHAR  G_GainB0;
    UCHAR  G_GainB1;
    UCHAR  B_OffsetB0;
    UCHAR  B_OffsetB1;
    UCHAR  B_GainB0;
    UCHAR  B_GainB1;
} SCANNER_SET_AFE_GAIN2_PARAM, *PSCANNER_SET_AFE_GAIN2_PARAM;

typedef struct _SCANNER_GET_SENSOR_TIMING_PARAM {
    USHORT Reserved;
} SCANNER_GET_SENSOR_TIMING_PARAM, *PSCANNER_GET_SENSOR_TIMING_PARAM;

typedef struct _SCANNER_SET_SENSOR_TIMING_PARAM {
    UCHAR SensorRedB0;
    UCHAR SensorRedB1;
    UCHAR SensorRedB2;
    UCHAR SensorRedB3;
    UCHAR SensorGreenB0;
    UCHAR SensorGreenB1;
    UCHAR SensorGreenB2;
    UCHAR SensorGreenB3;
    UCHAR SensorBlueB0;
    UCHAR SensorBlueB1;
    UCHAR SensorBlueB2;
    UCHAR SensorBlueB3;
} SCANNER_SET_SENSOR_TIMING_PARAM, *PSCANNER_SET_SENSOR_TIMING_PARAM;

typedef struct _SCANNER_READ_REGISTER_PARAM {
    UCHAR RegisterIndex;
    UCHAR Reserved;
} SCANNER_READ_REGISTER_PARAM, *PSCANNER_READ_REGISTER_PARAM;

typedef struct _SCANNER_WRITE_REGISTER_PARAM {
    UCHAR RegisterIndex;
    UCHAR RegisterValue;
} SCANNER_WRITE_REGISTER_PARAM, *PSCANNER_WRITE_REGISTER_PARAM;

typedef struct _SCANNER_GET_ID_PARAM {
    UCHAR  VIDB0;
    UCHAR  VIDB1;
    UCHAR  PIDB0;
    UCHAR  PIDB1;
    UCHAR  DIDB0;
    UCHAR  DIDB1;
    UCHAR  DIDB2;
    UCHAR  DIDB3;
    UCHAR  FIDB0;
    UCHAR  FIDB1;
} SCANNER_GET_ID_PARAM, *PSCANNER_GET_ID_PARAM;

typedef struct _SCANNER_STI_GET_ONLINE_STATUS_PARAM {
    USHORT Reserved;
} SCANNER_STI_GET_ONLINE_STATUS_PARAM, *PSCANNER_STI_GET_ONLINE_STATUS_PARAM;

typedef struct _SCANNER_STI_DIAGNOSTIC_PARAM {
    USHORT Reserved;
} SCANNER_STI_DIAGNOSTIC_PARAM, *PSCANNER_STI_DIAGNOSTIC_PARAM;

typedef struct _SCANNER_STI_RESET_PARAM {
    USHORT Reserved;
} SCANNER_STI_RESET_PARAM, *PSCANNER_STI_RESET_PARAM;

typedef struct _SCANNER_GET_VERSION_PARAM {
    USHORT Reserved;
} SCANNER_GET_VERSION_PARAM, *PSCANNER_GET_VERSION_PARAM;

typedef struct _SCANNER_STI_GETBUTTONSTATUS_PARAM {
    USHORT Reserved;
} SCANNER_STI_GETBUTTONSTATUS_PARAM, *PSCANNER_STI_GETBUTTONSTATUS_PARAM;

typedef	union {
            struct _SCANNER_SET_AFE_GAIN_PARAM          ScannerSetAFEGainParam;
            struct _SCANNER_SET_WINDOW_PARAM            ScannerSetWindowParam;
            struct _SCANNER_SET_START_SCAN_PARAM        ScannerSetStartScanParam;
            struct _SCANNER_SET_END_SCAN_PARAM          ScannerSetEndScanParam;
            struct _SCANNER_GENERAL_IO_PARAM            ScannerGeneralIOParam;
            struct _SCANNER_SEND_USER_COMMAND_PARAM     ScannerSendUserCommandParam;
            struct _SCANNER_SET_EXPOSURE_TIME_PARAM     ScannerSetExposureTimeParam;
            struct _SCANNER_SET_LINE_TIME_PARAM         ScannerSetLineTimeParam;
            struct _SCANNER_SET_LINE_TIME2_PARAM        ScannerSetLineTime2Param;
            struct _SCANNER_SET_AFE_GAIN2_PARAM         ScannerSetAFEGain2Param;
            struct _SCANNER_MISC_CONTROL_PARAM          ScannerMiscControlParam;
            struct _SCANNER_GET_SENSOR_TIMING_PARAM     ScannerGetSensorTimingParam;
            struct _SCANNER_SET_SENSOR_TIMING_PARAM     ScannerSetSensorTimingParam;
            struct _SCANNER_READ_REGISTER_PARAM         ScannerReadRegisterParam;
            struct _SCANNER_WRITE_REGISTER_PARAM        ScannerWriteRegisterParam;
            struct _SCANNER_GET_FW_INFO_PARAM           ScannerGetFWInfoParam;
            struct _SCANNER_GET_ID_PARAM                ScannerGetIDParam;
            struct _SCANNER_STI_GET_ONLINE_STATUS_PARAM ScannerSTIGetOnLineStatusParam;
            struct _SCANNER_STI_DIAGNOSTIC_PARAM        ScannerSTIDiagnosticParam;
            struct _SCANNER_STI_RESET_PARAM             ScannerSTIResetParam;
            struct _SCANNER_GET_VERSION_PARAM           ScannerGetVersionParam;
}SCANNER_DATA_HEADER;



typedef struct _SCANNER_PARAM {
    SCANNER_PARAM_HEADER        ScannerParamHeader;
    SCANNER_DATA_HEADER		ScannerDataHeader;
} SCANNER_PARAM, *PSCANNER_PARAM;

typedef struct _SCANNER_RETURNED_STATUS {
    UCHAR  Status[64];
} SCANNER_RETURNED_STATUS, *PSCANNER_RETURNED_STATUS;

#define SIZEOF_SCANNER_PARAM_HEADER                     sizeof(SCANNER_PARAM_HEADER)
#define SIZEOF_SCANNER_SET_AFE_GAIN_PARAM               (SIZEOF_SCANNER_PARAM_HEADER + sizeof(SCANNER_SET_AFE_GAIN_PARAM         ))
#define SIZEOF_SCANNER_SET_WINDOW_PARAM                 (SIZEOF_SCANNER_PARAM_HEADER + sizeof(SCANNER_SET_WINDOW_PARAM           ))
#define SIZEOF_SCANNER_SET_START_SCAN_PARAM             (SIZEOF_SCANNER_PARAM_HEADER + sizeof(SCANNER_SET_START_SCAN_PARAM       ))
#define SIZEOF_SCANNER_SET_END_SCAN_PARAM               (SIZEOF_SCANNER_PARAM_HEADER + sizeof(SCANNER_SET_END_SCAN_PARAM         ))
#define SIZEOF_SCANNER_GENERAL_IO_PARAM                 (SIZEOF_SCANNER_PARAM_HEADER + sizeof(SCANNER_GENERAL_IO_PARAM           ))
#define SIZEOF_SCANNER_SEND_USER_COMMAND_PARAM          (SIZEOF_SCANNER_PARAM_HEADER + sizeof(SCANNER_SEND_USER_COMMAND_PARAM    ))
#define SIZEOF_SCANNER_GET_FW_INFO_PARAM                (SIZEOF_SCANNER_PARAM_HEADER + sizeof(SCANNER_GET_FW_INFO_PARAM          ))
#define SIZEOF_SCANNER_SET_EXPOSURE_TIME_PARAM          (SIZEOF_SCANNER_PARAM_HEADER + sizeof(SCANNER_SET_EXPOSURE_TIME_PARAM    ))
#define SIZEOF_SCANNER_SET_LINE_TIME_PARAM              (SIZEOF_SCANNER_PARAM_HEADER + sizeof(SCANNER_SET_LINE_TIME_PARAM        ))
#define SIZEOF_SCANNER_SET_LINE_TIME2_PARAM             (SIZEOF_SCANNER_PARAM_HEADER + sizeof(SCANNER_SET_LINE_TIME2_PARAM       ))
#define SIZEOF_SCANNER_SET_AFE_GAIN2_PARAM              (SIZEOF_SCANNER_PARAM_HEADER + sizeof(SCANNER_SET_AFE_GAIN2_PARAM        ))
#define SIZEOF_SCANNER_MISC_CONTROL_PARAM               (SIZEOF_SCANNER_PARAM_HEADER + sizeof(SCANNER_MISC_CONTROL_PARAM         ))
#define SIZEOF_SCANNER_GET_SENSOR_TIMING_PARAM          (SIZEOF_SCANNER_PARAM_HEADER + sizeof(SCANNER_GET_SENSOR_TIMING_PARAM    ))
#define SIZEOF_SCANNER_SET_SENSOR_TIMING_PARAM          (SIZEOF_SCANNER_PARAM_HEADER + sizeof(SCANNER_SET_SENSOR_TIMING_PARAM    ))
#define SIZEOF_SCANNER_READ_REGISTER_PARAM              (SIZEOF_SCANNER_PARAM_HEADER + sizeof(SCANNER_READ_REGISTER_PARAM        ))
#define SIZEOF_SCANNER_WRITE_REGISTER_PARAM             (SIZEOF_SCANNER_PARAM_HEADER + sizeof(SCANNER_WRITE_REGISTER_PARAM       ))
#define SIZEOF_SCANNER_GET_ID_PARAM                     (SIZEOF_SCANNER_PARAM_HEADER + sizeof(SCANNER_GET_ID_PARAM               ))
#define SIZEOF_SCANNER_STI_GET_ONLINE_STATUS_PARAM      (SIZEOF_SCANNER_PARAM_HEADER + sizeof(SCANNER_STI_GET_ONLINE_STATUS_PARAM))
#define SIZEOF_SCANNER_STI_DIAGNOSTIC_PARAM             (SIZEOF_SCANNER_PARAM_HEADER + sizeof(SCANNER_STI_DIAGNOSTIC_PARAM       ))
#define SIZEOF_SCANNER_STI_RESET_PARAM                  (SIZEOF_SCANNER_PARAM_HEADER + sizeof(SCANNER_STI_RESET_PARAM            ))
#define SIZEOF_SCANNER_GET_VERSION_PARAM                (SIZEOF_SCANNER_PARAM_HEADER + sizeof(SCANNER_GET_VERSION_PARAM          ))
#define SIZEOF_SCANNER_STI_GETBUTTONSTATUS_PARAM        (SIZEOF_SCANNER_PARAM_HEADER + sizeof(SCANNER_STI_GETBUTTONSTATUS_PARAM  ))


typedef struct _SCANNER_WORKING_BUFFER {
    SCANNER_PARAM                ScannerParam;
    SCANNER_RETURNED_STATUS      ScannerReturnedStatus;
    UCHAR   ProcessReadImageBuffer[COMPRESS_BUFFER_SIZE];
    ULONG   PRIBOffset           ; // = 0;
    ULONG   ThisProcessedBytes   ; // = 0;
    ULONG   ThisTimeReadBytes    ; // = 0;
    PUCHAR  ThisTimeReadOffset   ; // = 0;
    ULONG   ProcessWidthIndex    ; // = 0;
    ULONG   ProcessHeightIndex   ; // = 0;
    ULONG   BufferCurrentPointer ; // = 0;
    ULONG   BufferCurrentIndex   ; // = 0;
    USHORT  LineColorMap[MAXIMUM_HEIGHT * 3];
} SCANNER_WORKING_BUFFER, *PSCANNER_WORKING_BUFFER;

UCHAR
qar8adwu132_dcnj(
        IN      PUCHAR  ImageBufferIn
        );
        
BOOL
ey36cbc8_316db(
        IN      PUCHAR  ImageBufferIn
        );
        
BYTE
i126ecb67eddslsdkc(
        IN      ULONG   BitSteps
        );

#ifdef __cplusplus
}
#endif

#endif
