/* BearPaw1200CU panel

   Copyright (C) 2001 Mustek. 
   Maintained by Tom Wang and Alex Lin
   <tom.wang@mustek.com.tw, alex_lin@mustek.com.tw>

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston,
   MA 02111-1307, USA.
*/


/**
 * mainwindow.c
 *
 * Responsible for building the main window of the application
 * and handling its signals, and those of its user interface.
 **/

#include <sys/stat.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <fcntl.h>
#include <gnome.h>
#include <pthread.h>

#include "mainwindow.h"
#include "mainwindowhandlers.h"
#include "imageviewerutility.h"
#include "fileselect.h"
#include "ImageWindow.h"
#include "setting.h"
#include "GetFaxNumber.h"
#include "g_procedure.h"
#include "prevwnd.h"


#define Open 0
#define Scan 1
#define Char_In_Script 300


static char action[256];
gchar lpchFaxNumber[256];
gchar lpchDevice[256];
gchar * lpchNumAndDev = NULL;
pthread_t g_thread;
GtkWidget *mainwindow;
int scan_timer;


/* =============================== MENUBARS ================================ */
/*
static GnomeUIInfo file_menu[] = 
{
   GNOMEUIINFO_MENU_OPEN_ITEM ( FileOpenHandler, NULL ),
   GNOMEUIINFO_MENU_CLOSE_ITEM ( FileCloseHandler, NULL ),
   GNOMEUIINFO_SEPARATOR,
   GNOMEUIINFO_MENU_EXIT_ITEM ( FileExitItemHandler, NULL ),
   GNOMEUIINFO_END
};


static GnomeUIInfo help_menu[] = 
{
   GNOMEUIINFO_MENU_ABOUT_ITEM ( HelpAboutHandler, NULL ),
   GNOMEUIINFO_END
};


static GnomeUIInfo menu_bar[] = 
{
   GNOMEUIINFO_SUBTREE ( "_File", file_menu ),
   GNOMEUIINFO_SUBTREE ( "_Help", help_menu ),
   GNOMEUIINFO_END
};
*/

void PreviewImageOrNot(GtkWidget *widget, gpointer callback_name)
{

  char * lpchOption = 0;

  lpchOption = Option("[Preview]","ScanPreview");

  if(!strcmp(lpchOption, "TRUE")) {
         PreScan(callback_name);
  }
  else 
  {
         if(!strcmp(callback_name, "scan_callback")) {
	     scan_callback( NULL, NULL );
	 }
	 else if(!strcmp(callback_name, "copy_callback") ) {
	     copy_callback( NULL, NULL );
	 }
	 else if(!strcmp(callback_name, "mail_callback")) {
	     mail_callback( NULL, NULL );
	 }
	 else if(!strcmp(callback_name, "fax_callback")) {
	     fax_callback( NULL, NULL );
	 }
  }
}

/* =============================== TOOLBARS ================================ */
static GnomeUIInfo toolbar_icons[] = 
{  
   {  GNOME_APP_UI_ITEM, "Scan", "Scan image", PreviewImageOrNot,
    "scan_callback", NULL, GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_PIXMAP_EXEC},

   {  GNOME_APP_UI_ITEM, "Copy", "Scan, then print",PreviewImageOrNot,
    "copy_callback", NULL, GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_PIXMAP_COPY},

   {  GNOME_APP_UI_ITEM, "Mail",  "Scan, then mail to...", PreviewImageOrNot, 
    "mail_callback", NULL, GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_PIXMAP_MAIL},

   {  GNOME_APP_UI_ITEM, "Fax",   "Scan, then fax to..." , PreviewImageOrNot, 
    "fax_callback", NULL, GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_PIXMAP_PRINT},

   GNOMEUIINFO_SEPARATOR,
   
   {  GNOME_APP_UI_ITEM, "Setting", "Set user's own environment",SettingHandler, NULL, NULL,
    GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_PIXMAP_PROPERTIES},
   
   GNOMEUIINFO_SEPARATOR,

   /*
   {GNOME_APP_UI_ITEM, "Open", "Load up a graphic", FileOpenHandler
                     , NULL, NULL, GNOME_APP_PIXMAP_STOCK
                     , GNOME_STOCK_PIXMAP_OPEN},
   
   {GNOME_APP_UI_ITEM, "Close", "Close the current graphic"
                     , FileCloseHandler, NULL, NULL
                     , GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_PIXMAP_CLOSE},
   GNOMEUIINFO_SEPARATOR,
   
   {GNOME_APP_UI_ITEM, "Rotate", "Rotate image", RotateHandler
                     , NULL, NULL, GNOME_APP_PIXMAP_STOCK
                     , GNOME_STOCK_PIXMAP_REDO},
   {GNOME_APP_UI_ITEM, "Flip", "Flip image horizontally", FlipHandler
                     , GINT_TO_POINTER (0), NULL, GNOME_APP_PIXMAP_STOCK
                     , GNOME_STOCK_PIXMAP_BACK},
   {GNOME_APP_UI_ITEM, "Flip", "Flip image vertically", FlipHandler
                     , GINT_TO_POINTER (1), NULL, GNOME_APP_PIXMAP_STOCK
                     , GNOME_STOCK_PIXMAP_DOWN},
   
   GNOMEUIINFO_SEPARATOR,
   */

   {  GNOME_APP_UI_ITEM, "Exit", "Quit the panel", FileExitItemHandler,
    NULL, NULL, GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_PIXMAP_EXIT},
   
   GNOMEUIINFO_END
};

     

/**=== MAINWINDOW USER INTERFACE HANDLERS ====**/
/**
 * FileExitItemHandler () 
 * 
 * Callback triggered when user hits Exit on the File menu - shuts the 
 * application down after first checking with the user that that is what 
 * they want to do 
 **/
void FileExitItemHandler ( GtkMenuItem * item, gpointer data )
{
   if ( CheckUserExit () )
      gtk_widget_destroy ( mainwindow );
}


/** FileCloseHandler ()
* 
* CallbackTriggered when user hits Close on the File menu - shuts the 
* last selected window down. 
**/
void FileCloseHandler ( GtkMenuItem * item, gpointer data )
{
   CloseActiveImage ();
}


/**
 * HelpAboutHandler ()
 * 
 * Callback triggered when user hits About on the Help menu. Displays the 
 * about box.
 **/
void HelpAboutHandler ( GtkMenuItem * item, gpointer data )
{
   GtkWidget *aboutdialog;
   const gchar *authors[] = { "Tom and Alex", NULL };
   
   aboutdialog = gnome_about_new ( "BearPaw1200CU", "1.0",
   "(C)2001 Open Source", ( const gchar ** ) authors,
   "Welcome to scanner panel - an scanner UI from Mustek",
   NULL );
   
   gtk_widget_show ( aboutdialog );
}


/**
 * FileOpenHandler()
 * 
 * Callback triggered when user hits Open on the File menu - gets the 
 * user to choose a file, and then loads up the image viewer form 
 * with the image loaded inside. 
 **/
void FileOpenHandler ( GtkMenuItem * item, gpointer data )
{
   gchar *filename;
   filename = ChooseFile (Open);

   if ( filename != NULL )
   {
      ShowImage ( filename );
      g_free ( filename );
   }
}


/**
 * RotateHandler()
 * 
 * Callback triggered when user hits the Rotate button on the toolbar. 
 * The code calls the image window's rotate code to rotate the image 
 **/
void RotateHandler ( GtkButton * button, gpointer data )
{
   RotateImage ();
}


/** 
 * FlipHandler ()
 * 
 * Callback triggered when the user hits either of the flip buttons on 
 * the toolbar. The data parameter is converted to an integer, 0 for 
 * horizontal flip, 1 for a vertical flip, before the appropriate code 
 * is called. 
 **/
void FlipHandler ( GtkButton * button, gpointer data )
{
   if ( GPOINTER_TO_INT ( data ) == 0 )
      FlipImage ( TRUE );
   else
      FlipImage ( FALSE );
}


/**=============================== MAINWINDOW SIGNAL HANDLERS =========**/
/**
 * QueryShutdown
 * 
 * When the user tries to close down the main application window, this
 * callback checks whether or not the user is sure about what they want to 
 * do. Assuming they are, the signal is allowed to be handler by GTK which
 * will destroy the window in the usual manner 
 **/
gboolean QueryShutdown ( GtkWidget * widget, GdkEvent * event,
                          gpointer data)
{
   if (g_thread == 0) {
	   return ( !CheckUserExit () );
   } else {
   	return TRUE;
   }
}


/**
 * CloseTheApp ()
 *
 * Stops the application from running when the main window is destroyed.
 * This can only happen after the user has been queried as to whether or 
 * not they are sure they want to quit the app 
 **/
void CloseTheApp ( GtkWidget * widget, gpointer data )
{
   gtk_main_quit ();
}


void configure_callback(GtkWidget* widget, gpointer data)
{
  if( !strcmp(action, "s") ) {
    strcpy(action, "");
    scan_callback(widget, data);
  }
  else if( !strcmp(action, "m") ) {
    strcpy(action, "");
    mail_callback(widget, data);
  }
  else if( !strcmp(action, "f") ) {
    strcpy(action, "");
    fax_callback(widget, data);
  }
  else if( !strcmp(action, "c") ) {
    strcpy(action, "");
    copy_callback(widget, data);
  }

}


/** This function make Hot Key working
 *
 */
void parse_cmd( char arg[256] )
{
  strcpy( action, arg );
}


gint scan_timeout( gpointer lpchFileName )
{
    if(g_fpercent == 100.0)
    {
       gtk_timeout_remove(scan_timer);
       program_to_run(lpchFileName);
       free(lpchFileName);
    }

    return(TRUE);
}


void scan_callback(GtkWidget * widget, gpointer data)
{
  FILE * fCanOpen = 0;
  gchar * lpchFileName = 0;
  

  if (g_thread != 0) return;
  
  lpchFileName = ChooseFile(Scan);
  
  if (!lpchFileName)  //click cancel button
  {
      g_free(lpchFileName);
      lpchFileName = NULL;
      return;
  }
  
  fCanOpen=fopen(lpchFileName, "a");
  if(!fCanOpen)  // permission denied
  {
      GtkWidget * messagebox;
      messagebox=gnome_message_box_new ("Sorry, you specified an invalid path! (Permission denied.)",
					GNOME_MESSAGE_BOX_ERROR,GNOME_STOCK_BUTTON_OK, NULL);
      gnome_dialog_run_and_close (GNOME_DIALOG ( messagebox ));
      free (lpchFileName);
      lpchFileName = NULL;
  }
  else
  {
    StartScanner(lpchFileName, "[Scan]");
    scan_timer = gtk_timeout_add (500, scan_timeout, lpchFileName);
  }
}


void* copy_thread(void* arg)
{
  char chUsrHomeDir[256];
  char * lpchCopyScript = NULL;
  char chCopyScript[Char_In_Script];
  int iCountCharInScript;
  int pid;
  FILE * fCanOpen = NULL;
  

  // MAKE PRINT COMMAND
  strcpy(chCopyScript, "/usr/bin/pnmtops ");
  Get1200UBPath(chUsrHomeDir);
  strcat(chCopyScript, chUsrHomeDir);
  strcat(chCopyScript, "/tmpfile.pnm | lpr");

  
  // GET PICTURE PATH FOR SCAN
  strcat(chUsrHomeDir,"/tmpfile.pnm");

  if(!StartScanner(chUsrHomeDir, "[Scan]"))
  {
      printf("Scan failed\n");
      g_thread = 0;

      return NULL;
  }

  /* Waiting for scanner finish scan */
  do {} while (g_fpercent != 100.0);
   
  pid = fork();
  
  if (pid > 0)
  { 
    //g_print("fork for pnmtops successful. child pid = %d\n", pid);
  }
  else if (pid < 0)
  {
    //g_print("fork for pnmtops failed.\n");
  }
  else
  {      
      fCanOpen=NULL;


      // GET COPY_SCRIPT FILE PATH
      Get1200UBPath(chUsrHomeDir);
      strcat(chUsrHomeDir,"/copy_script");
      lpchCopyScript=chCopyScript;
    
      iCountCharInScript=0;
      for ( ; *lpchCopyScript != '\0'; lpchCopyScript++) {iCountCharInScript++;}   // find end of string
      //g_print("Copy_script has %d charactor\n",iCountCharInScript);

      fCanOpen=fopen(chUsrHomeDir,"r");
      if (!fCanOpen)
      {
	  int out;
	  // create copy_script file, chmod +r+w+x
	  out=open(chUsrHomeDir,O_WRONLY|O_CREAT, S_IRUSR|S_IWUSR|S_IXUSR);
	  // write print command to script file. the third argument is size, come from end of string
	  write(out,chCopyScript, iCountCharInScript); 
	  close(out);
      }
      execlp("xterm", "xterm", "-e", chUsrHomeDir, NULL);
  }


  g_thread = 0;
  return NULL;
}

void copy_callback(GtkWidget *widget, gpointer data)
{
	if (g_thread != 0) return;
	pthread_create(&g_thread, NULL, copy_thread, NULL);
}

void* mail_thread(void* arg)
{
  pid_t pid;
  pid_t jpg_pid;
  char chUsrHomeDir[256];
  char chCommand[256];
  char chScriptName[256];
  char temp[256];
  char chAttachFile[256];


  
  Get1200UBPath(chUsrHomeDir);
  strcat(chUsrHomeDir,"/tmpfile.pnm");

  StartScanner(chUsrHomeDir, "[Mail]");

  do {} while (g_fpercent != 100.0);
  
  // GET COPY_SCRIPT FILE NAME
  Get1200UBPath(chScriptName);
  strcat(chScriptName, "/convert_jpeg");

  
  strcpy(chCommand, "cjpeg ");
  strcat(chCommand, chUsrHomeDir);
  strcat(chCommand, " > ");
  Get1200UBPath(temp);
  strcat(chCommand, temp);
  strcat(chCommand, "/tmpfile.jpeg");
  if(MakeScript(chScriptName, chCommand))
  {
      jpg_pid = fork();
      switch( jpg_pid )
      {
          case 0:
	    execlp("xterm", "xterm", "-e", chScriptName, NULL );
	    break;
          case -1:
	    //g_print("fork for convert_jpeg failed. \n");
	    break;
          default:
	    //g_print("fork for convert_jpeg successful. child pid=%d\n",pid);
	    break;
      }
  }
  else
  {
    //printf("Make script file failed\n");
    return NULL;
  }


  Get1200UBPath(chAttachFile);
  strcat(chAttachFile, "/tmpfile.jpeg");

  
  if(jpg_pid != 0)
  {
     int stat_val;
     pid_t child_pid;

     child_pid = wait(&stat_val);
     if( WIFEXITED(stat_val) )
     {
       pid=fork();
       switch( pid )
	 { 
	 case 0:
	   execlp("xterm", "xterm","-T", "Send this picture to someone",
		  "-e", "/usr/bin/pine" ,"-attach",chAttachFile, NULL);
	   break;
	 case -1:
	   //g_print("fork for pine failed. \n");
	   break;
	 default:
	   //g_print("fork for pine successful. child pid=%d\n",pid);
	   break;
	 }
     }
  }
  g_thread = 0;

  
  return NULL;
}    
  
void mail_callback(GtkWidget *widget, gpointer data)
{
	if (g_thread != 0) return;
	pthread_create(&g_thread, NULL, mail_thread, NULL);  
}

void* fax_thread(void* arg)
{  
  char chCommand[Char_In_Script];
  char chWhereIsTheFile[256];
  char temp[256];
  char * lpchScriptFile = NULL;
  char * lpchImgFile = NULL;
  char * lpchCommand = NULL;
  int iCountCharInCmd;
  int pid;
  int iFile;


  lpchImgFile = "/tmpfile.pnm";   // Image file name
  lpchScriptFile = "/Fax";        // Script file name

  // Get file path for scan
  Get1200UBPath(chWhereIsTheFile);
  strcat(chWhereIsTheFile, lpchImgFile);

  StartScanner(chWhereIsTheFile, "[Fax]");
  do {} while ( g_fpercent != 100.0 );

  pid = fork();
  
  if (pid > 0)
  { 
    //      g_print("fork for pnmtops successful. child pid = %d\n", pid);
  }
  else if (pid < 0)
  {
    //      g_print("fork for pnmtops failed.\n");
  }
  else
  {
      //=======Make the command line which can transfer Image to postscript========//

      Get1200UBPath(temp);

      // Make the command line which can transfer image to postscript
      strcpy(chCommand, "/usr/bin/pnmtops ");
      strcat(chCommand, temp );
      strcat(chCommand, lpchImgFile );
      strcat(chCommand, " > " );
      strcat(chCommand, temp);
      strcat(chCommand, "/tmpfile.ps\n");

      /* Get path of script file */
      Get1200UBPath(chWhereIsTheFile);
      strcat(chWhereIsTheFile,lpchScriptFile);


      // create script file, chmod +r+w+x
      iFile=open(chWhereIsTheFile, O_WRONLY|O_CREAT, S_IRUSR|S_IWUSR|S_IXUSR);

      
      //=======  Write command to file =======//
      // find end of string
      iCountCharInCmd=0;
      lpchCommand=chCommand;
      for ( ; *lpchCommand != '\0'; lpchCommand++) {iCountCharInCmd++;}
      // Write command to file. The third argument is size, come from end of string
      write(iFile,chCommand, iCountCharInCmd); 

      
      //========Make the command line which can transfer postscript to TiffG3=========//
      //=================This paragraph of code similiar with above===================//
      
      strcpy(chCommand, "gs -q -sDEVICE=tiffg3 -dNOPAUSE -sOutputFile=");
      strcat(chCommand, temp);
      strcat(chCommand, "/tmpfile.%03d ");
      strcat(chCommand, temp);
      strcat(chCommand, "/tmpfile.ps </dev/null\n");
      
      iCountCharInCmd=0;
      lpchCommand=chCommand;
      for ( ; *lpchCommand != '\0'; lpchCommand++) { iCountCharInCmd++; }
            
      write( iFile, chCommand, iCountCharInCmd ); 
     
      
      //=====================Make the command line which can fax =====================//
      //=================This paragraph of code similiar with above===================//

      strcpy( chCommand, "efax -d " );
      strcat( chCommand, lpchDevice );
      strcat( chCommand, " -t " );
      strcat( chCommand, lpchFaxNumber );
      strcat( chCommand, " ");
      strcat( chCommand, temp );
      strcat( chCommand, "/tmpfile.001" );

      iCountCharInCmd=0;
      lpchCommand=chCommand;
      for ( ; *lpchCommand != '\0'; lpchCommand++) { iCountCharInCmd++; }

      write( iFile, chCommand, iCountCharInCmd );
      close( iFile );

      Get1200UBPath(chWhereIsTheFile);
      strcat( chWhereIsTheFile, lpchScriptFile );

      execlp("xterm", "xterm", "-e", chWhereIsTheFile, NULL);
  }
    
    g_thread = 0;
        
    return NULL;
}

void fax_callback(GtkWidget *widget, gpointer data)
{
  
    if (g_thread != 0) return;
  
  
    // Get device name and fax number.
    strcpy(lpchDevice,"");
    strcpy(lpchFaxNumber,"");
    lpchNumAndDev = GetFaxNumberDialog();    // Ask user the fax number and fax device.
    sscanf( lpchNumAndDev, "%s %s",lpchDevice, lpchFaxNumber );

    if((strlen( lpchDevice ) == 0) || (strlen( lpchFaxNumber ) == 0))
    {
      /* Do not free lpchNumAndDev. It point to a charactor variable. */
	lpchNumAndDev = NULL;
	g_thread = 0;
        return; 
    }
	
    pthread_create(&g_thread, NULL, fax_thread, NULL);
}

     
void SettingHandler(gint argc, gchar *argv[])
{
  SettingDialog();
}



/**====== MAINWINDOW UTILITY FUNCTIONS ======**/
/** 
 * BuildMainWindow 
 * 
 * Builds up the main window and it's user interface 
 **/
void BuildMainWindow ( gchar * appname, gchar * appversion )
{

   g_thread = 0;

   /* Create the GnomeApp widget and set it up at the top of the screen */
   mainwindow = gnome_app_new ( appname, "BearPaw1200CU Panel" );

     
   gtk_window_set_default_size ( GTK_WINDOW ( mainwindow ),
                                 gdk_screen_width () - 400, 80 );

   //   gtk_widget_set_uposition ( mainwindow, 0, 0 );
   gtk_window_set_position( GTK_WINDOW ( mainwindow ), GTK_WIN_POS_MOUSE );

   gtk_window_set_policy( GTK_WINDOW( mainwindow ), FALSE, FALSE, FALSE );
   
   /* Next, connect up the main window signal handlers */
   gtk_signal_connect ( GTK_OBJECT ( mainwindow ),
			"delete_event", GTK_SIGNAL_FUNC (QueryShutdown), NULL);

   gtk_signal_connect (GTK_OBJECT (mainwindow),
		       "destroy", GTK_SIGNAL_FUNC (CloseTheApp), NULL);

   gtk_signal_connect (GTK_OBJECT (mainwindow),
		       "configure_event", GTK_SIGNAL_FUNC (configure_callback), NULL);
   
   /** Build up the menu bar **/
   //   gnome_app_create_menus (GNOME_APP (mainwindow), menu_bar);

   /** Build up the toolbar **/
   gnome_app_create_toolbar ( GNOME_APP (mainwindow),  toolbar_icons );
      
   /* Finally, show the window */
   gtk_widget_show_all ( mainwindow );
 
   /* Clear the active window pointer, since the app just got going */
   ClearActiveWindow ();


}


/**
 * CheckUserExit()
 *
 * Puts up a message box asking the user if they really want to quit. 
 * Returns TRUE if they do. 
 */
gboolean CheckUserExit ()
{
   GtkWidget *msgbox;
   
   if (g_thread != 0) {
     g_print("g_thread != 0\n");
     return FALSE;
   }

   msgbox = gnome_message_box_new ( "Are you sure you want to quit ?",
             GNOME_MESSAGE_BOX_QUESTION, GNOME_STOCK_BUTTON_YES,
             GNOME_STOCK_BUTTON_NO, NULL);
   gtk_window_set_position ( GTK_WINDOW ( msgbox ), GTK_WIN_POS_CENTER );
   
   /* If the user clicked on the first button ( the YES button), return
    * TRUE  */
   return ( gnome_dialog_run_and_close ( GNOME_DIALOG ( msgbox ) ) == 0 );
}




