/* BearPaw1200CU panel

   Copyright (C) 2001 Mustek. 
   Maintained by Tom Wang and Alex Lin
   <tom.wang@mustek.com.tw, alex_lin@mustek.com.tw>

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston,
   MA 02111-1307, USA.
*/


#include <gnome.h>

#include "g_procedure.h"
#include "mainwindow.h"
#include "progresswnd.h"


#define EVENT_METHOD(i, x) GTK_WIDGET_CLASS(GTK_OBJECT(i)->klass)->x


GnomeCanvasItem * image_item;
GnomeCanvasItem * rectangle_item;
GtkWidget * PrevWnd;
GnomeCanvasGroup * group;
GdkImlibImage * image;
char chUsrHomeDir[256];




void adjust_region(void)
{
  pos.x1 = 0.0;
  pos.x2 = 212.0;
  pos.y1 = 0.0;
  pos.y2 = 292.0;
}


gboolean preview_window_exist()
{
  if(PrevWnd) {
     return(TRUE);
  } else {
    return(FALSE);
  }

}

void on_change_mouse_cursor (GtkWidget * widget, gpointer user_data)
{
  GdkCursor *cursor;
  
  cursor = gdk_cursor_new (GDK_TCROSS); // GDK_TCROSS = 130
  gdk_window_set_cursor (widget->window, cursor);
  gdk_cursor_destroy (cursor);
}


gboolean on_scan_button_clicked ( GtkButton * button, gpointer callback_name )
{
    
    gtk_widget_set_sensitive( GTK_WIDGET( PrevWnd ), FALSE );

    if( pos.x2 < pos.x1 ) {
        swap( &pos.x2, &pos.x1 );
    }
    
    if( pos.y2 < pos.y1 ) {
        swap( &pos.y2, &pos.y1 );
    }
    

    if( !strcmp( callback_name, "scan_callback" ) ) {
        scan_callback( NULL, NULL );
    }
    else if ( !strcmp( callback_name, "copy_callback" ) ) {
        copy_callback( NULL, NULL );
    }
    else if ( !strcmp( callback_name, "mail_callback" ) ) {
        mail_callback( NULL, NULL );
    }
    else if ( !strcmp( callback_name, "fax_callback" ) ) {
        fax_callback ( NULL, NULL );
    }

    gtk_widget_set_sensitive( GTK_WIDGET( PrevWnd ), TRUE );

    
    return(TRUE);
}



gboolean on_preview_button_clicked( GtkButton *button, gpointer callback_name )

{
  
  /* Refresh coordinate */
  adjust_region();

  /* Create progress window and start scanner */
  ProgressWnd( PrevWnd, group, chUsrHomeDir, "color", "25", 0, 212, 0, 292);

  if( image ) {
      gdk_imlib_kill_image(image);
      image = NULL;
  }
  
  if( image_item  ) {
      gtk_object_destroy(GTK_OBJECT(image_item));
      image_item = NULL;
  }

  if( rectangle_item ) {
      gtk_object_destroy(GTK_OBJECT(rectangle_item));
      rectangle_item = NULL;
  }

  return(TRUE);
}
  


gboolean on_cancel_button_clicked ( GtkButton * button, gpointer image_data )
{

  adjust_region();
  
  
  if( rectangle_item ) {
      gtk_object_destroy(GTK_OBJECT(rectangle_item));
      rectangle_item = NULL;
  }
  
  if( image_item ) {
      gtk_object_destroy( GTK_OBJECT(image_item) );
      image_item = NULL;
  }
  
  if( image ) {
      gdk_imlib_kill_image( image );
      image = NULL;
  }

  destroy_image_data_in_progresswnd();
  
  gtk_widget_destroy(PrevWnd);
  PrevWnd = 0;
  
  return(TRUE);
}


gboolean prevwnd_delete_event ( GtkWidget * window,GdkEvent * event, gpointer data )
{
  return(TRUE);
}

gboolean draw_rectangle_handler( GtkWidget *canvas, GdkEvent *event,
				 GnomeCanvasGroup *group )
{

  switch(event->type)
  {
       case GDK_MOTION_NOTIFY:

	    /* DRAW RECTANGLE
	     *
	     **/
	    if((event->motion.state & GDK_BUTTON1_MASK))  // Drag mouse
	    {
	        if( rectangle_item == NULL)
		{
		    
		    /* Buttom_right -> Top_left */
		    if(( pos.x1 >= event->button.x ) && ( pos.y1 >= event->button.y ))
		    {
		         rectangle_item = gnome_canvas_item_new (
					  group,
					  gnome_canvas_rect_get_type(),
					  "x1", event->button.x,
					  "x2", pos.x1,
					  "y1", event->button.y,
					  "y2", pos.y1,
					  "fill_color", NULL,
					  "outline_color", "gray",
					  "width_units", 0.5,
					  NULL );
		    }
		    /* Buttom_left -> Top_right */
		    else if(( pos.x1 <= event->button.x ) && ( pos.y1 >= event->button.y ))  
		    {		     
		         rectangle_item = gnome_canvas_item_new (
					  group,
					  gnome_canvas_rect_get_type(),
					  "x1", event->button.x,
					  "x2", pos.x1,
					  "y1", event->button.y,
					  "y2", pos.y1,
					  "fill_color", NULL,
					  "outline_color", "gray",
					  "width_units", 0.5,
					  NULL );
		    }
		    /* Top_right -> Buttom_left */
		    else if(( pos.x1 >= event->button.x ) && ( pos.y1 <= event->button.y ))
		    {
		        rectangle_item = gnome_canvas_item_new (
					 group,
					 gnome_canvas_rect_get_type(),
					 "x1", event->button.x,
					 "x2", pos.x1,
					 "y1", pos.y1,
					 "y2", event->button.y,
					 "fill_color", NULL,
					 "outline_color", "gray",
					 "width_units", 0.5,
					 NULL );
		    }
		    /* Top_left -> Bottom_right */
		    else if(( pos.x1 <= event->button.x ) && ( pos.y1 <= event->button.y ))
		    { 
		        rectangle_item = gnome_canvas_item_new (
		                         group,
					 gnome_canvas_rect_get_type(),
					 "x1", pos.x1,
					 "x2", event->button.x,
					 "y1", pos.y1,
					 "y2", event->button.y,
					 "fill_color", NULL,
					 "outline_color", "gray",
					 "width_units", 0.5,
					 NULL );
		    }
		}


		/* RESIZE RECTANGLE
		 *
		 **/
		else
		{
		  //if( event->button.x > 212 ) { event->button.x = 210; }
		  //if( event->button.y > 292 ) { event->button.y = 290; }
		  
		  /* Buttom_right -> Top_left */
		   if(( pos.x1 >= event->button.x ) && ( pos.y1 >= event->button.y ))
		   {
		        gnome_canvas_item_set( rectangle_item,
					       "x1", event->button.x,
					       "x2", pos.x1,
					       "y1", event->button.y,
					       "y2", pos.y1,
					       NULL );
		   }
		   /* Buttom_left -> Top_right */
		   else if(( pos.x1 <= event->button.x ) && ( pos.y1 >= event->button.y ))
		   {

			    gnome_canvas_item_set( rectangle_item,
						   "x1", pos.x1,
						   "x2", event->button.x,
						   "y1", event->button.y,
						   "y2", pos.y1,
						   NULL );
		   }
		   /* Top_right -> Buttom_left */
		   else if(( pos.x1 >= event->button.x ) && ( pos.y1 <= event->button.y ))
		   {
		            gnome_canvas_item_set( rectangle_item,
						   "x1", event->button.x,
						   "x2", pos.x1,
						   "y1", pos.y1,
						   "y2", event->button.y,
						   NULL );
		   }
		   /* Top_left -> Bottom_right */
		   else if(( pos.x1 <= event->button.x ) && ( pos.y1 <= event->button.y ))
		   {
		            gnome_canvas_item_set( rectangle_item,
						   "x1", pos.x1,
						   "x2", event->button.x,
						   "y1", pos.y1,
						   "y2", event->button.y,
						   NULL );
		   }
		}
	    }
	    break;

       
       case GDK_BUTTON_PRESS:
	    if( rectangle_item != NULL ) { 
	        gtk_object_destroy( GTK_OBJECT(rectangle_item) );
		rectangle_item = NULL;
	    }

	    pos.x1 = event->button.x;
	    pos.y1 = event->button.y;
	    break;
	    

       case GDK_ENTER_NOTIFY:
	    on_change_mouse_cursor (canvas, NULL);
	    break;

	    
       case GDK_BUTTON_RELEASE:
	    if( event->button.x > 212.0 ) { event->button.x = 212.0; }
	    if( event->button.x < 0.0 ) { event->button.x = 0.0; }
	    if( event->button.y > 292.0 ) { event->button.y = 292.0; }
	    if( event->button.y < 0.0 ) { event->button.y = 0.0; }
	    pos.x2 = event->button.x;
	    pos.y2 = event->button.y;
	    break;

	    
       default:
            break;
  }

  return(TRUE);
}

     
void PreScan (gpointer callback_name)
{

  FILE * lpfp = NULL;
  GtkWidget *table, *view_table;
  GtkWidget *vbox;
  GtkWidget *canvas;
  GtkWidget *hruler, *vruler;
  GtkWidget *Scan_button, *Cancel_button, *Preview_button;




  /*================================ Window ================================*/

  ////////// Main window

  if( preview_window_exist() ) {
      return;
  }

  PrevWnd = gtk_window_new (GTK_WINDOW_TOPLEVEL);
  gtk_object_set_data (GTK_OBJECT (PrevWnd), "PrevWnd", PrevWnd);
  gtk_widget_set_usize (PrevWnd, 237, 345);
  gtk_window_set_title (GTK_WINDOW (PrevWnd), _("Preview window"));
  gtk_window_set_position (GTK_WINDOW (PrevWnd), GTK_WIN_POS_MOUSE);
  gtk_window_set_policy(GTK_WINDOW(PrevWnd), FALSE, FALSE, FALSE );
  //  gtk_window_set_modal (GTK_WINDOW (PrevWnd), TRUE);

  vbox = gtk_vbox_new (FALSE, 6);
  gtk_widget_ref (vbox);
  gtk_object_set_data_full (GTK_OBJECT (PrevWnd), "vbox", vbox,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (vbox);
  gtk_container_add (GTK_CONTAINER (PrevWnd), vbox);
  gtk_widget_set_usize (vbox, 66, 39);
  gtk_container_set_border_width (GTK_CONTAINER (vbox), 3);



  ////////// View_table

  view_table = gtk_table_new (2, 2, FALSE);
  gtk_widget_ref (view_table);
  gtk_object_set_data_full (GTK_OBJECT (PrevWnd), "view_table", view_table,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (view_table);
  gtk_box_pack_start (GTK_BOX (vbox), view_table, FALSE, TRUE, 0);
  


  ////////// Image
  Get1200UBPath(chUsrHomeDir);
  strcat( chUsrHomeDir, "/preview_img.pnm" );
  if( ( lpfp = fopen( chUsrHomeDir, "r" ) ) != NULL )
  {
       image = gdk_imlib_load_image ( chUsrHomeDir );
       fclose(lpfp);
  }

 
  
  ////////// Canvas

  gtk_widget_push_visual (gdk_imlib_get_visual ()); 
  gtk_widget_push_colormap (gdk_imlib_get_colormap ()); 
  canvas = gnome_canvas_new ();
  gtk_widget_pop_colormap ();
  gtk_widget_pop_visual ();
  gtk_widget_ref (canvas);
  gtk_object_set_data_full (GTK_OBJECT (PrevWnd), "canvas", canvas,
			    (GtkDestroyNotify) gtk_widget_unref);

  gtk_table_attach (GTK_TABLE (view_table), canvas, 1, 2, 1, 2,
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                    (GtkAttachOptions) (GTK_FILL), 0, 0);
  gtk_widget_set_usize (canvas, 212, 292);
  gnome_canvas_set_scroll_region (GNOME_CANVAS (canvas), 0, 0, 212, 292);


  
  ////////// Canvas item

  group = gnome_canvas_root ( GNOME_CANVAS ( canvas ) );
  if(image)
  {
     image_item = gnome_canvas_item_new ( group,
					  gnome_canvas_image_get_type(),
					  "image", image,
					  /* Window region is depend on image size */ 
					  "x", ((double)image->rgb_width) / 2.0 , 
					  "y", ((double)image->rgb_height) / 2.0,
					  "anchor", GTK_ANCHOR_CENTER,
					  "width", ((double)image->rgb_width),
					  "height",((double)image->rgb_height),
					  NULL);
  }
  
  gtk_signal_connect ( GTK_OBJECT (canvas), "event",
		       GTK_SIGNAL_FUNC( draw_rectangle_handler), group );

  gtk_widget_show (canvas);
  


  ////////// Ruler
  
  hruler = gtk_hruler_new ();
  gtk_widget_ref (hruler);
  gtk_object_set_data_full (GTK_OBJECT (PrevWnd), "hruler", hruler,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (hruler);
  gtk_table_attach (GTK_TABLE (view_table), hruler, 1, 2, 0, 1,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);
  gtk_ruler_set_range (GTK_RULER (hruler), 0, 8.5, 0.556769, 0);
  
  gtk_signal_connect_object( GTK_OBJECT(canvas), "event",
			     (GtkSignalFunc)EVENT_METHOD(hruler, motion_notify_event),
			     GTK_OBJECT(hruler) );
  
  vruler = gtk_vruler_new ();
  gtk_widget_ref (vruler);
  gtk_object_set_data_full (GTK_OBJECT (canvas), "vruler", vruler,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (vruler);
  gtk_table_attach (GTK_TABLE (view_table), vruler, 0, 1, 1, 2,
                    (GtkAttachOptions) (0),
                    (GtkAttachOptions) (GTK_FILL), 0, 0);
  gtk_ruler_set_range (GTK_RULER (vruler), 0, 11.7, 3.95548, 0);
  gtk_signal_connect_object( GTK_OBJECT(canvas), "event",
			     (GtkSignalFunc)EVENT_METHOD(vruler,motion_notify_event ),
			     GTK_OBJECT(vruler) );

  

  ////////// button

  table = gtk_table_new (1, 2, TRUE);
  gtk_widget_ref (table);
  gtk_object_set_data_full (GTK_OBJECT (PrevWnd), "table", table,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (table);
  gtk_box_pack_start (GTK_BOX (vbox), table, TRUE, TRUE, 0);
  gtk_table_set_row_spacings (GTK_TABLE (table), 3);
  gtk_table_set_col_spacings (GTK_TABLE (table), 4);

  Scan_button = gtk_button_new_with_label (_("Next"));
  gtk_widget_ref (Scan_button);
  gtk_object_set_data_full (GTK_OBJECT (PrevWnd), "Scan_button", Scan_button,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (Scan_button);
  gtk_table_attach (GTK_TABLE (table), Scan_button, 0, 1, 0, 1,
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                    (GtkAttachOptions) (GTK_EXPAND), 0, 0 );

  Preview_button = gtk_button_new_with_label (_("Preview"));
  gtk_widget_ref (Preview_button);
  gtk_object_set_data_full (GTK_OBJECT (PrevWnd), "Preview_button", Preview_button,
			    (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (Preview_button);
  gtk_table_attach (GTK_TABLE (table), Preview_button, 1, 2, 0, 1,
		    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
		    (GtkAttachOptions) (GTK_EXPAND), 0, 0 );
  
  Cancel_button = gtk_button_new_with_label (_("Cancel"));
  gtk_widget_ref (Cancel_button);
  gtk_object_set_data_full (GTK_OBJECT (PrevWnd), "Cancel_button", Cancel_button,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (Cancel_button);
  gtk_table_attach (GTK_TABLE (table), Cancel_button, 2, 3, 0, 1,
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                    (GtkAttachOptions) (GTK_EXPAND), 0, 0);

  gtk_signal_connect (GTK_OBJECT (PrevWnd), "delete_event",
                      GTK_SIGNAL_FUNC (prevwnd_delete_event),
                      NULL);
  gtk_signal_connect (GTK_OBJECT (Cancel_button), "clicked",
                      GTK_SIGNAL_FUNC (on_cancel_button_clicked),
                      NULL);
  gtk_signal_connect (GTK_OBJECT (Scan_button), "clicked",
		      GTK_SIGNAL_FUNC (on_scan_button_clicked),
		      callback_name);
  gtk_signal_connect (GTK_OBJECT (Preview_button), "clicked",
		      GTK_SIGNAL_FUNC (on_preview_button_clicked),
		      callback_name);

  gtk_widget_show(PrevWnd);
}
