/* BearPaw1200CU panel

   Copyright (C) 2001 Mustek. 
   Maintained by Tom Wang and Alex Lin
   <tom.wang@mustek.com.tw, alex_lin@mustek.com.tw>

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston,
   MA 02111-1307, USA.
*/


/** 
 * progresswnd.c
 *
 * Code to implement a progress widget
 */

#include <gnome.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <fcntl.h>
#include <pthread.h>

#include "g_procedure.h"

char PIPE_FILE[256];

typedef struct _ProgressData
{
  GtkWidget * window;
  GdkImlibImage * new_image;
  GnomeCanvasItem * new_image_item;
  int timer;
  pthread_t g_pipe_thread;

  char resoulation[30];
  char scan_mode[30];
  int x1, x2;
  int y1, y2;
  
  /* Come from preview window */
  GtkWidget * prevwnd;
  GnomeCanvasGroup * preview_group;
  char path[256];
}
ProgressData;

  
ProgressData in_progress;


void destroy_image_data_in_progresswnd()
{
  if(in_progress.new_image_item) {
     gtk_object_destroy(GTK_OBJECT(in_progress.new_image_item));
     in_progress.new_image_item = NULL;
  }
  if(in_progress.new_image) {
     gdk_imlib_kill_image(in_progress.new_image);
     in_progress.new_image = NULL;
  }  
}


gboolean progresswnd_delete_event ( GtkWidget * window,GdkEvent * event, gpointer data )
{
  return(TRUE);
}


void ButtonClicked ( GtkButton * button, gpointer progress )
{

  /* First, find out the current value */
  gfloat value = gtk_progress_get_value ( GTK_PROGRESS ( progress ) );

  /* Add 10 to the value and work out the perecentage */
  gfloat percent = gtk_progress_get_percentage_from_value ( GTK_PROGRESS ( progress ), value + 10 );

  /* Update the progress bar with the new percentage */
  gtk_progress_bar_update ( GTK_PROGRESS_BAR ( progress ), percent );
}


gint progress_timeout( gpointer progress )
{
 
  /* Set the new value */
  gtk_progress_set_value (GTK_PROGRESS (progress), g_fpercent);

  if ( g_fpercent == 100.0 )
  {    
       gtk_timeout_remove (in_progress.timer);
       
       if(in_progress.prevwnd) {
	  gtk_widget_set_sensitive( GTK_WIDGET(in_progress.prevwnd), TRUE );
	  //gtk_window_set_modal (GTK_WINDOW (in_progress.prevwnd), TRUE);
       }
    
       gtk_widget_destroy (in_progress.window); 
           
       if(in_progress.preview_group)
       {
	   if(in_progress.new_image_item) {
	       gtk_object_destroy(GTK_OBJECT(in_progress.new_image_item));
	   }

	   if(in_progress.new_image) {
	       gdk_imlib_kill_image(in_progress.new_image);
	   }
	   
	   in_progress.new_image = gdk_imlib_load_image(in_progress.path);
	   in_progress.new_image_item = gnome_canvas_item_new (
			                in_progress.preview_group,
					gnome_canvas_image_get_type(),
					"image", in_progress.new_image,
					"x", ((double)in_progress.new_image->rgb_width) / 2.0, 
					"y", ((double)in_progress.new_image->rgb_height) / 2.0,
					"anchor", GTK_ANCHOR_CENTER,
					"width", ((double)in_progress.new_image->rgb_width),
					"height",((double)in_progress.new_image->rgb_height),
					NULL );
       }
      
       in_progress.g_pipe_thread = 0;
       in_progress.preview_group = NULL;
       in_progress.prevwnd = NULL;
  }
  
  /* As this is a timeout function, return TRUE so that it
   * continues to get called */
  return(TRUE);
} 


void* open_pipe_thread(void* arg)
{
  
  unsigned char data;
  int pipe;
  int try_count;

  
  pipe = open(PIPE_FILE,O_RDONLY);
  if (pipe == -1)
  {
      printf("can not open pipe.\n");
      exit(1);
      return 0;
  }


  try_count = 0;
  while (1) 
  {	
      // to prevent entering infinite loop by accident
      if (try_count > 10000) break;
      read(pipe,&data,1);
      //printf("scan progress to %d\n", data);
      g_fpercent = (gfloat)data;
      if (data == 100) break;
      try_count++;
  }  
  close(pipe);
  

  try_count = 0;
  while (access(PIPE_FILE,F_OK)!=-1 && try_count < 40) {
         remove(PIPE_FILE);
	 try_count++;
  }
  

  return NULL;
}



/* Start scanner and call time_out function to
 * update the progress bar
 */
void Engage(GtkWidget * progress )
{

  int res;
  pid_t fork_result;
  char frame[256];
  char exe_path[256];
  

  if (access(PIPE_FILE, F_OK) == -1) {
          res = mkfifo(PIPE_FILE,0777);
	  if (res != 0) {
	          fprintf(stderr, "Could not create pipe %s\n",PIPE_FILE);
		  return;
	  }
  }

  fork_result = fork();	
  if (fork_result == (pid_t) -1) {
	fprintf(stderr, "Fork failure.\n");
	return;
  }
  if (fork_result == (pid_t) 0) { // child process... execute the scan process
      // scan a resolution 100 file with color mode to oo.pnm file, progress pipe is PIPE_FILE
      sprintf(frame, "%d,%d,%d,%d",in_progress.x1, in_progress.x2, in_progress.y1, in_progress.y2);
      GetExePath(exe_path);
      if(strlen(exe_path) != 0)
      {
	 if(!chdir(exe_path)) {
	    strcat(exe_path, "ps1");
	    execlp( exe_path,
		    exe_path,
		    "-r",in_progress.resoulation,
		    "-s",in_progress.scan_mode,
		    "-f",in_progress.path,
		    "-p",PIPE_FILE,
		    "-frame", frame,
		    NULL );
	 }
	 else {
	       g_print("<chdir> failed in progrsswnd.c file.\n");
	       gtk_main_quit();
	 }
      }
      else
      {
	 g_print("Get executable path failed\n");
	 gtk_main_quit();
      }
  }
  else
  {
      pthread_create(&in_progress.g_pipe_thread, NULL, open_pipe_thread, NULL);
  }
	
  
  /* Add a timer callback to update the value of the progress bar */
  in_progress.timer = gtk_timeout_add (100, progress_timeout, progress);
}



gint ProgressWnd (GtkWidget * widget, GnomeCanvasGroup * group,
		  char * lpchUsrHomeDir, char * lpchScanMode,
		  char * lpchResoulation,
		  int x1, int x2, int y1, int y2 )
{

  //GtkWidget * button;
  GtkWidget * vbox;
  GtkWidget * progress;
  GtkAdjustment * adj;
  
  g_fpercent = 0.0;
  in_progress.prevwnd = widget;
  in_progress.preview_group = group;
  strcpy(in_progress.path, lpchUsrHomeDir);
  strcpy(in_progress.scan_mode, lpchScanMode);
  strcpy(in_progress.resoulation, lpchResoulation);
  in_progress.x1 = x1;
  in_progress.x2 = x2;
  in_progress.y1 = y1;
  in_progress.y2 = y2;


  Get1200UBPath(PIPE_FILE);
  strcat(PIPE_FILE, "/ProgressPile");

  /*  Disable mother window */
  if(in_progress.prevwnd)
     gtk_widget_set_sensitive( GTK_WIDGET( in_progress.prevwnd ), FALSE );
  
  /* Set up the window in the usual way */
  in_progress.window = gtk_window_new ( GTK_WINDOW_TOPLEVEL );
  gtk_window_set_title ( GTK_WINDOW ( in_progress.window ), "Scanning..." );
  gtk_container_set_border_width ( GTK_CONTAINER ( in_progress.window ), 10 );
  gtk_window_set_default_size ( GTK_WINDOW ( in_progress.window ), 320, 40 );
  gtk_window_set_position (GTK_WINDOW (in_progress.window), GTK_WIN_POS_CENTER);
  gtk_window_set_modal (GTK_WINDOW ( in_progress.window ), TRUE);

  /* Set up a vbox to hold the progress bar and button */
  vbox = gtk_vbox_new ( FALSE, 5 );

  /* Create a Adjusment object to hold the range of the progress bar */
  adj = (GtkAdjustment *) gtk_adjustment_new (0, 0, 100, 0, 0, 0);
  
  /* Add a progress bar to the vbox */
  progress = gtk_progress_bar_new_with_adjustment (adj);
  gtk_progress_set_show_text ( GTK_PROGRESS ( progress ), TRUE );
  gtk_box_pack_start ( GTK_BOX ( vbox ), progress, FALSE, FALSE, 5 );
  gtk_widget_show( progress );

  /* Add a button to the box to... the progress bar */
  //  button = gtk_button_new_with_label ( "Update Progress Bar" );
  //  gtk_box_pack_start ( GTK_BOX ( vbox ), button, FALSE, FALSE, 5);
   
  gtk_container_add ( GTK_CONTAINER ( in_progress.window ), vbox );

  /* Connect the signal handlers */
  gtk_signal_connect ( GTK_OBJECT ( in_progress.window ), "delete_event", 
		       GTK_SIGNAL_FUNC ( progresswnd_delete_event ), 
		       NULL );
  /*
  gtk_signal_connect ( GTK_OBJECT ( button ), 
		       "clicked", 
		       GTK_SIGNAL_FUNC ( ButtonClicked ), 
		       progress );
  */
  Engage(progress);
  gtk_widget_show_all ( in_progress.window );
  
  
  return 0;

}


