/* BearPaw1200CU panel

   Copyright (C) 2001 Mustek. 
   Maintained by Tom Wang and Alex Lin
   <tom.wang@mustek.com.tw, alex_lin@mustek.com.tw>

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston,
   MA 02111-1307, USA.
*/


#include <fcntl.h>
#include "BaseTypedef.h"
#include "scan_module.h"

void help(char*);

int main (int argc, char* argv[])
{
	UINT time;
	int scan_mode;
	char path[256];
	char pipe_name[256];
	int pipe;
	int resolution;
	int i;
	int x1, x2, y1, y2;
	
	time = GetTickCount();
	resolution = 100;
	sprintf(path, "%sresult.pnm",argv[0]);
	scan_mode = SM_COLOR24;
	x1 = x2 = y1 = y2 = -1;
	sprintf(pipe_name,"/tmp/ps1_progress_pipe");
	for (i = 1; i < argc; i++) {
		if ( strcmp(argv[i],"-h") == 0) {
			help(argv[0]);
			return 1;	
		} else if (strcmp(argv[i], "-frame") == 0) {
			i++;
			if (i > argc) {
				printf("no frame area specified.\n");
				help(argv[0]);
				return 1;
			}
			if (sscanf(argv[i],"%d,%d,%d,%d",&x1,&x2,&y1,&y2)!= 4)
				x1 = x2 = y1 = y2 = -1;
		
		}  else if (strcmp(argv[i],"-r")==0) {
			i++;
			if (i >= argc) {
				printf("no resolution specified.\n");
				help(argv[0]);	
				return 1;
			}
			resolution = atoi(argv[i]);	
		} else if (strcmp(argv[i], "-s")==0) {
			i++;
			if (i >= argc) {
				printf("no scan mode specified.\n");
				help(argv[0]);
				return 1;
			}
			if (strcmp(argv[i],"color")==0) {
				scan_mode=SM_COLOR24;
			} else if (strcmp(argv[i],"gray")==0) {
				scan_mode=SM_GRAY8;
			} else if (strcmp(argv[i],"lineart")==0) {
				scan_mode=SM_LINEART;
			} else {
				printf("Illegal scan mode %s\n",argv[i]);
				help(argv[0]);
				return 1;
			}
		} else if (strcmp(argv[i], "-f")==0) {
			i++;
			if (i >= argc) {
				printf("Missing target filename.\n");
				help(argv[0]);
				return 1;
			}
			strcpy(path, argv[i]);
		} else if (strcmp(argv[i], "-p")==0) {
			i++;
			if (i >= argc) {
				printf("Missing progress pipe info.\n");
				help(argv[0]);
				return 1;	
			}
			strcpy(pipe_name, argv[i]);
		} else {
			printf("Unknown syntex %s\n",argv[i]);
			help(argv[0]);
			return 1;
		}
	}
	
	if (x1 == x2 && x2 == y1 && y1 == y2 && y2 == -1) {
		x1 = y1 = 0;
		x2 = (int) (8.5 * resolution);
		y2 = (int) (11.69* resolution);	
	}
	
	pipe = open(pipe_name, O_WRONLY);	
	scan_image(path,scan_mode, x1, x2, y1, y2,resolution, pipe);
	close(pipe);
	time = GetTickCount()-time;

	printf("The scan process takes %d.%d seconds\n", time/1000, time%1000);
	return 1;
}

void help(char* path)
{
	char* name;
	name = path + strlen(path);
	name--;
	while (*name != '/' && name > path)
		name--;
	if (*name == '/') name++;
	
	printf("Syntex: %s [-r resolution] [-s scan mode] [-f target filename] [-p progress info pipe name].\n", name);
	printf("           [-frame frame area].\n");
	printf("resolution can be any integer number.\n");
	printf("scan mode can be color gray lineart.\n");
	printf("target file name can be any legal filename. %s will save it as pnm format.\n",name);
	printf("scan progress info will write to progress info pipe.\n");
	printf("frame area should be in the following syntext: -frame x1,x2,y1,y2.\n");
}
