/* BearPaw1200CU panel

   Copyright (C) 2001 Mustek. 
   Maintained by Tom Wang and Alex Lin
   <tom.wang@mustek.com.tw, alex_lin@mustek.com.tw>

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston,
   MA 02111-1307, USA.
*/


#include "scan_module.h"
#include "BaseTypedef.h"
#include "SpExport.h"
#include "ScanModule.h"

#define BUCK_SIZE 65536

int scan_image(const char* path, int scanmode, int x1, int x2, int y1, int y2, int resolution, int progress_pipe)
{
	DS_IMAGEDESCRIPTOR DefaultSetting;	
	DWORD total_rows, read_rows;
	int bytes_per_row;
	unsigned char buffer[BUCK_SIZE];
	unsigned char* line_buffer;
	int lines_read;
	FILE* ImageFile;
	int i,j,index;
	BOOL result;	
	DS_BLOCKDESCRIPTOR ImageBlock;


	DefaultSetting.u32Size=sizeof(DS_IMAGEDESCRIPTOR);
	OpenDS(&DefaultSetting);
	DefaultSetting.u32Resolution = resolution;
	DefaultSetting.u32XOrigin = x1;
	DefaultSetting.u32YOrigin = y1;
	DefaultSetting.u32ImageWidth = x2 - x1;
	DefaultSetting.u32ImageHeight = y2 - y1;
	switch (scanmode) {
		case SM_COLOR24:
			DefaultSetting.smScanMode = DSSM_COLOR24;
			break;
		case SM_GRAY8:
			DefaultSetting.smScanMode = DSSM_GRAY;
			break;
		case SM_LINEART:
			DefaultSetting.smScanMode = DSSM_GRAY;
			break;
	}
	QueryImage(&DefaultSetting);
	
	read_rows = total_rows = DefaultSetting.u32ImageHeight;
	ImageFile = fopen(path,"w+b");
	switch (scanmode) {
	case SM_COLOR24: 
		sprintf((char*)buffer,"P6\n%d %d\n255\n",
				DefaultSetting.u32ImageWidth,
				DefaultSetting.u32ImageHeight);
		fwrite((void*) buffer, strlen((char*)buffer),1, ImageFile);
		ImageBlock.pBuffer = buffer;
		while (read_rows > 0) {
			lines_read = BUCK_SIZE / (DefaultSetting.u32ImageWidth*3);
			if (lines_read > read_rows) lines_read = read_rows;
			ImageBlock.u32Size = sizeof(DS_BLOCKDESCRIPTOR);
			ImageBlock.u16BufferLines=lines_read;
			ImageBlock.u16XferLines = 0;
			ImageBlock.pBuffer = buffer;
			QueryBlock(&ImageBlock);
			
			fwrite((void*) buffer, DefaultSetting.u32ImageWidth*3, ImageBlock.u16XferLines, ImageFile);
			read_rows = read_rows - ImageBlock.u16XferLines;
			unsigned char progress = (unsigned char) ((total_rows-read_rows)*100/total_rows);
			if (progress_pipe!=-1) {
				write(progress_pipe, (void*) &progress	, 1);
			}
		}
		break;
	case SM_GRAY8:		
		sprintf((char*)buffer,"P5\n%d %d\n255\n", 
						DefaultSetting.u32ImageWidth,
						DefaultSetting.u32ImageHeight);
		fwrite((void*) buffer, strlen((char*)buffer),1,ImageFile);
		ImageBlock.pBuffer = buffer;
		while(read_rows >0) {
			lines_read = BUCK_SIZE / (DefaultSetting.u32ImageWidth);
			if (lines_read > read_rows) lines_read = read_rows;
			ImageBlock.u32Size = sizeof(DS_BLOCKDESCRIPTOR);
			ImageBlock.u16BufferLines=lines_read;
			ImageBlock.u16XferLines = 0;
			ImageBlock.pBuffer = buffer;
			QueryBlock(&ImageBlock);
			fwrite((void*) buffer, DefaultSetting.u32ImageWidth, ImageBlock.u16XferLines, ImageFile);
			read_rows = read_rows - ImageBlock.u16XferLines;
			unsigned char progress = (unsigned char) ((total_rows-read_rows)*100/total_rows);
			if (progress_pipe!=-1) {
				write(progress_pipe, (void*) &progress	, 1);
			}
		}
		
		break;
	case SM_LINEART:
		sprintf((char*)buffer,"P4\n%d %d\n",
						DefaultSetting.u32ImageWidth,
						DefaultSetting.u32ImageHeight);
/*		fwrite((void*) buffer, strlen((char*)buffer),1,ImageFile);
		ImageBlock.pBuffer = buffer;
		while(read_rows >0) {
			lines_read = (BUCK_SIZE / (DefaultSetting.u32ImageWidth+7)/8);
			if (lines_read > read_rows) lines_read = read_rows;
			ImageBlock.u32Size = sizeof(DS_BLOCKDESCRIPTOR);
			ImageBlock.u16BufferLines=lines_read;
			ImageBlock.u16XferLines = 0;
			ImageBlock.pBuffer = buffer;
			QueryBlock(&ImageBlock);
			fwrite((void*) buffer, DefaultSetting.u32ImageWidth, ImageBlock.u16XferLines, ImageFile);
			read_rows = read_rows - ImageBlock.u16XferLines;
			unsigned char progress = (unsigned char) ((total_rows-read_rows)*100/total_rows);
			if (progress_pipe!=-1) {
				write(progress_pipe, (void*) &progress	, 1);
			}
		}						
*/						
		fwrite((void*) buffer, strlen((char*)buffer),1,ImageFile);
		line_buffer = (unsigned char*) malloc ((DefaultSetting.u32ImageWidth+7)/8);
		ImageBlock.pBuffer = buffer;
		while(read_rows >0) {
			lines_read = BUCK_SIZE / DefaultSetting.u32ImageWidth;
			if (lines_read > read_rows) lines_read = read_rows;
			ImageBlock.u32Size = sizeof(DS_BLOCKDESCRIPTOR);
			ImageBlock.u16BufferLines=lines_read;
			ImageBlock.u16XferLines = 0;
			ImageBlock.pBuffer = buffer;
			QueryBlock(&ImageBlock);
			for (i = 0; i < lines_read; i++) {
				for (j=0; j<DefaultSetting.u32ImageWidth; j++) {
					index = 7-(j%8);
					if (index == 7) line_buffer[j/8] = 0;
					if (buffer[i*DefaultSetting.u32ImageWidth+j]>128)
						buffer[i*DefaultSetting.u32ImageWidth+j] = 0;
					else
						buffer[i*DefaultSetting.u32ImageWidth+j] = 1;
					line_buffer[j/8] = line_buffer[j/8]|(buffer[i*DefaultSetting.u32ImageWidth+j]<<index);
				}
				fwrite((void*)line_buffer, (DefaultSetting.u32ImageWidth+7)/8,1,ImageFile);
			}
			read_rows = read_rows - lines_read;
			unsigned char progress = (unsigned char) ((total_rows-read_rows)*100/total_rows);
			if (progress_pipe!=-1) {
				write(progress_pipe, (void*) &progress	, 1);
			}			
		}
		free(line_buffer);		
		break;
	default:
		break;
	}
	
	fclose(ImageFile);
	StopQuery();
	CloseDS();
	return SM_OK;
}


