/*
 * Driver for GT-6801 USB Scanners (linux-2.4.0)
 *
 * Copyright (C) 2001 Anthony Sale
 * anthony_sale@jumble.i12.com
 *
 *
 * Based on scanner.c by David E. Nelson
 *
 * Portions may be copyright David E. Nelson, Brad Keryan and Michael Gee
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */ 

#include <linux/module.h>
#include <linux/kernel.h>
#include <linux/errno.h>
#include <asm/uaccess.h>
#include <linux/init.h>
#include <linux/slab.h>
#include <linux/delay.h>
#include <linux/ioctl.h>
#include <linux/sched.h>
#include <linux/smp_lock.h>

#define DEBUG

#include <linux/usb.h>

static __s32 vendor=-1, product=-1, read_timeout=0;

MODULE_AUTHOR("Anthony Sale, anthony_sale@jumble.i12.com, http://www.jumble.i12.com/gt6801");
MODULE_DESCRIPTION("GT-6801 USB Scanner Driver");

MODULE_PARM(vendor, "i");
MODULE_PARM_DESC(vendor, "User specified USB idVendor");

MODULE_PARM(product, "i");
MODULE_PARM_DESC(product, "User specified USB idProduct");

MODULE_PARM(read_timeout, "i");
MODULE_PARM_DESC(read_timeout, "User specified read timeout in seconds");


static struct usb_device_id gt6801_device_ids [] = {
	/* Grand Tech Semiconductors */
	{ USB_DEVICE(0x05d8, 0x4002) },	/* GT-6801 all in 1 scanner */
	{ }				/* Terminating entry */
};

MODULE_DEVICE_TABLE (usb, gt6801_device_ids);

#define IS_EP_BULK(ep)  ((ep).bmAttributes == USB_ENDPOINT_XFER_BULK ? 1 : 0)
#define IS_EP_BULK_IN(ep) (IS_EP_BULK(ep) && ((ep).bEndpointAddress & USB_ENDPOINT_DIR_MASK) == USB_DIR_IN)
#define IS_EP_BULK_OUT(ep) (IS_EP_BULK(ep) && ((ep).bEndpointAddress & USB_ENDPOINT_DIR_MASK) == USB_DIR_OUT)
#define IS_EP_INTR(ep) ((ep).bmAttributes == USB_ENDPOINT_XFER_INT ? 1 : 0)

#define USB_GT6801_MINOR(X) MINOR((X)->i_rdev) - GT6801_BASE_MNR

#ifdef DEBUG
#define GT6801_DEBUG(X) X
#else
#define GT6801_DEBUG(X)
#endif

#define IBUF_SIZE 32768
#define OBUF_SIZE 4096

#define GT6801_IOCTL 0xB4A0

/* read_scanner timeouts -- RD_NAK_TIMEOUT * RD_EXPIRE = Number of seconds */
#define RD_NAK_TIMEOUT (10*HZ)	/* Default number of X seconds to wait */
#define RD_EXPIRE 1		/* Number of attempts to wait X seconds */


#define GT6801_MAX_MNR 16		
#define GT6801_BASE_MNR 160		/* An unallocted minor no */

struct gt6801_usb_data {
	struct usb_device *gt6801_dev;
	unsigned int ifnum;	/* Interface number of the USB device */
	kdev_t gt6801_minor;	/* Scanner minor - used in disconnect() */
	unsigned char button;	/* Front panel buffer */
	char isopen;		/* Not zero if the device is open */
	char present;		/* Not zero if device is present */
	char *obuf, *ibuf;	/* transfer buffers */
	char bulk_in_ep; /* Endpoint assignments */
	wait_queue_head_t rd_wait_q; /* read timeouts */
	struct semaphore gen_lock; /* lock to prevent concurrent reads or writes */
	unsigned int rd_nak_timeout; /* Seconds to wait before read() timeout. */
};

static struct gt6801_usb_data *p_gt6801_table[GT6801_MAX_MNR] = { NULL, /* ... */};

static struct usb_driver gt6801_driver;
