/***************************************************************************
    begin                : Tue Oct 9 2001
    copyright            : (C) 2001 by Anthony Sale
    email                : anthony_sale@jumble.i12.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "boeder.h"

#include <sys/ioctl.h>
#include <asm/byteorder.h>

BOOLEAN startup(struct arguments *args, FILE *scanner)
{
 	struct ioctl_arguments ioctl_args_out;
 	struct ioctl_arguments ioctl_args_in;
 	
 	char outbuffer[0x40];
 	char inbuffer[0x40];
 	
 	int index_count = 0;
 	
 	int filenum = fileno(scanner);

	printf("Preparing scanner....\n");

	memset(firmbuffer, 0, 0x40);

	if(!loadfirmware(args, scanner))
		return FALSE;

	//get capabilities I assume
	ioctl_args_out.request = 0x01;
	ioctl_args_out.requesttype = 0x40;
 	ioctl_args_out.value = 0x2010;
//	ioctl_args_out.index = 0x7181;
 	ioctl_args_out.size = 0x40;
 	ioctl_args_out.data = outbuffer;
	memset(outbuffer, 0, 0x40);
	outbuffer[0] = 0x50;
	outbuffer[1] = 0x01;
	outbuffer[2] = 0x80;

	if(ioctl(filenum, GT6801_IOCTL, &ioctl_args_out) < 0)
 	{
 		//error occurred
 		printf("An error occurred performing outbound ioctl operation on scanner %s\n\r", args->dev);
 		return FALSE;
 	}

	ioctl_args_in.request = 0x01;
 	ioctl_args_in.requesttype = 0xc0;
 	ioctl_args_in.value = 0x2011;
 	ioctl_args_in.index = 0x7300;
 	ioctl_args_in.size = 0x40;
 	ioctl_args_in.data = inbuffer;
    memset(inbuffer, 0, 0x40);

    if(ioctl(filenum, GT6801_IOCTL, &ioctl_args_in) < 0)
 	{
 		//error occurred
 		printf("An error occurred performing inbound ioctl operation on scanner %s\n\r", args->dev);
 		return FALSE;
 	}

	//resend firmware with extra bytes on end

	//set up additional details for firm ware load
	memset(firmbuffer, 0, 0x40);
	firmbuffer[0x2f] = 0x15;
	firmbuffer[0x30] = 0x4b;
	firmbuffer[0x31] = 0xe5;
	firmbuffer[0x32] = 0x4b;
	firmbuffer[0x33] = 0x70;
	firmbuffer[0x34] = 0x07;
	firmbuffer[0x35] = 0x85;
	firmbuffer[0x36] = 0x46;
	firmbuffer[0x37] = 0x4b;
	firmbuffer[0x38] = 0xc2;
	firmbuffer[0x39] = 0x50;
	firmbuffer[0x3a] = 0xd2;
	firmbuffer[0x3b] = 0x39;
	firmbuffer[0x3c] = 0x22;
	firmbuffer[0x3e] = 0x02;
	firmbuffer[0x3f] = 0x03;
	memset(inbuffer, 0, 0x40);
	
	if(!loadfirmware(args, scanner))
		return FALSE;

 	return TRUE;
 }

 BOOLEAN loadfirmware(struct arguments *args, FILE *scanner)
 {
 	// This method loads the firm ware to the scanner
 	
 	// we expect the firware file to be in our directory (this will change) and to be called our name
 	// in this case boeder.firm
 	
 	FILE *firmware = fopen(args->path, "r");
 	
 	struct ioctl_arguments ioctl_args_out;
 	struct ioctl_arguments ioctl_args_in;
 	
 	char outbuffer[0x40];
 	char inbuffer[0x40];
 	
 	int index_count = 0;
 	
 	int filenum = fileno(scanner);
 	
 	//set up ioctl_args
 	ioctl_args_out.request = 0x01;
 	ioctl_args_out.requesttype = 0x40;
 	ioctl_args_out.value = 0x2009;
 	ioctl_args_out.size = 0x40;
 	ioctl_args_out.data = outbuffer;
 	
 	ioctl_args_in.request = 0x01;
 	ioctl_args_in.requesttype = 0xc0;
 	ioctl_args_in.value = 0x200a;
 	ioctl_args_in.index = 0x8c00;
 	ioctl_args_in.size = 0x40;
 	ioctl_args_in.data = inbuffer;
 	
 	// we read firmware file 40 bytes at a time and ioctl it to the scanner.
 	// the arguments for the scanner have been generated from a USB Snoopy log
 	while((!feof(firmware)))// && (indexes[index_count]))
 	{
 		//clear buffer before each read, file does not end on a compete read
 		memcpy(outbuffer, firmbuffer, 0x40);
 		memset(inbuffer, 0, 0x40);
 		
 		//read firware record
 		fread(outbuffer, 0x40, 1, firmware);
 		
 		//build and send ioctl request to send bytes
// 		ioctl_args_out.index = indexes[index_count];
 		if(ioctl(filenum, GT6801_IOCTL, &ioctl_args_out) < 0)
 		{
 			//error occurred
 			printf("An error occurred performing outbound ioctl operation on scanner %s\n\r", args->dev);
 			return FALSE;
 		}
 		
 		//get response from scanner
 		if(ioctl(filenum, GT6801_IOCTL, &ioctl_args_in) < 0)
 		{
 			//error occurred
 			printf("An error occurred performing inbound ioctl operation on scanner %s\n\r", args->dev);
 			return FALSE;
 		}
 		//check we got what we expected back inbuffer[0] = 0 and inbuffer[1] = 0x40
 		/*if((inbuffer[0] != 0) || (inbuffer[1] != 0x40))
 		{
 			printf("Scanner did not respond as expected\n\r");
 			printf("We expected the following values 0x00 0x40, we got %x %x\n\r", inbuffer[0], inbuffer[1]);
 			return FALSE;
 		} */

 		//increment index count
 		index_count ++;
 	}
 	
 	//send final message
 	ioctl_args_out.value = 0x2010;
// 	ioctl_args_out.index = 0xea38;
 	memset(outbuffer, 0, 0x40);
 	outbuffer[0] = 0x69;
 	outbuffer[1] = 0x01;
 	
 	if(ioctl(filenum, GT6801_IOCTL, &ioctl_args_out) < 0)
 	{
 		//error occurred
 		printf("An error occurred performing outbound ioctl operation on scanner %s\n\r", args->dev);
 		return FALSE;
 	}
 	
 	ioctl_args_in.value = 0x2011;
 	ioctl_args_in.value = 0x7300;
 	memset(inbuffer, 0, 0x40);
 	
 	if(ioctl(filenum, GT6801_IOCTL, &ioctl_args_in) < 0)
 	{
 		//error occurred
 		printf("An error occurred performing inbound ioctl operation on scanner %s\n\r", args->dev);
 		return FALSE;
 	}

 	return TRUE;
 }

 BOOLEAN configure(struct arguments *args, FILE *scanner)
 {
 	BOOLEAN result = FALSE;
 	
 	//This method runs the CIS configuration routine
	result = TRUE;
 	
  	return result;
 }

 BOOLEAN startscan(struct arguments *args, FILE *scanner)
 {
 	BOOLEAN result = FALSE;

	FILE *output;

 	int filenum = fileno(scanner);
	int count = 0;

	int overscan;
 	
 	struct scan_details scanargs;
 	
 	struct ioctl_arguments ioctl_args_out;
 	struct ioctl_arguments ioctl_args_in;

 	char outbuffer[0x40];
 	char inbuffer[0x40];
 	
 	char *scan_buffer;
	char *r_buffer;
	char *g_buffer;
	char *b_buffer;

 	__u16 ywork, xwork, wait_result;

	int bufcount = 0;
	int bufpos = 0;

 	//This method runs the scan and creates a PNM file in the current directory.

	printf("Scanning image .....\n");
	
 	//set scan arguments
 	//set defaults / unknowns
 	scanargs.instruction = __cpu_to_le16(288);
 	scanargs.mode = __cpu_to_le16(args->mode);
 	scanargs.unknown = __cpu_to_le16(0);
 	scanargs.unknown2 = __cpu_to_le16(1);
 	scanargs.unknown3 = __cpu_to_le16(600);
 	
 	//set scan details for y
 	
 	//start pos
 	ywork = args->ty * 600;
 	scanargs.ystart = __cpu_to_le16(ywork + 190);
 	
 	//height
 	ywork = (args->by - args->ty) * 600;
 	scanargs.yheight = __cpu_to_le16(ywork);
 	
 	//set scan details for x
 	//start pos
	xwork = (8.5 - args->rx) * 600;

	//xwork = args->rx * 600;
 	scanargs.xstart = __cpu_to_le16(xwork);
 	
 	//set scan width, no lines and dpi
 	//scan width
 	xwork = (args->rx - args->lx) * args->dpi;
	overscan = 64 - (xwork % 64);
	xwork += overscan;
 	scanargs.scanwidth = __cpu_to_le16(xwork);

	//width
	switch(args->dpi)
	{
		case 300:
		case 200:
		case 250:
			scanargs.xwidth = __cpu_to_le16(((xwork * 600) / args->dpi) - 4);
			break;
		case 100:
		case 150:
			scanargs.xwidth = __cpu_to_le16(((xwork * 600) / args->dpi) - 8);
			break;
		case 50:
			scanargs.xwidth = __cpu_to_le16(((xwork * 600) / args->dpi) - 12);
			break;
		default:
			scanargs.xwidth = __cpu_to_le16((xwork * 600) / args->dpi);
	}
	//no lines
	ywork = ((args->by - args->ty) * args->dpi);
	if(args->mode == COLOUR)
		ywork *= 3;
 	scanargs.scanlines = __cpu_to_le16(ywork);
 	
 	//resulution & mode
 	scanargs.resolution = args->dpi;
 	
 	//allocate buffer
 	scan_buffer = malloc(scanargs.scanwidth);
	 	
 	if(!scan_buffer)
 	{
 		printf("Unable to allocate sufficiant memory for scan buffer\n\r");
 		return FALSE;
 	}

	//try and open output file
	output = fopen("gt6801scan.pnm", "w");
	if(!output)
	{
		printf("Unable to open output file.\n\n");
		return FALSE;		
	}


	//write header for image file
	switch(scanargs.mode)
	{
		case COLOUR:
			fprintf(output, "P6\n%d\n%d\n255\n", scanargs.scanwidth - overscan, scanargs.scanlines / 3);
			r_buffer = malloc(scanargs.scanwidth);
			g_buffer = malloc(scanargs.scanwidth);
			b_buffer = malloc(scanargs.scanwidth);
			if(!r_buffer || !g_buffer || !b_buffer)
			{
				printf("Unable to allocate sufficiant memory for scan buffers.");
				return FALSE;
			}
			break;
		case GREY:
			fprintf(output, "P5\n%d\n%d\n255\n", scanargs.scanwidth - overscan, scanargs.scanlines);
			break;
	}
 	
	//set up ioctl_args
 	ioctl_args_out.request = 0x01;
 	ioctl_args_out.requesttype = 0x40;
 	ioctl_args_out.value = 0x2010;
 	ioctl_args_out.size = 0x40;
 	ioctl_args_out.data = outbuffer;
 	
 	ioctl_args_in.request = 0x01;
 	ioctl_args_in.requesttype = 0xc0;
 	ioctl_args_in.value = 0x2011;
 	ioctl_args_in.index = 0x7300;
 	ioctl_args_in.size = 0x40;
 	ioctl_args_in.data = inbuffer;
 	
 	//start scan
 	memset(outbuffer, 0, 0x40);
 	outbuffer[0] = 0x77;
 	outbuffer[1] = 0x01;
 	outbuffer[2] = 0x57;
 	outbuffer[3] = 0x1b; 	
// 	ioctl_args_out.index = 0xc052;
 	
 	if(ioctl(filenum, GT6801_IOCTL, &ioctl_args_out) < 0)
 	{
 		//error occurred
 		printf("An error occurred performing outbound ioctl operation on scanner %s\n\r", args->dev);
 		return FALSE;
 	}
 	
 	memset(inbuffer, 0, 0x40);
 	if(ioctl(filenum, GT6801_IOCTL, &ioctl_args_in) < 0)
 	{
 		//error occured
 		printf("An error ocurred performing inbound ioctl operations on scanner %s\n\r", args->dev);
 		return FALSE;
 	}

 	memset(outbuffer, 0, 0x40);
 	outbuffer[0] = 0x76;
 	outbuffer[1] = 0x01;
 	outbuffer[2] = 0x04;
 	outbuffer[4] = 0x11;
 	outbuffer[5] = 0x01;
 	outbuffer[6] = 0x04;
 	outbuffer[8] = 0xa2;
	outbuffer[9] = 0x01;
 	outbuffer[10] = 0x04;
 	outbuffer[12] = 0x4b;
	outbuffer[13] = 0x01;
// 	ioctl_args_out.index = 0x7a88;
 	
 	if(ioctl(filenum, GT6801_IOCTL, &ioctl_args_out) < 0)
 	{
 		//error occurred
 		printf("An error occurred performing outbound ioctl operation on scanner %s\n\r", args->dev);
 		return FALSE;
 	}
 	
 	memset(inbuffer, 0, 0x40);
 	if(ioctl(filenum, GT6801_IOCTL, &ioctl_args_in) < 0)
 	{
 		//error occured
 		printf("An error ocurred performing inbound ioctl operations on scanner %s\n\r", args->dev);
 		return FALSE;
 	}

 	//send scan arguments
 	memset(outbuffer, 0, 0x40);
 	memcpy(outbuffer, &scanargs, sizeof(scanargs));
// 	ioctl_args_out.index = 0x63bf;
 	
 	if(ioctl(filenum, GT6801_IOCTL, &ioctl_args_out) < 0)
 	{
 		//error occurred
 		printf("An error occurred performing outbound ioctl operation on scanner %s\n\r", args->dev);
 		return FALSE;
 	}
 	
 	memset(inbuffer, 0, 0x40);
 	if(ioctl(filenum, GT6801_IOCTL, &ioctl_args_in) < 0)
 	{
 		//error occured
 		printf("An error ocurred performing inbound ioctl operations on scanner %s\n\r", args->dev);
 		return FALSE;
 	}

 	memset(outbuffer, 0, 0x40);
 	outbuffer[0] = 0x22;
 	outbuffer[1] = 0x01;
 	outbuffer[2] = 0x24;
 	outbuffer[3] = 0x06;
 	outbuffer[4] = 0x1e;
 	outbuffer[5] = 0x14;
 	outbuffer[6] = 0x1e;
 	outbuffer[7] = 0x14;
// 	ioctl_args_out.index = 0x7989;
 	
 	if(ioctl(filenum, GT6801_IOCTL, &ioctl_args_out) < 0)
 	{
 		//error occurred
 		printf("An error occurred performing outbound ioctl operation on scanner %s\n\r", args->dev);
 		return FALSE;
 	}
 	
 	memset(inbuffer, 0, 0x40);
 	if(ioctl(filenum, GT6801_IOCTL, &ioctl_args_in) < 0)
 	{
 		//error occured
 		printf("An error ocurred performing inbound ioctl operations on scanner %s\n\r", args->dev);
 		return FALSE;
 	}
 	
	memset(outbuffer, 0, 0x40);
 	outbuffer[0] = 0x43;
 	outbuffer[1] = 0x01;
 	outbuffer[2] = 0x24;
 	outbuffer[3] = 0x06;
// 	ioctl_args_out.index = 0x22f0;
 	
 	if(ioctl(filenum, GT6801_IOCTL, &ioctl_args_out) < 0)
 	{
 		//error occurred
 		printf("An error occurred performing outbound ioctl operation on scanner %s\n\r", args->dev);
 		return FALSE;
 	}
 	
 	memset(inbuffer, 0, 0x40);
 	if(ioctl(filenum, GT6801_IOCTL, &ioctl_args_in) < 0)
 	{
 		//error occured
 		printf("An error ocurred performing inbound ioctl operations on scanner %s\n\r", args->dev);
 		return FALSE;
 	}
 	
 	//wait for scanner
 	do{
 		memset(outbuffer, 0, 0x40);
	 	outbuffer[0] = 0x35;
 		outbuffer[1] = 0x01;
 	 	
	 	if(ioctl(filenum, GT6801_IOCTL, &ioctl_args_out) < 0)
 		{
 			//error occurred
	 		printf("An error occurred performing outbound ioctl operation on scanner %s\n\r", args->dev);
 			return FALSE;
	 	}
 	
 		memset(inbuffer, 0, 0x40);
	 	if(ioctl(filenum, GT6801_IOCTL, &ioctl_args_in) < 0)
 		{
 			//error occured
	 		printf("An error ocurred performing inbound ioctl operations on scanner %s\n\r", args->dev);
 			return FALSE;
	 	}

		memcpy(&wait_result, inbuffer, sizeof(wait_result));
	}while(wait_result != 0x0000);
	
	//read lines
	bufcount = 0;

	for(count = 0; count < scanargs.scanlines; count ++)
	{

		if(!fread(scan_buffer, scanargs.scanwidth, 1, scanner))
		{	
			printf("An error occurred performing read operation on scanner %s\n\r", args->dev);
			return FALSE;
		}

		if(scanargs.mode == COLOUR)
		{
			bufcount ++;
			switch(bufcount)
			{
				case 1:
						memcpy(r_buffer, scan_buffer, scanargs.scanwidth);
						break;
				case 2:
						memcpy(g_buffer, scan_buffer, scanargs.scanwidth);
						break;
				case 3:
						memcpy(b_buffer, scan_buffer, scanargs.scanwidth);
						break;
			}
			if(bufcount >= 3)
			{
				for(bufpos = 0; bufpos < scanargs.scanwidth - overscan; bufpos ++)
				{
					fwrite(&b_buffer[(scanargs.scanwidth - overscan) - bufpos], 1, 1, output);
					fwrite(&g_buffer[(scanargs.scanwidth - overscan) - bufpos], 1, 1, output);
					fwrite(&r_buffer[(scanargs.scanwidth - overscan) - bufpos], 1, 1, output);
				}
				bufcount = 0;
			}
		}
		else
		{
			for(bufpos = 0; bufpos < scanargs.scanwidth - overscan; bufpos ++)
				fwrite(&scan_buffer[(scanargs.scanwidth - overscan) - bufpos], 1, 1, output);
		}
	}
	//send end of scan command
	memset(outbuffer, 0, 0x40);
	outbuffer[0] = 0x42;
	outbuffer[1] = 0x01;
//	ioctl_args_out.index = 0x21f1;

	if(ioctl(filenum, GT6801_IOCTL, &ioctl_args_out) < 0)
 	{
 		//error occurred
 		printf("An error occurred performing outbound ioctl operation on scanner %s\n\r", args->dev);
 		return FALSE;
 	}
 	
 	memset(inbuffer, 0, 0x40);
 	if(ioctl(filenum, GT6801_IOCTL, &ioctl_args_in) < 0)
 	{
 		//error occured
 		printf("An error ocurred performing inbound ioctl operations on scanner %s\n\r", args->dev);
 		return FALSE;
 	}

	

	fflush(output);
	fclose(output);

	memset(outbuffer, 0, 0x40);
	outbuffer[0] = 0x78;
	outbuffer[1] = 0x01;
	outbuffer[2] = 0xef;
	outbuffer[3] = 0xd8;
//	ioctl_args_out.index = 0xf909;
	
	if(ioctl(filenum, GT6801_IOCTL, &ioctl_args_out) < 0)
 	{
 		//error occurred
 		printf("An error occurred performing outbound ioctl operation on scanner %s\n\r", args->dev);
 		return FALSE;
 	}
 	
 	memset(inbuffer, 0, 0x40);
 	if(ioctl(filenum, GT6801_IOCTL, &ioctl_args_in) < 0)
 	{
 		//error occured
 		printf("An error ocurred performing inbound ioctl operations on scanner %s\n\r", args->dev);
 		return FALSE;
 	}

	memset(outbuffer, 0, 0x40);
	outbuffer[0] = 0x34;
	outbuffer[1] = 0x01;

	if(ioctl(filenum, GT6801_IOCTL, &ioctl_args_out) < 0)
 	{
 		//error occurred
 		printf("An error occurred performing outbound ioctl operation on scanner %s\n\r", args->dev);
 		return FALSE;
 	}
 	
 	memset(inbuffer, 0, 0x40);
 	if(ioctl(filenum, GT6801_IOCTL, &ioctl_args_in) < 0)
 	{
 		//error occured
 		printf("An error ocurred performing inbound ioctl operations on scanner %s\n\r", args->dev);
 		return FALSE;
 	}

  	return result;
 }

void logrecord(char outbuffer[0x40], char inbuffer[0x40])
{
	int loop;

	FILE *logfile = fopen("/tmp/gt6801scan.log", "a");

	dbgcount ++;

	if(!logfile)
	{
		printf("can't access logfile");
		return;
	}

	fprintf(logfile, "Rec %d\t", dbgcount);
	
	for(loop = 0; loop < 0x40; loop += 2)
	{
		fprintf(logfile, "%02hhx%02hhx ", outbuffer[loop], outbuffer[loop + 1]);
	}

	fprintf(logfile, "\t");
	for(loop = 0; loop < 0x40; loop += 2)
	{
		fprintf(logfile, "%02hhx%02hhx ", inbuffer[loop], inbuffer[loop + 1]);
	}

	fprintf(logfile, "\n");

	fclose(logfile);
}