/***************************************************************************
                          main.c  -  description
                             -------------------
    begin                : Thu Oct  4 23:28:11 BST 2001
    copyright            : (C) 2001 by Anthony Sale
    email                : anthony_sale@jumble.i12.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <unistd.h>

#include "gt6801.h"
#include "boeder.h"

/*
Displays arguments for program
*/
void promptargs(int error)
{
	if(error)
		printf("Incorrect parameters please call as follows\n\r");
	printf("gt6801scan dpi=xxx lx=xx.x ty=xx.x rx=xx.x by=xx.x path=/path/to/firm mode=c|g|l [dev=pppppppppp]\n\r");		
}

/*
Parses command line into arguments structure.
Expects the following
dpi=xxx   50 to 600 in increments of 50
lx=xx.x		0 to 8.5 left side of scan
ty=xx.x   0 to 11.7 top of scan
by=xx.x		0 to 11.7 and > ty bottom of scan
path=\path\to\firm	Path to file containing firmware
mode=c|g|l				Scan mode c = colour, g = grey and l = line art
Optional entries
dev=pppppppp path to device file for scanner (Default /dev/usbgt6801-0)


return TRUE if parse sucsessfull else FALSE
*/
BOOLEAN parseargs(int argc, char *argv[], struct arguments *args)
{
	//check we atleast have the expected number of arguments
	char *arg;

	int dpi, mode;
	float lx, rx, ty, by;

	char *dev, *path;
	
	struct stat stats;
	
	if ((argc < 8) || (argc > 9))
	{
		promptargs(TRUE);
		return FALSE;
	}
	else
	{
		//crack paraemters

		//crack dpi
		arg = argv[1];
		if((strlen(arg) <= 4) || (strncmp(arg, "dpi=", 4) != 0))
		{
			promptargs(TRUE);
			return FALSE;
		}
		else
		{
			dpi = atoi(&arg[4]);
			
			//check dpi in valid range
			if((dpi < 50) || (dpi > 600) || ((dpi % 50) != 0))
			{
				printf("dpi must be between 50 and 600 in steps of 50\n\r");
				return FALSE;
			}
		}

		//crack lx
		arg = argv[2];
		if((strlen(arg) <= 3) || (strncmp(arg, "lx=", 3) != 0))
		{
			promptargs(TRUE);
			return FALSE;
		}
		else
		{
			lx = atof(&arg[3]);

			//check lx range
			if((lx < 0) || ( lx > 8.5))
			{
				printf("lx must be between 0 and 8.5\n\r");
				return FALSE;
			}
		}

		//crack ty
		arg = argv[3];
		if((strlen(arg) <= 3) || (strncmp(arg, "ty=", 3) != 0))
		{
			promptargs(TRUE);
			return FALSE;
		}
		else
		{
			ty = atof(&arg[3]);
	
			//check ty range
			if((ty < 0) || (ty > 11.7))
			{
				printf("ty must be between 0 and 11.7\n\r");
              return FALSE;
			}
		}
		
		//crack rx
		arg = argv[4];
       if((strlen(arg) <= 3) || (strncmp(arg, "rx=", 3) != 0))
       {
       	promptargs(TRUE);
       	return FALSE;
       }
       else
       {
       	rx = atof(&arg[3]);
       	
       	//check rx range
       	if((rx <0) || (rx > 8.5) || (rx < lx))
       	{
       		printf("rx must be between 0 and 8.5 and >= lx\n\r");
       		return FALSE;
       	}
       }

       //crack by
       arg = argv[5];
       if((strlen(arg) <= 3) || (strncmp(arg, "by=", 3) != 0))
       {
       	promptargs(TRUE);
       	return FALSE;
       }
       else
       {
       	by = atof(&arg[3]);
       	
       	//heck by range
       	if((by < 0) || (by > 11.7) || (by < ty))
       	{
       		printf("by must be between 0 ands 11.7 and >= ty\n\r");
       		return FALSE;
       	}
       }

       //crack path to firmware
       arg = argv[6];
       if((strlen(arg) <= 5) || (strncmp(arg, "path=", 5) != 0))
       {
       	promptargs(TRUE);
       	return FALSE;
       }
       else
       {
       	path = &arg[5];
       	
       	//check we have access to the firmware file
       	if(stat(path, &stats) < 0)
       	{
       		printf("Firmware file %s is not accesable or does not exist\n\r", path);
       		return FALSE;
       	}
       }

		//crack scan mode
		arg = argv[7];
		if((strlen(arg) <= 5) || (strncmp(arg, "mode=", 5) != 0))
		{
			promptargs(TRUE);
			return FALSE;
		}
		else
		{
			if(arg[5] != 'c' && arg[5] != 'g' && arg[5] != 'l')
			{
				printf("Invalid scan mode %c\n", arg[5]);
				promptargs(TRUE);
				return FALSE;
			}
			else
			{
				switch(arg[5])
				{
					case 'c':
						mode = COLOUR;
						break;
					case 'g':
						mode = GREY;
						break;
					case 'l':
						printf("Line art not implemented yet\n\n");
						return FALSE;
				}
			}
		}
		

       //crack device file
       if(argc >= 9)
       {
       	arg = argv[8];
       	if((strlen(arg) <= 4) || (strncmp(arg, "dev=", 4) != 0))
       	{
       		promptargs(TRUE);
       		return FALSE;
       	}
       	else
       		dev = &arg[4];
       		
       	//check device file exists
       	if(stat(dev, &stats) < 0)
       	{
       		printf("Device file %s is not accesable or does not exist\n\r", dev);
       		return FALSE;
       	}
       	
       }
       else
       	dev = DEFAULT_DEVICE;

       //if we got here every thing is ok build args structure and return TRUE
       args->dpi = dpi;
       args->lx = lx;
       args->ty = ty;
       args->rx = rx;
       args->by = by;
       args->path = path;
       args->dev = dev;
		args->mode = mode;

       return TRUE;
	}
	
	//if you get here something is seriously wrong
	return FALSE;
}

int main(int argc, char *argv[])
{
    struct arguments args;

	dbgcount = 0;

    if((argc != 2) && (argc < 7))
    {
    	promptargs(TRUE);
    	return -1;
    }

    if(strcmp(argv[1], "--help") == 0)
    {
    	promptargs(FALSE);
    	return 0;
    }

    if(parseargs(argc, argv, &args) == TRUE)
    {
    	FILE *scanner = fopen(args.dev, "r");
    	
    	if(!scanner)
    	{
    		printf("Unable to open scanner device %s\n\r", args.dev);
    		return -1;
    	}

    	if(startup(&args, scanner) == TRUE)
    		if(configure(&args, scanner) == TRUE)
    			startscan(&args, scanner);
    }
    else
    {
    	return 1;
    }

    return 1;
}
